/*
	$Id: AnnotationTypeTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Type;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>SampleAnnotationType</dd>
<dt><b>Unused columns</b></dt>
<dd>addedDate</dd>
<dt><b>Notes</b></dt>
<dd> Enumerated annotations are all of {@link net.sf.basedb.core.Type#STRING}.
</dd>
</dl>

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class AnnotationTypeTransfer extends CommonItemTransfer
{
	/**
	 	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.AnnotationTypeTransfer");
	
	/**
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	 */
	AnnotationTypeTransfer(UserTransfer userT, GroupTransfer groupT)
	{
		super(userT, groupT);
		log.debug("Checking that necessary transfers have been done before proceeding with AnnotationTypes");
		if (!userT.isDone() || !groupT.isDone())
		{
			if (!userT.isDone()) log.debug("Dependency: Users haven't been transfered");
			if (!groupT.isDone()) log.debug("Dependency: Groups haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring AnnotationTypes...");
		int count = getInt("countSampleAnnotationTypes");
		initIdMap(count);
		startProgress(count, "AnnotationTypes");
		run(prepareStatementFromFile("selectSampleAnnotationTypes"));
		done();
		log.info("Transferring AnnotationTypes...DONE");
	}

	/**
	Creates an {@link AnnotationType} from the current position in
	<code>ResultSet</code>.
	
	@param b2c {@link DbControl} used to transfer this item
	@param rs <code>ResultSet</code> containing the data for this item.
	@return <code>AnnotationType</code>
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	AnnotationType createItem(DbControl b2c, ResultSet rs)
			throws BaseException, SQLException
	{
		AnnotationType annotationType = null;
		switch (rs.getString(9).toCharArray()[0])
		{
			case 'i':
			{
				log.debug("The annotation type is an INT");
				annotationType = AnnotationType.getNew(b2c, Type.INT);
				int width = rs.getInt(10);
				if (width > 0) annotationType.setWidth(width);
				break;
			}
			case 'f':
			{
				log.debug("The annotation type is a FLOAT");
				annotationType = AnnotationType.getNew(b2c, Type.FLOAT);
				int width = rs.getInt(10);
				if (width > 0) annotationType.setWidth(width);
				break;
			}
			case 't':
			{
				log.debug("The annotation type is TEXT");
				annotationType = AnnotationType.getNew(b2c, Type.TEXT);
				int width = rs.getInt(10);
				if (width > 0) annotationType.setWidth(width);
				break;
			}
			case 'e':
			{
				log.debug("The annotation type is a STRING");
				annotationType = AnnotationType.getNew(b2c, Type.STRING);
				annotationType.setEnumeration(true);
				annotationType.setValues(Arrays.asList(rs.getString(10).split(
						"\\|")));
				annotationType.setDisplayAsList(true);
				break;
			}
			default:
				log.error("The annotationtype is unknown");
				stop();
		}
		setCommonItemData(b2c, (CommonItem) annotationType, rs);
		String defaultValue = rs.getString(11);
		if (defaultValue != null && defaultValue.length() > AnnotationType.MAX_DEFAULT_VALUE_LENGTH)
		{
			defaultValue = defaultValue.substring(0, AnnotationType.MAX_DEFAULT_VALUE_LENGTH);
		}
		annotationType.setDefaultValue(defaultValue);
		annotationType.enableForItem(Item.SAMPLE);
		if(log.isDebugEnabled())
		{
			log.debug("Transfered AnnotationType:" + annotationType.getName() + "\tBASE1 id(" + rs.getInt(1) + ")\tBASE2 id(" + annotationType.getId() + ")");
			log.debug("--------------------------------------------------------------------------");
		}
		return annotationType;
	}

	/**
	Returns a AnnotationType from a base1 annotationtype id.
	
	@param dbc DbControl used to get the annotationType
	@param base1AnnotationTypeId Base1 annotationtype id
	@throws BaseException If accessing the data fails.
	 */
	public AnnotationType getAnnotationTypeById(DbControl dbc,
			int base1AnnotationTypeId) throws BaseException
	{
		Integer base2Id = getBase2Id(base1AnnotationTypeId);
		return base2Id == null ? null : AnnotationType.getById(dbc, base2Id);
	}
}
