/*
	$Id: ArrayBatchTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Jari Hakkinen, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.ArrayBatch;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.Type;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>ArrayBatch</dd>
<dt><b>Unused columns</b></dt>
<dd>addedDate, printStartDate, printEndDate</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li>ArrayType is named ArrayDesign in BASE 2.</li>
<li>Hardware used for each array batch is not available in base 1.</li>
<li>Notes are merged with description</li>
<li>surfaceType is an annotation</li>
</ul>
</dd>
</dl>

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class ArrayBatchTransfer extends CommonItemTransfer
{
	private ProtocolTransfer protocolT = null;
	private ArrayTypeTransfer arrayTypeT = null;
	private AnnotationType surfaceType = null;
	
	/**
	 	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.ArrayBatchTransfer");

	/**
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	@param protocolT A finished {@link ProtocolTransfer}
	@param arrayTypeT A finished {@link ArrayTypeTransfer}
	 */
	ArrayBatchTransfer(UserTransfer userT, GroupTransfer groupT,
			ProtocolTransfer protocolT, ArrayTypeTransfer arrayTypeT)
	{
		super(userT, groupT);
		if (protocolT.isDone() && arrayTypeT.isDone())
		{
			this.protocolT = protocolT;
			this.arrayTypeT = arrayTypeT;
		}
		else
		{
			if (!protocolT.isDone()) log.debug("Dependency: Protocols haven't been transfered");
			if (!arrayTypeT.isDone()) log.debug("Dependency: ArrayTypes haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring ArrayBatches...");
		// Create a String type annotation of the surfaceType
		DbControl dc = newDbControl();
		AnnotationType annotationType = AnnotationType.getNew(dc, Type.STRING);
		annotationType.setWidth(255);
		annotationType.setName("Surface Type");
		annotationType.setDescription("Created during migration");
		annotationType.enableForItem(Item.ARRAYBATCH);
		dc.saveItem(annotationType);
		surfaceType = annotationType;
		commit(dc);
		// Start ArrayBatch transfer
		int count = getInt("countArrayBatches");
		initIdMap(count);
		startProgress(count, "ArrayBatches");
		runUnBatched(prepareStatementFromFile("selectArrayBatches"), 0, getBatchSize());
		done();
		log.info("Transferring ArrayBatches...DONE");
	}

	/**
	Creates an {@link ArrayBatch} from the current position in the
	<code>ResultSet</code>.
	
	@param rs <code>ResultSet</code> containing data for this item.
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected void createItem(ResultSet rs) throws BaseException, SQLException
	{
		Integer arrayTypeId = arrayTypeT.getBase2Id(rs.getInt(9));
		Integer protocolId = protocolT.getBase2Id(rs.getInt(13));
		if (arrayTypeId != null)
		{
			DbControl dc = newDbControl();
			ArrayDesign arrayType = ArrayDesign.getById(dc, arrayTypeId);
			Protocol protocol = protocolId == null ? null : Protocol.getById(dc, protocolId);
			ArrayBatch arrayBatch = ArrayBatch.getNew(dc, arrayType);
			setCommonItemData(dc, (CommonItem) arrayBatch, rs);
			arrayBatch.setDescription(arrayBatch.getDescription() + "\nNOTES\n"
					+ rs.getString(14));
			arrayBatch.setProtocol(protocol);
			dc.saveItem(arrayBatch);
			// Add annotation
			AnnotationSet set = arrayBatch.getAnnotationSet();
			Annotation annotation = set.getAnnotation(surfaceType);
			annotation.setValue(rs.getString(12));
			dc.saveItem(annotation);
			commit(dc);
			// Map id's this should be moved into the runUnBatched() method
			if (log.isDebugEnabled())
			{
				log.debug("Transfered ArrayBatch: " + arrayBatch.getName() + "\tBASE1 id(" + rs.getInt(1) + ")\tBASE2 id(" + arrayBatch.getId() + ")");
			}
			mapId(rs.getInt(1), arrayBatch.getId());
		}
		else
		{
			log.debug("ArrayDesign id=" + arrayTypeId + ", protocolId = " + protocolId);
		}
	}

	/**
	@param dbc DbControl used to get the ArrayBatch
	@param base1BatchId
	@return ArrayBatch or null if no array batch is found matching the
	        provided base1BatchId
	 */
	public ArrayBatch getArrayBatchById(DbControl dbc, int base1BatchId)
	{
		try
		{
			Integer base2Id = getBase2Id(base1BatchId);
			return base2Id == null ? null : ArrayBatch.getById(dbc, base2Id);
		}
		catch (BaseException e)
		{
			log.error("FAILED getting ArrayBatch with id: " + base1BatchId + " from BASE1", e);
			stop();
		}
		return null;
	}
}
