/*
	$Id: ImageFileTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005, 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Image;
import net.sf.basedb.core.Scan;
import net.sf.basedb.core.User;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>Image</dd>
<dt><b>Unused columns</b></dt>
<dd>addedDate</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li></li>
</ul>
</dd>
</dl>

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class ImageFileTransfer extends FileTransfer
{
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.ImageFileTransfer");
	
	private ScanTransfer scanT = null;
	private UploadFileTransfer uploadFT = null;

	/**
	Constructs a new ImageTransfer.
	
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	@param uploadFT A finished {@link UploadFileTransfer}
	@param scanT A finished {@link ScanTransfer}
	@param path <code>String</code> path to where Image files are stored on
	       the filesystem for base 1.
	 */
	ImageFileTransfer(UserTransfer userT, GroupTransfer groupT,
			UploadFileTransfer uploadFT, ScanTransfer scanT, String path)
	{
		super(userT, groupT);
		log.debug("Checking that necessary transfers have been done before proceeding with ImageFile");
		if (scanT != null && scanT.isDone() && uploadFT != null
				&& uploadFT.isDone())
		{
			this.scanT = scanT;
			this.uploadFT = uploadFT;
		}
		else
		{
			if (!scanT.isDone()) log.debug("Dependency: Scans haven't been transfered");
			if (!uploadFT.isDone()) log.debug("Dependency: Uploaded files haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring ImageFiles...");
		int count = getInt("countImages");
		initIdMap(count);
		// Initiate imageChannels
		startProgress(count, "ImageFiles");
		runUnBatched(prepareStatementFromFile("selectImages"), 0, getBatchSize());
		done();
		log.info("Transferring ImageFiles...DONE");
	}

	/**
	Creates a {@link Image} from the current position in
	<code>ResultSet</code>.
	
	@param rs <code>ResultSet</code> containing the data for this item.
	@throws BaseException If a new Image cannot be created.
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected void createItem(ResultSet rs) throws BaseException, SQLException
	{
		DbControl dc = newDbControl();
		User owner = getOwner(dc, rs.getInt(2));
		Scan scan = scanT.getScanById(dc, rs.getInt(9));
		if (scan != null && owner != null)
		{
			File file = uploadFT.getImageFileById(dc, rs.getInt(1));
			// Create new image object
			Image image = createImage(rs, scan, file);
			saveItem(dc, image, rs.getInt(1));
			if (log.isDebugEnabled())
			{
				log.debug("Transfered Image: " + image.getName() + "\tBASE1 id(" + rs.getInt(1) + ")");
				log.debug("--------------------------------------------------------------------------");
			}
		}
		commit(dc);
	}

	/**
	Returns an Image. All files should already be transfered using
	{@link UploadFileTransfer}.
	
	@param rs ResultSet containing the data for this image
	@param scan Scan used to create this image
	@param file File connected to this image
	@return A new image from the provided scan
	 */
	private Image createImage(ResultSet rs, Scan scan, File file)
	{
		Image image = scan.newImage();
		try
		{
			image.setName(rs.getString(6));
			image.setDescription(rs.getString(7) + "\nChannel: "
					+ rs.getInt(10));
			image.setRemoved(rs.getBoolean(8));
			if (file != null)
			{
				image.setFile(file);
				log.debug("Image connected with file: " + file.getName());
			}
		}
		catch (SQLException e)
		{
			log.error("FAILED executing SQL statement", e);
			stop();
		}
		return image;
	}


	public Image getImageById(DbControl dbc, int base1ImageId)
	{
		try
		{
			Integer id = getBase2Id(base1ImageId);
			return id == null ? null : Image.getById(dbc, id);
		}
		catch (BaseException e)
		{
			log.warn("Couldn't get Image with id: " + base1ImageId + " from BASE1");
			if (log.isDebugEnabled())log.debug("\t" + e);
		}
		return null;
	}
}
