/*
	$Id: SampleTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.MeasuredBioMaterial;
import net.sf.basedb.core.Sample;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

/**
This class provides methods for transferring Sample data from base 1 to BASE
2. The original quantity of a sample is unknown in base 1. A minimum quantity
could be calculated from the sum over extract quantities but. sampleDate is
mapped to a BioMaterialEvent indicating when the sample was taken.

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class SampleTransfer extends MeasuredBioMaterialTransfer
{
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.SampleTransfer");
	
	/**
	base 1 sample id's are mapped to totalQuantity amount
	 */
	private Map<Integer, Float> sampleQuantities = null;
	private SampleTissueTransfer sampleTissueT = null;

	/**
	Constructs a new SampleTransfer.
	
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	@param protocolFT A finished {@link ProtocolTransfer}
	@param sampleTissueT A finished {@link SampleTissueTransfer}
	 */
	SampleTransfer(UserTransfer userT, GroupTransfer groupT,
			ProtocolTransfer protocolFT, SampleTissueTransfer sampleTissueT)
	{		
		super(userT, groupT, protocolFT);
		log.debug("Checking that necessary transfers have been done before proceeding with Samples");
		if (sampleTissueT.isDone())
		{
			this.sampleTissueT = sampleTissueT;
		}
		else
		{
			log.debug("Dependency: SampleTissues haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring Samples...");
		int count = getInt("countSamples");
		initIdMap(count);
		sampleQuantities = new HashMap<Integer, Float>(count);
		PreparedStatement ps = prepareStatementFromFile("selectSampleQuantities");
		try
		{
			ResultSet rs = ps.executeQuery();
			while (rs.next())
			{
				sampleQuantities.put(rs.getInt(1), rs.getFloat(2));
			}
		}
		catch (Exception e)
		{
			log.error("FAILED when getting sample quantities", e);
			stop();
		}
		startProgress(count, "Samples");
		run(prepareStatementFromFile("selectSamples"), 0, getBatchSize());
		done();
		log.info("Transferring Samples...DONE");
	}

	/**
	Creates a {@link Sample} from the current position in
	<code>ResultSet</code>.
	
	@param b2c {@link DbControl} used to transfer this item
	@param rs <code>ResultSet</code> containing the data for this item.
	@return <code>Sample</code>
	@throws BaseException If SampleTissue is missing.
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected Sample createItem(DbControl b2c, ResultSet rs)
			throws BaseException, SQLException
	{
		if (log.isDebugEnabled()) log.debug("Transferring Sample:" + rs.getString(6) + "BASE1 id(" + rs.getInt(1) + ")");
		Sample sample = Sample.getNew(b2c);
		setMeasuredBioMaterialData(b2c, (MeasuredBioMaterial) sample, rs);
		// Base 1 does not have a original quantity for samples so we set it to
		// the calculated value from
		// Extract quantities
		sample.setOriginalQuantity(sampleQuantities.get(rs.getInt(1)));
		String sampleTissue = sampleTissueT.getTissue(rs.getInt(12));
		String description = sample.getDescription();
		String sampleOriginString = "Sample origin: " + sampleTissue;
		if (description == null)
		{
			description = sampleOriginString;
		}
		else
		{
			description += "\n" + sampleOriginString;
		}
		sample.setDescription(description);
		if (log.isDebugEnabled())
		{
			 
			if (sampleTissue != null) log.debug("SampleTissue " + sampleTissue + " is included in the sample's description");
			log.debug("Transfered Sample: " + sample.getName() + "\tBASE1 id(" + rs.getInt(1) + ")");
			log.debug("--------------------------------------------------------------------------");
		}
		return sample;
	}

	/**
	Returns a Sample from a base1 sample id.
	
	@param dbc DbControl used to get the sample
	@param base1SampleId Base1 sample id
	 */
	public Sample getSampleById(DbControl dbc, int base1SampleId)
	{
		try
		{
			Integer id = getBase2Id(base1SampleId);
			return id == null ? null : Sample.getById(dbc, id);
		}
		catch (BaseException e)
		{
			log.warn("Couldn't get sample in BASE1 by id: " + base1SampleId);
			if (log.isDebugEnabled()) log.debug("\t" + e);
		}
		return null;
	}
}
