/*
	$Id: SharedItemTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Jari Hakkinen, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Group;
import net.sf.basedb.core.GroupPermissions;
import net.sf.basedb.core.ItemKey;
import net.sf.basedb.core.OwnedItem;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.SharedItem;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.User;
import java.sql.ResultSet;
import java.util.EnumSet;

/**
This class provides methods to change owner and permissions for SharedItems.

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

abstract class SharedItemTransfer extends Transfer
{
	/**
	Array of base 1 to BASE 2 user id's
	 */
	protected UserTransfer userT = null;
	/**
	Array of base 1 to BASE 2 group id's
	 */
	protected GroupTransfer groupT = null;
	/**
	Column index in the ResultSet
	 */
	protected final int ID = 1;
	/**
	Column index in the ResultSet
	 */
	protected final int OWNER = 2;
	/**
	Column index in the ResultSet
	 */
	protected final int GID = 3;
	/**
	Column index in the ResultSet
	 */
	protected final int GROUP_ACCESS = 4;
	/**
	Column index in the ResultSet
	 */
	protected final int WORLD_ACCESS = 5;
	
	/**
	 	Logger
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.SharedItemTransfer");

	SharedItemTransfer()
	{}
	
	SharedItemTransfer(UserTransfer userT, GroupTransfer groupT)
	{
		log.debug("Checking that necessary transfers have been done before proceeding with SharedItems");
		if (userT.isDone() && groupT.isDone())
		{
			this.userT = userT;
			this.groupT = groupT;
		}
		else
		{
			if (!userT.isDone()) log.debug("Dependency: Users haven't been transfered");
			if (!groupT.isDone()) log.debug("Dependency: Groups haven't been transfered");
			dependencyFailure();
		}		
	}

	/**
	Changes group and world permissions for this item.
	
	@param dc <code>DbControl</code>
	@param item <code>SharedItem</code> for which permissions should be set
	@param groupId <code>int</code> Base 1 group id
	@param groupAccess <code>int</code> Base 1 groupAccess value
	@param worldAccess <code>int</code> Base 1 worldAccess value
	 */
	protected void chmod(DbControl dc, SharedItem item, int groupId,
			int groupAccess, int worldAccess)
	{
		Group group = groupT.getGroupById(dc, groupId);
		try
		{
			GroupPermissions groupPermissions = new GroupPermissions();
			if (group != null && groupAccess > 0)
			{
				dc.reattachItem(group);
				switch (groupAccess)
				{
					case 1:
						groupPermissions.setPermissions(group, 
							EnumSet.of(Permission.READ, Permission.USE));
						break;
					case 2:
						groupPermissions.setPermissions(group, 
							EnumSet.of(Permission.WRITE));
						break;
				}
			}
			if (worldAccess == 1)
			{
				// Set world permissions
				Group everyone = Group.getById(dc, SystemItems.getId(Group.EVERYONE));
				groupPermissions.setPermissions(everyone, EnumSet.of(Permission.READ, Permission.USE));
			}
			if (groupPermissions.size() > 0)
			{
				ItemKey ik = ItemKey.getNewOrExisting(dc, null, groupPermissions);
				item.setItemKey(ik);
			}
		}
		catch (Exception e)
		{
			log.error("FAILED changing permission on sharedItem: " + item, e);
			stop();
		}
	}

	/**
	Changes owner of the Ownable Item using the BASE 2 userId.
	
	@param dc DbControl used to retrieve the corresponding User
	@param item OwnedItem for which ownership should be changed
	@param base1UserId Id of a Base 1 user
	 */
	protected void chown(DbControl dc, OwnedItem item, int base1UserId)
	{
		User user = userT.getUserById(dc, base1UserId);
		if (user != null)
		{
			try
			{
				item.setOwner(user);
			}
			catch (Exception e)
			{
				log.error("FAILED setting owner in BASE2", e);
				stop();
			}
		}
	}

	/**
	Sets SharedItem data values.
	
	@param dc DbControl used to retrieve the item
	@param item <code>CommonItem</code> for which to set data values
	@param rs <code>ResultSet</code> used to get values from
	 */
	protected void setSharedItemData(DbControl dc, SharedItem item, ResultSet rs)
	{
		log.debug("setting SharedItemData");
		try
		{
			chown(dc, (OwnedItem) item, rs.getInt(OWNER));
			chmod(
					dc, item, rs.getInt(GID), rs.getInt(GROUP_ACCESS),
					rs.getInt(WORLD_ACCESS));
		}
		catch (Exception e)
		{
			log.error("FAILED setting SharedItemData on item: " + item, e);
			stop();
		}
	}

	/**
	Returns a user from database.
	 */
	protected User getOwner(DbControl dc, int userId)
	{
		try
		{
			User user = userT.getUserById(dc, userId);
			return user;
		}
		catch (Exception e)
		{
			log.error("FAILED getting owner by userId: " + userId, e);
			stop();
		}
		return null;
	}
}
