/*
	$Id: UploadFileTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.User;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

/**
All files are migrated in one go. This is done to ease the transfer of file
related items down the road. In BASE 2 files are unique in regards to their
md5sum. No files having equal md5sum will be migrated.

@author Gregory Vincic
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class UploadFileTransfer extends FileTransfer
{
	/**
	Path to base 1 userfiles e.g. /usr/local/base/data
	 */
	private String userfiles;
	/**
	Current path to files e.g userfiles + "/raw data"
	 */
	private String path;
	/**
	Stores unique md5 -> fileId
	 */
	private HashMap<String, Integer> md5ToIdMap = null;
	/**
	Map of protocol related base 1 -> BASE 2 files
	 */
	private HashMap<Integer, Integer> protocolFileMap = null;
	/**
	Map of image related base 1 -> BASE 2 files
	 */
	private HashMap<Integer, Integer> imageFileMap = null;
	/**
	Map of printmap related base 1 -> BASE 2 files
	 */
	private HashMap<Integer, Integer> printmapFileMap = null;
	/**
	Map of rawBioAssay related base 1 -> BASE 2 files
	 */
	private HashMap<Integer, Integer> rawBioAssayFileMap = null;
	/**
	Whether or not item id's are used to calculate a subpath directory.
	 */
	private boolean useSubpath = true;
	/**
	If {@link #createItem(ResultSet)} should create a protocol related file.
	 */
	private boolean createProtocol = false;
	/**
	If {@link #createItem(ResultSet)} should create an image related file.
	 */
	private boolean createImage = false;
	/**
	If {@link #createItem(ResultSet)} should create a printmap related file.
	 */
	private boolean createPrintmap = false;
	/**
	If {@link #createItem(ResultSet)} should create a rawBioAssay related
	file.
	 */
	private boolean createRawBioAssay = false;
	
	/**
	 	This class's logger	 	
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.UploadFileTransfer");

	/**
	@param userT UserTransfer used to find owners of files
	@param groupT GroupTransfer used to find group owners of files
	@param path Subdirectory within the userfiles directory containing the
	       files to transfer
	 */
	UploadFileTransfer(UserTransfer userT, GroupTransfer groupT, String path)
	{
		super(userT, groupT);
		log.debug("Checking that necessary transfers have been done before proceeding with UploadFile");
		this.userfiles = path;
		md5ToIdMap = new HashMap<String, Integer>();
		protocolFileMap = new HashMap<Integer, Integer>();
		imageFileMap = new HashMap<Integer, Integer>();
		rawBioAssayFileMap = new HashMap<Integer, Integer>();
		printmapFileMap = new HashMap<Integer, Integer>();
		if (!new java.io.File(path).exists())
		{
			String message = "Path for BASE 1 user files not found: " + path;
			if (!QUICK_TRANSFER)
			{
				log.warn(message);
			}
			else
			{
				log.error(message);
				stop();
			}
		}
	}

	/**
	Starts this transfer.
	 */
	public void start()
	{
		log.info("Transferring files...");
		int rows = getInt("countUploads") + getInt("countProtocols")
				+ getInt("countImages") + getInt("countRawBioAssays")
				+ getInt("countPrintmaps");
		startProgress(rows, "Uploaded files");
		transferUploads(prepareStatementFromFile("selectUploads"));
		transferProtocolFiles(prepareStatementFromFile("selectProtocolFiles"));
		transferImages(prepareStatementFromFile("selectImageFiles"));
		transferPrintmaps(prepareStatementFromFile("selectPrintmapFiles"));
		transferRawBioAssayFiles(prepareStatementFromFile("selectRawBioAssayFiles"));
		done();
		log.info("Transferring files...DONE");
	}

	/**
	Starts transfer of rawBioAssay related files. Path to rawBioAssays is
	hardcoded here.
	
	@param ps Select statement listing rawBioAssay's from base 1
	 */
	private void transferRawBioAssayFiles(PreparedStatement ps)
	{
		path = userfiles + "/rawdata";
		useSubpath = true;
		createRawBioAssay = true;
		log.info("\trawBioAssay related files");
		runUnBatched(ps, 0, QUICK_TRANSFER ? 1000 : 10);
		createRawBioAssay = false;
	}

	/**
	Starts transfer of printmap related files Path to printmap is hardcoded here.
	
	@param ps Select statement listing image's from base 1
	 */
	private void transferPrintmaps(PreparedStatement ps)
	{
		path = userfiles + "/printmaps";
		useSubpath = false;
		createPrintmap = true;
		log.info("\tprint map related files");
		runUnBatched(ps, 0, QUICK_TRANSFER ? 1000 : 10);
		createPrintmap = false;
	}
	
	/**
	Starts transfer of image related files Path to images is hardcoded here.
	
	@param ps Select statement listing image's from base 1
	 */
	private void transferImages(PreparedStatement ps)
	{
		path = userfiles + "/rawimages";
		useSubpath = true;
		createImage = true;
		log.info("\timage related files");
		runUnBatched(ps, 0, QUICK_TRANSFER ? 1000 : 10);
		createImage = false;
	}

	/**
	Starts transfer of protocol related files Path to protocols is hardcoded
	here.
	
	@param ps Select statement listing protocol's from base 1
	 */
	private void transferProtocolFiles(PreparedStatement ps)
	{
		path = userfiles + "/protocols";
		useSubpath = false;
		createProtocol = true;
		log.info("\tprotocol related files");
		runUnBatched(ps, 0, QUICK_TRANSFER ? getBatchSize() : 10);
		createProtocol = false;
	}

	/**
	Starts transfer of upload related files Path to uploads is hardcoded
	here.
	
	@param ps Select statement listing upload's from base 1
	 */
	private void transferUploads(PreparedStatement ps)
	{
		path = userfiles + "/uploads";
		useSubpath = true;
		log.info("\tupload related files");
		runUnBatched(ps, 0, QUICK_TRANSFER ? getBatchSize() : 10);
	}

	/**
	Creates a file from the current row in the resultSet. This method is
	called from {@link Transfer#runUnBatched(PreparedStatement, int, int)}
	
	@param rs ResultSet containing information about the file.
	 */
	protected void createItem(ResultSet rs) throws BaseException, SQLException
	{
		String location = path + "/" + rs.getString(ID);
		if (useSubpath)
		{
			String subpath = "" + (int) (rs.getInt(ID) / 1000f);
			location = path + "/" + subpath + "/" + rs.getString(ID);			
		}
		log.debug("File's location in BASE1 is: " + location);
		DbControl dc = newDbControl();
		User user = getOwner(dc, rs.getInt(2));
		String fileName = rs.getString(6);
		if (fileName == null)
		{
			fileName = "unknown";
			log.debug("Unknown file name");
		}
		if (user != null)
		{
			fileName = replaceForbiddenCharacters(fileName, "_");
			// If quickTransfer is set to yes we use the name as a unique
			// identifier
			String md5 = QUICK_TRANSFER ? fileName : md5sum(location);
			if (md5 == null) 
			{
				log.debug("Couldn't get md5 for file: " + fileName + ":" + rs.getString(ID));
				log.warn("File:" + fileName + " with Base1Id:" + rs.getString(ID) + " will not be uploaded to BASE2");
				md5 = fileName;
			}

			File file = getFileByMd5(dc, user, md5);
			if (file == null)
			{
				Directory homeDir = user.getHomeDirectory();
				if (homeDir == null)
				{
					log.info("...upload related files");
					{
						log.debug("User " + user.getName() + " is missing a home directory");
					// Something is wrong here homeDir is not null when you
					// look through the web interface
					// homeDir = Directory.createHomeDirectory(dc, user,
					// true);
					// dc.saveItem(homeDir);
					}
				}
				int prefix = 0;
				String prefixedFileName = fileName;
				while (File.exists(dc, homeDir, prefixedFileName))
				{
					prefixedFileName = prefix + "_" + fileName;
					prefix++;
				}
				// All files are stored in the users homedirectory
				file = File.getNew(dc, homeDir);
				setCommonItemData(dc, file, rs);
				file.setName(prefixedFileName); // Reset the filename incase
												// we've prefixed it in the
												// above while loop
				dc.saveItem(file);
				log.debug("File item is saved into BASE2");
				transferFile(file, location, md5);
				if (createRawBioAssay || createPrintmap) file.setWriteProtected(true);
				commit(dc);
				String fileMd5 = file.getMd5();
				if (fileMd5 != null)
				{
					// We assume that each file generates a unique md5sum
					md5ToIdMap.put(
							user.getId() + "_" + fileMd5, file.getId());						
				}
				if (log.isDebugEnabled())
				{
					log.debug("Transfered File: " + file.getName() + "\tBASE1 id(" + rs.getInt(1) + ")");						
				}
			}
			else
			{
				log.info( file.getName() + " already exists");
			}
			if (createProtocol)
			{
				protocolFileMap.put(rs.getInt(1), file.getId());
			}
			if (createImage)
			{
				imageFileMap.put(rs.getInt(1), file.getId());
			}
			if (createRawBioAssay)
			{
				rawBioAssayFileMap.put(rs.getInt(1), file.getId());
			}
			if (createPrintmap)
			{
				printmapFileMap.put(rs.getInt(1), file.getId());
			}
		}
		else
		{
			log.warn("User with id " + rs.getInt(2) + " doesn't exist.");
		}
		if (dc.isConnected())
		{
			dc.commit();
		}
		log.debug("--------------------------------------------------------------------------");
	}

	/**
	Returns the file matching owner and md5.
	
	@param dc DbControl used to get the file
	@param owner
	@param md5
	@return File or null if no matching file is found
	 */
	public File getFileByMd5(DbControl dc, User owner, String md5)
	{
		try
		{
			String key = owner.getId() + "_" + md5;
			if (md5ToIdMap.containsKey(key))
			{
				Integer id = md5ToIdMap.get(key);
				if (id != null && id.intValue() != 0)
				{
					return File.getById(dc, id);
				}
			}
		}
		catch (Exception e)
		{
			log.error("Couldn't get file by Md5: " + md5, e);
			stop();
		}
		return null;
	}

	/**
	Returns the protocol related file matching a base1Id.
	
	@param dc DbControl used to get the file.
	@param base1Id
	 */
	public File getProtocolFileById(DbControl dc, int base1Id)
	{
		try
		{
			Integer id = protocolFileMap.get(base1Id);
			if (id != null && id.intValue() != 0)
			{
				return File.getById(dc, id);
			}
		}
		catch (Exception e)
		{
			log.error("Couldn't get BASE1 protocol file by id: " + base1Id, e);
			stop();
		}
		return null;
	}

	/**
	Returns the image related file matching a base1Id.
	
	@param dc DbControl used to get the file.
	@param base1Id
	 */
	public File getImageFileById(DbControl dc, int base1Id)
	{
		try
		{
			Integer id = imageFileMap.get(base1Id);
			if (id != null && id.intValue() != 0)
			{
				return File.getById(dc, id);
			}
		}
		catch (Exception e)
		{
			log.error("Couldn't get BASE1 image file by id: " + base1Id, e);
			stop();
		}
		return null;
	}
	
	/**
	Returns the printmap related file matching a base1Id.
	
	@param dc DbControl used to get the file.
	@param base1Id Of the array design
	 */
	public File getPrintmapFileById(DbControl dc, int base1Id)
	{
		try
		{
			Integer id = printmapFileMap.get(base1Id);
			if (id != null && id.intValue() != 0)
			{
				return File.getById(dc, id);
			}
		}
		catch (Exception e)
		{
			log.error("Couldn't get BASE1 printmap file by id: " + base1Id, e);
			stop();
		}
		return null;
	}

	/**
	Returns the rawBioAssay related file matching a base1Id.
	
	@param dc DbControl used to get the file.
	@param base1Id
	 */
	public File getRawBioAssayFileById(DbControl dc, int base1Id)
	{
		try
		{
			Integer id = rawBioAssayFileMap.get(base1Id);
			if (id != null && id.intValue() != 0)
			{
				return File.getById(dc, id);
			}
		}
		catch (Exception e)
		{
			log.error("Couldn't get BASE1 raw bioassay file by id: " + base1Id, e);
			stop();
		}
		return null;
	}
}
