/*
	$Id: UserTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Quota;
import net.sf.basedb.core.Role;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.User;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>UserGroup, UserGroupParent, UserAccount</dd>
<dt><b>Unused columns</b></dt>
<dd><b>UserGroup</b>: owner, addedDate<br/> <b>UserAccount</b>:
lastExperiment, lastExploreSet, lastPlateType, lastPlateTypeCreated,
defaultGid, defaultGroupAccess, defaultWorldAccess<br/> <b>UserGroupParent</b>:
isReviewer</dd>

</dl>

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class UserTransfer extends Transfer
{
	/**
	Roles have to be transfered first
	 */
	private RoleTransfer roleT = null;
	
	private QuotaTransfer quotaT = null;
	
	/**
	File quotas are stored and later used to calculate a users new filequota.
	Maps Base 1 ID -> total quota min bytes
	 */
	private Map<Integer, Long> base1FileQuota = null;
	/**
	A base 1 user may have additional permissions if he/she is a member of a
	group. These permissions are stored and later used together with the
	users own permissions.
	 */
	private Map<Integer, Integer> groupUserAccessMap = null;
	/**
	Holds information about existing users. Logins are mapped to User
	objects.
	 */
	private HashMap<String, User> existingUsers = null;

	/**
		This class logger
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.UserTransfer");
	
	UserTransfer(QuotaTransfer quotaT)
	{
		this.quotaT = quotaT;
	}
	
	/**
	@param roleT {@link RoleTransfer}
	 */
	UserTransfer(RoleTransfer roleT, QuotaTransfer quotaT)
	{
		log.debug("Checking that necessary transfers have been done before proceeding with User");
		if (roleT.isDone() && quotaT.isDone())
		{
			this.roleT = roleT;
			this.quotaT = quotaT;
			existingUsers = new HashMap<String, User>();
		}
		else
		{			
			if (!roleT.isDone()) log.debug("Dependency: Roles haven't been transfered");
			if (!quotaT.isDone()) log.debug("Dependency: Quotas haven't been transfered");
			dependencyFailure();			
		}
		//Standalone - do not add to root logger
	}

	public void start()
	{
		log.info("Transferring users...");
		DbControl b2c = null;
		try
		{
			b2c = newDbControl();
			ItemQuery<User> query = User.getQuery();
			query.include(Include.REMOVED);
			query.include(Include.NOT_REMOVED);
			ItemResultList<User> users = query.list(b2c);
			for (User u : users)
			{
				existingUsers.put(u.getLogin(), u);
			}
			b2c.close();
			int count = getInt("countUsers");
			initIdMap(count);
			startProgress(count, "Users");
			groupUserAccessMap = new HashMap<Integer, Integer>(count);
			base1FileQuota = new HashMap<Integer, Long>(count);
			// idUserMap = new User[ idMap.length + 1 ];
			// Find out combined group access privileges for each user.
			PreparedStatement groupPrivilegesPS = prepareStatementFromFile("selectGroupPrivileges");
			if (groupPrivilegesPS != null)
			{
				ResultSet rs = groupPrivilegesPS.executeQuery();
				while (rs.next())
				{
					Integer userId = rs.getInt(1);
					Integer mask = rs.getInt(2);
					Integer currentMask = groupUserAccessMap.get(userId);
					if (currentMask != null) mask |= currentMask;
					groupUserAccessMap.put(userId, mask);
				}
				rs.close();
				PreparedStatement usersPS = prepareStatementFromFile("selectUsers");
				runUnBatched(usersPS, 0, 100);
				progress.stop();
				b2c = newDbControl();
				query = User.getQuery();
				query.include(Include.REMOVED);
				query.include(Include.NOT_REMOVED);
				users = query.list(b2c);
				// Create home directories for transferred users
				log.info("Creating home directories for transfered users");
				ProgressThread progress = new ProgressThread(users.size(), "Home directories");
				progress.start();
				for (User user : users)
				{
					Directory home = user.getHomeDirectory();
					if (home == null)
					{
						home = Directory.createHomeDirectory(b2c, user, true);
						log.debug("Create home directory for " + user.getLogin());
						progress.increase();
					}
				}
				commit(b2c);
				progress.stop();
			}
			else
			{
				log.error("SQL statement not found");
				stop();
			}
		}
		catch (Exception e)
		{
			if (b2c != null && b2c.isConnected())
			{
				b2c.close();
			}
			log.error("User transfer FAILED", e);
			stop();
		}
		done();
		log.info("Transferring Users...DONE");
	}

	/**
	Creates a User from the current row in a given ResultSet.
	
	@param rs {@link ResultSet}
	 */
	void createItem(ResultSet rs)
	{
		DbControl dc = null;
		try
		{
			dc = newDbControl();
			String login = rs.getString(2);
			String name = rs.getString(3);
			int userId = rs.getInt(1);
			User user = null;
			if (existingUsers.containsKey(login))
			{
				user = existingUsers.get(login);
				mapId(userId, user.getId());
				dc.close();
				log.warn("'" + login + "' was not transfered cause there already exists an user in BASE2 with this login");
				return;
			}
			else
			{
				// True password is set below
				user = User.getNew(dc, login, "");
				user.setBase1Password(rs.getString(8));
				if (login.equals("") && name.equals(""))
				{
					login = rs.getString(1);
					name = rs.getString(1);
				}
				/* Unique logins are required */
				if (login.equals(""))
				{
					login = name;
				}
				if (name.equals(""))
				{
					name = login;
				}
				// Store the BASE 1 userId in the name, and fix this later
				user.setName(name);
				// fileQuota is saved for later use in finalizeQuota()
				long byteQuota = rs.getLong(11);
				if (rs.wasNull()) byteQuota = Quota.UNLIMITED;
				base1FileQuota.put(userId, byteQuota);
				/*
				 * Initially quota is set to unlimited. When all transfers are
				 * finished quota is updated to a suitible level.
				 */
				Quota unlimitedQuota = Quota.getById(dc, SystemItems.getId(Quota.UNRESTRICTED));
				user.setQuota(unlimitedQuota);
				try
				{
					user.setExpirationDate(rs.getDate(12));
				}
				catch (SQLException e)
				{
					user.setExpirationDate(null);
				}
			}
			user.setAddress(rs.getString(4));
			user.setDescription(rs.getString(5));
			user.setEmail(rs.getString(6));
			user.setFax(rs.getString(7));
			user.setPhone(rs.getString(9));
			user.setRemoved(rs.getBoolean(10));
			dc.saveItem(user);

			// Combine(OR) user accessMask with group accessMask
			Integer mask = rs.getInt(13);
			Integer groupMask = groupUserAccessMap.get(userId);
			if (groupMask != null) mask |= groupMask;
			Set<Integer> roleIds = roleT.getRoleIds(mask);
			for (Integer roleId : roleIds)
			{
				Role r = Role.getById(dc, roleId);
				r.addUser(user);
				if (log.isDebugEnabled()) 
				{
					log.debug("Assigning role '" + r.getName() + "' to user '" + user.getName() + "'");
				}
			}			
			dc.commit();
			mapId(userId, user.getId());
			if (log.isDebugEnabled())
			{
				log.debug("Transfered user '" + user.getLogin() + "'\tBASE1 id (" + userId + ")\tBASE2 id (" + user.getId() + ")");
			}
		}
		catch (Exception e)
		{
			if (dc != null && dc.isConnected())
			{
				dc.close();
			}
			log.warn("FAILED when transferring user");
			if (log.isDebugEnabled()) log.debug("\t" + e);
		}
		log.debug("--------------------------------------------------------------------------");
	}

	public void finalizeQuota()
	{
		startProgress(base1FileQuota.size(), "FinalizeQuota");
		DbControl dc = newDbControl();
		log.info("Adjusting Quota in BASE2");
		try
		{
			for (Map.Entry<Integer, Long> entry : base1FileQuota.entrySet())
			{
				Integer base1Id = entry.getKey();
				Long base1Quota = entry.getValue();
				
				User user = getUserById(dc, base1Id);
				Quota quota = quotaT.getQuotaForTotal(dc, base1Quota);
				log.debug("Setting quota '" + quota + "' for user '" + user.getName() + "'");
				user.setQuota(quota);
				progress.increase();
			}
			dc.commit();
		}
		finally
		{
			if (dc != null) dc.close();
			progress.stop();
		}
	}
	
	/**
		Store quota map
	*/
	protected void writeRestorePointData(ObjectOutputStream out)
		throws IOException
	{
		super.writeRestorePointData(out);
		out.writeObject(base1FileQuota);
	}
	
	/**
		Store ID map and some boolean flags
	*/
	@SuppressWarnings("unchecked")
	protected void readRestorePointData(ObjectInputStream in)
		throws IOException, ClassNotFoundException
	{
		super.readRestorePointData(in);
		base1FileQuota = (Map<Integer, Long>)in.readObject();
	}
	
	public User getUserById(DbControl dc, int base1Id)
	{
		try
		{
			Integer id = getBase2Id(base1Id);
			return id == null ? null : User.getById(dc, id);
		}
		catch (Exception e)
		{
			log.error("FAILED to get the user by id", e);
		}
		return null;
	}
}
