/*
	$Id: WellTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Plate;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.Well;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

/**
Transfers well annotations for all previously transferred plates.
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>Well</dd>
<dt><b>Unused columns</b></dt>
<dd><b>Well</b>: reporter_id</dd>
</dl>

@author gregory
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class WellTransfer extends Transfer
{
	private UnmappedPlateTransfer unmappedPlateT = null;
	private AnnotationType seqVerifiedAT = null;
	private AnnotationType bandsAT = null;
	private AnnotationType bacterialGrowthAT = null;
	private AnnotationType commentAT = null;
	
	/**
		The logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.WellTransfer");

	public WellTransfer(UnmappedPlateTransfer unmappedPlateT)
	{
		log.debug("Checking that necessary transfers have been done before proceeding with Well");
		if (unmappedPlateT.isDone())
		{
			this.unmappedPlateT = unmappedPlateT;
		}
		else
		{
			log.debug("Dependency: UnmappedPlates haven't been transfered");
			dependencyFailure();
		}
	}

	@Override
	void start()
	{
		log.info("Transferring Wells...");
		DbControl dc = newDbControl();
		log.debug("Getting annotation for 'Sequence verified'");
		seqVerifiedAT = getAnnotationTypeByName(dc, "Sequence verified");
		log.debug("Getting annotation for 'Bands'");
		bandsAT = getAnnotationTypeByName(dc, "Bands");
		log.debug("Getting annotation for 'Bacterial growth'");
		bacterialGrowthAT = getAnnotationTypeByName(dc, "Bacterial growth");
		log.debug("Getting annotation for 'Comment'");
		commentAT = getAnnotationTypeByName(dc, "Comment");
		dc.close();
		startProgress(countPlates(), "Wells");
		annotateWellsForEachPlate();
		progress.stop();
		done();
		log.info("Transferring Wells...DONE");
	}

	private void annotateWellsForEachPlate()
	{
		for (Map.Entry<Integer, Integer> entry : unmappedPlateT.getIdMap().entrySet())
		{
			Integer base1Id = entry.getKey();
			Integer base2Id = entry.getValue();
			if (base2Id != null)
			{
				DbControl dc = newDbControl();
				Plate plate = Plate.getById(dc, base2Id);
				if (plate != null)
				{
					dc.reattachItem(seqVerifiedAT);
					dc.reattachItem(bandsAT);
					dc.reattachItem(bacterialGrowthAT);
					dc.reattachItem(commentAT);
					ResultSet rs = null;
					PreparedStatement ps = prepareStatementFromFile("selectWellAndProbesByPlateId");
					try
					{
						ps.setInt(1, base1Id);
						rs = ps.executeQuery();
						annotateWells(dc, plate, rs);
					}
					catch (SQLException e)
					{
						log.error("FAILED when executing SQL statement", e);
						stop();
					}
				}
				if (dc != null && dc.isConnected()) dc.commit();
				progress.increase();
				if (log.isDebugEnabled())
				{
					log.debug("Annotated wells for Plate: " + plate);
					log.debug("--------------------------------------------------------------------------");
				}
			}
		}
	}

	/**
	Annotate wells for a given plate.
	
	@param dc <code>DbControl</code>
	@param plate Wells from this plate will be connected
	 */
	private void annotateWells(DbControl dc, Plate plate, ResultSet rs)
	{
		try
		{
			while (rs.next())
			{
				int col = rs.getInt(2);
				int row = rs.getInt(3);
				Well well = plate.getWell(row, col);
				if (well != null)
				{
					// Reporter data is connected i unmappdePlateTransfer
					// Annotate well
					AnnotationSet as = well.getAnnotationSet();
					dc.saveItem(as);
					String seqVerifiedValue = rs.getString(5);
					if (!"".equals(seqVerifiedValue))
					{
						Annotation seqVerified = as.getAnnotation(seqVerifiedAT);
						seqVerified.setValue(matchEnumeratedValue(
								seqVerifiedAT, seqVerifiedValue));
						dc.saveItem(seqVerified);
						log.debug("seqVerified annotationvalue: " + seqVerifiedValue + " added to well");
					}
					String bandsValue = rs.getString(6);
					if (!"".equals(bandsValue))
					{
						Annotation bands = as.getAnnotation(bandsAT);
						bands.setValue(matchEnumeratedValue(bandsAT, bandsValue));
						dc.saveItem(bands);
						log.debug("bands annotation value: " + bandsValue + " added to well");
					}
					String bacterialGrowthValue = rs.getString(7);
					if (!"".equals(bacterialGrowthValue))
					{
						Annotation bacterialGrowth = as.getAnnotation(bacterialGrowthAT);
						bacterialGrowth.setValue(matchEnumeratedValue(
								bacterialGrowthAT, bacterialGrowthValue));
						dc.saveItem(bacterialGrowth);
						log.debug("bacterialGrowth annotation value: " + bacterialGrowth + " added to well");
					}
					String commentValue = rs.getString(8);
					if (!"".equals(commentValue))
					{
						Annotation comment = as.getAnnotation(commentAT);
						comment.setValue(commentValue);
						dc.saveItem(comment);
						log.debug("comment annotation value: " + commentValue + " added to well");
					}
				}
			}
		}
		catch (SQLException e)
		{
			log.error("FAILED executing SQL statement", e);
			stop();
		}
		catch (InvalidDataException e)
		{
			log.warn("Invalid data when annotate well");
			if (log.isDebugEnabled())log.debug("\t" + e);
		}
	}

	@SuppressWarnings("unchecked")
	private String matchEnumeratedValue(AnnotationType at, String inValue)
	{
		List<String> values = (List<String>) at.getValues();
		for (String value : values)
		{
			if (((String) value).equalsIgnoreCase(inValue))
			{
				return value;
			}
		}
		return null;
	}

	/**
	@param dc
	@param name AnnotationType name too look for
	@return AnnotationType
	@throws ItemNotFoundException if AnnotationType by that name doesn't
	        exsist
	@throws BaseException if the query fails
	 */
	private AnnotationType getAnnotationTypeByName(DbControl dc, String name)
			throws ItemNotFoundException, BaseException
	{
		ItemQuery<AnnotationType> query = AnnotationType.getQuery(Item.WELL);
		query.restrict(Restrictions.eq(
				Hql.property("name"), Expressions.parameter("name")));
		query.setParameter("name", name, Type.STRING);
		ItemResultList<AnnotationType> result = query.list(dc);
		if (result.isEmpty())
		{
			log.warn("Couldn't find AnnotationType called: " + name);
			throw new ItemNotFoundException("AnnotationType named " + name
					+ " doesn't exsist");
		}
		else
		{
			return result.get(0);
		}
	}

	private int countPlates()
	{
		return unmappedPlateT.getCount();
	}
}
