/**
	$Id:ExtensionsDirectory.java 4187 2008-03-20 11:15:25Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.extensions;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.regex.Pattern;

import javax.servlet.ServletContext;

import net.sf.basedb.util.RegexpFileFilter;
import net.sf.basedb.util.extensions.Registry;
import net.sf.basedb.util.extensions.xml.PathConverter;
import net.sf.basedb.util.extensions.xml.VariableConverter;
import net.sf.basedb.util.extensions.xml.XmlLoader;

/**
	Represents a directory with extensions. This class can automatically
	detect changes and install, update or uninstall extensions when files
	are added, changed or removed from the specified directory. 
	Resources, such as images, JSP files, etc. are automatically extracted
	from JAR files and placed in the resources directory, which is a
	directory that should be accessible from the web server.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class ExtensionsDirectory
{

	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.clients.web.extensions.ExtensionsDirectory");
	
	private final File extensionsDir;
	private final File resourcesDir;
	private final String resourcesUrl;
	private final String rootUrl;
	
	private final Map<File, ExtensionsFile> installedFiles;
	private final Map<String, ExtensionsFile> installedExtensions;
	private final Set<File> staticIgnore;
	private final Set<File> ignore;
	private final VariableConverter variableConverter;
	private final PathConverter pathConverter;
	
	public ExtensionsDirectory(File extensionsDir, File resourcesDir, 
		String rootUrl, String resourcesUrl)
	{
		this.extensionsDir = extensionsDir;
		this.resourcesDir = resourcesDir;
		this.resourcesUrl = resourcesUrl;
		this.rootUrl = rootUrl;
		
		this.installedFiles = new TreeMap<File, ExtensionsFile>();
		this.installedExtensions = new HashMap<String, ExtensionsFile>();
		this.staticIgnore = new HashSet<File>();
		this.ignore = new HashSet<File>();
		this.variableConverter = new VariableConverter();
		this.variableConverter.setVariable("ROOT", rootUrl);
		this.variableConverter.setVariable("HOME", null);
		this.pathConverter = new PathConverter(rootUrl, null);
	}
	
	/**
		Get the file object representing the directory where
		extensions are installed.
		@return A file object
	*/
	public File getExtensionDirectory()
	{
		return extensionsDir;
	}
	
	/**
		Get the file object representing the directory where
		resource files are extracted.
		@return A file object
	*/
	public File getResourcesDirectory()
	{
		return resourcesDir;
	}
	
	/**
		Get the URL that points to the resources directory when accessed
		through the web server. The URL only includes the local server path,
		not the server name. For example, /base2/resources
		@return The URL
	*/
	public String getResourcesUrl()
	{
		return resourcesUrl;
	}
	
	/**
		Get the URL that points to the resources directory for a specific
		extensions file when accessed through the web server. The URL only 
		includes the local server path,	not the server name. For example, 
		/base2/resources/example-extensions.jar
		@return The URL
	*/
	public String getResourcesUrl(ExtensionsFile extFile)
	{
		return resourcesUrl + "/" + extFile.getName();
	}
	
	/**
		Get the URL that points to the root directory when accessed
		through the web server. The URL only includes the local server path,
		not the server name. For example, /base2
		@return The URL
	*/
	public String getRootUrl()
	{
		return rootUrl;
	}
	
	/**
		Add a file to the ignore list. Files in this list will never
		be checked for extensions again. This is only neccessary for XML/JAR
		files that doesn't contain extensions.
		@param file The file to add
		@see #removeIgnore(File)
	*/
	public void addIgnore(File file)
	{
		ignore.add(file);
		staticIgnore.add(file);
	}
	
	/**
		Remove a file from the ignore list.
		@param file The file to remove
		@see #addIgnore(File)
	*/
	public void removeIgnore(File file)
	{
		ignore.remove(file);
		staticIgnore.remove(file);
	}
	
	/**
		Get an iterator returning all XML/JAR files which contains installed
		extensions.
	*/
	public Iterator<ExtensionsFile> getFiles()
	{
		// Create a copy, or the iterator will fail if new
		// files are added by another thread
		List<ExtensionsFile> copy = 
			new ArrayList<ExtensionsFile>(installedFiles.values());
		return Collections.unmodifiableList(copy).iterator();	
	}
	
	/**
		Get information about an installed extensions file.
		@param filename The filename of the file
		@return An {@link ExtensionsFile} object or null if
			the given file doesn't exists or isn't an extensions file
	*/
	public ExtensionsFile getFile(String filename)
	{
		return installedFiles.get(new File(extensionsDir, filename));
	}
	
	/**
		Get information about the file a given extension or extension
		point is defined in.
		@param extensionId The ID of an extension or extension point
		@return Information about the file the extension is defined in,
			or null if no extension with the given ID is found
	*/
	ExtensionsFile getFileByExtensionId(String extensionId)
	{
		return installedExtensions.get(extensionId);
	}

	void addRegistered(String id, ExtensionsFile file)
	{
		ExtensionsFile other = installedExtensions.get(id);
		if (other == null)
		{
			installedExtensions.put(id, file);
		}
		else if (other != file)
		{
			throw new RuntimeException("Extension '" + id + 
				"' is already registered in file: " + other.getName());
		}
	}
	
	void removeRegistered(String id, ExtensionsFile file)
	{
		ExtensionsFile other = installedExtensions.get(id);
		if (other == file)
		{
			installedExtensions.remove(id);
		}
	}
	
	/**
		Scan the directory for new/modified/deleted extension files
		and perform neccesary changed in the registry.

		@param registry The registry holding registered extension points 
			and extensions
		@param manualScan TRUE if this is a manual scan, FALSE if it is an automatic
		@param forceUpdate TRUE to force an update of unmodified extensions,
			FALSE to leave them as they are
		@return The result of the scan
	*/
	synchronized ScanResults installAndUpdateExtensions(Registry registry, 
		ServletContext servletContext, boolean manualScan, boolean forceUpdate)
	{
		log.info("Begin scan for new/updated/deleted extensions");
		ScanResults results = new ScanResults(manualScan, forceUpdate);
		int numDeleted = 0;
		int numNew = 0;
		int numUpdated = 0;
		int numForced = 0;
		try
		{
			if (forceUpdate) 
			{
				// Re-scan failed JAR/XML files; Doesn't affect manually added ignore files
				resetIgnore();
				installedExtensions.clear();
			}
			
			// 1. Unregister all extensions that no longer exists
			// This method also deletes extracted resources
			numDeleted = unregisterDeleted(registry, results);
			
			// 2. Find new and modified extensions
			int[] numNewAndUpdated = scanForNewAndUpdated(forceUpdate, results);
			numNew = numNewAndUpdated[0];
			numUpdated = numNewAndUpdated[1];
			numForced = numNewAndUpdated[2];
			
			// Do we need to do anything?
			if (numNew > 0 || numUpdated > 0 || numForced > 0)
			{
				// 3. Load new and modified definitions
				loadDefinitions(forceUpdate, results);
				
				// 4. Extract resources 
				extractResources(forceUpdate, results);
				
				// 5. Load servlets
				loadServlets(servletContext, forceUpdate, results);
				
				// 6. Register new and updated extensions with the registry
				registerExtensions(registry, forceUpdate, results);
			}
		}
		finally
		{
			// Generate summary message, etc.
			String summary = numDeleted + " deleted extensions\n" + 
				numNew + " new extensions\n" +
				(numUpdated + numForced) + " updated extensions";
			if (results.getNumErrorFiles() > 0)
			{
				summary += "\n" + results.getNumErrorFiles() + " errors";
			}
			results.setSummary(summary);
			results.setEnded();
		}
		log.info("End scan for new/updated/deleted extensions");
		return results;
	}

	/**
		Unregister extensions/extension points that no longer
		exists. This method will also delete extracted resources.
	*/
	private int unregisterDeleted(Registry registry, ScanResults results)
	{
		log.info("Checking for deleted extensions in directory: " + 
			extensionsDir.getAbsolutePath());
		// Unregister extensions that have been deleted
		int numDeleted = 0;
		int numError = 0;
		Iterator<ExtensionsFile> it = installedFiles.values().iterator();
		while (it.hasNext())
		{
			ExtensionsFile extFile = it.next();
			boolean exists = extFile.exists();
			log.debug("File '" + extFile.getName() + "' " + (exists ? "exists" : "has been deleted"));
			if (!exists)
			{
				log.info("Unregistering extensions from " + extFile.getName());
				try
				{
					// Unregister the extensions
					int numExtensions = extFile.unregisterAll(registry);

					// Remove extracted resources
					int numResources = extFile.removeResources();
					
					// Set scan result messages 
					results.setStatus(extFile, "Deleted");
					results.addMessage(extFile, numExtensions + " extensions unregisterered.");
					results.addMessage(extFile, numResources + " resources deleted.");

					it.remove();
					numDeleted++;
				}
				catch (Throwable ex)
				{
					extFile.setError(true);
					results.addErrorMessage(extFile, 
							"Could not unregister extensions: " + ex.getMessage());
					log.error("Could not unregister " + extFile.getName(), ex);
					numError++;
				}
			}
		}
		log.info(numDeleted + " extensions has been deleted from directory: " + 
			extensionsDir.getAbsolutePath());
		if (numError > 0)
		{
			log.warn(numError + " extensions could not be deleted from directory: " + 
				extensionsDir.getAbsolutePath());
		}
		return numDeleted;
	}
	
	/**
		Scan for new extensions. This method checks in the 
		extensions directory for *.xml and *.jar files. New files
		will be validated and added to the {@link #installedFiles} map
		if valid or to the {@link #ignore} set if not valid. Files already
		in the {@link #ignore} set are not checked.
		
		@return [0] = num new; [1] = num updated; [2] = num forced update
	*/
	private int[] scanForNewAndUpdated(boolean forceUpdate, ScanResults results)
	{
		log.info("Scanning for new extensions in directory: " + 
			extensionsDir.getAbsolutePath());
		// Search for new .xml or .jar files; don't search subdirectories
		FileFilter filter = new RegexpFileFilter(".*\\.(xml|jar)", "");
		
		int numNew = 0;
		int numUpdated = 0;
		int numForced = 0;
		for (File file : extensionsDir.listFiles(filter))
		{
			// Ignore non-extension files (eg. settings.xml)
			if (ignore.contains(file)) continue;
			
			// Is this a known file or not?
			if (!installedFiles.containsKey(file))
			{
				log.debug("Found new file: " + file.getName());
				ExtensionsFile extFile = 
					new ExtensionsFile(this, file, new File(resourcesDir, file.getName()), createXmlLoader());
				
				// Is is a valid extensions file?
				if (extFile.isValid())
				{
					// This is a new valid extensions file; add it
					log.info("Found extensions in file: " + file.getName());
					installedFiles.put(file, extFile);
					numNew++;
					
					// Set scan result messages -- details will be added later
					results.setStatus(extFile, "Installed");
				}
				else
				{
					// Not a valid extensions file; add to ignore list
					Throwable validationError = extFile.getValidationError();
					log.info("Not a valid extensions file: " + file.getName(), validationError);
					ignore.add(file);
					
					// Set scan result messages 
					results.setStatus(extFile, "Ignored");
					String msg = validationError != null ? validationError.getMessage() : "";
					results.addMessage(extFile, "Not a valid extensions XML or JAR file: " + msg);
				}
			}
			else
			{
				// This is a known file, has it been modified?
				ExtensionsFile extFile = installedFiles.get(file);
				if (extFile.isModified())
				{
					log.debug("File has been updated: " + file.getName());
					results.setStatus(extFile, "Updated");
					numUpdated++;
				}
				else if (forceUpdate)
				{
					log.debug("File is already forced update: " + file.getName());
					results.setStatus(extFile, "Forced update");
					numForced++;
				}
				else
				{
					log.debug("File is already installed: " + file.getName());
					results.setStatus(extFile, "Already installed");
				}
			}
		}
		log.info(numNew + " new extensions found in directory: " + 
			extensionsDir.getAbsolutePath());
		log.info(numUpdated + " updated extensions found in directory: " + 
			extensionsDir.getAbsolutePath());
		log.info(numForced + " force updated extensions found in directory: " + 
				extensionsDir.getAbsolutePath());
		return new int[] { numNew, numUpdated , numForced };
	}
	
	/**
		Load the extension definitions from XML files. Unless forceUpdate
		is TRUE only new or modified extensions will be loaded.
	*/
	private void loadDefinitions(boolean forceUpdate, ScanResults results)
	{
		log.info("Loading new or updated extension definitions from: " + 
				extensionsDir.getAbsolutePath());
		log.debug("forceUpdate=" + forceUpdate);
		Iterator<ExtensionsFile> it = installedFiles.values().iterator();
		int numLoaded = 0;
		int numError = 0;
		while (it.hasNext())
		{
			ExtensionsFile extFile = it.next();
			try
			{
				// Only load new or updated file, unless forced
				boolean isModified = extFile.isModified();
				boolean isNew = extFile.isNew();
				if (forceUpdate || isModified)
				{
					log.debug(extFile.getName() + " is " + 
							(isNew ? "new" : isModified ? "modified" : "forced update"));
					
					// Reset error flag
					extFile.setError(false);
							
					// Set home directory for resources on value converters 
					String homePath = getResourcesUrl(extFile);
					log.debug("Home URL=" + homePath);
					variableConverter.setVariable("HOME", homePath);
					pathConverter.setHome(homePath);
					extFile.loadExtensions();
					log.debug(extFile.getName() + " has been loaded");
					numLoaded++;
				}
				else
				{
					log.debug("Extension " + extFile.getName() + " is up to date. Skipping.");
				}
			}
			catch (Throwable ex)
			{
				extFile.setError(true);
				results.addErrorMessage(extFile, 
						"Could not load extensions: " + ex.getMessage());
				log.error("Could not load definitions" , ex);
				numError++;
			}
		}
		log.info(numLoaded + " extensions loaded from directory: " + 
				extensionsDir.getAbsolutePath());
		if (numError > 0)
		{
			log.warn(numError + " extensions could not be loaded from directory: " + 
				extensionsDir.getAbsolutePath());
		}
	}
	
	/**
		Extract resources from JAR files. All files in /resources directory 
		inside extension JAR files are extract to the home directory of the
		extension. Unless forceUpdate is TRUE only resources which has a different
		length or last modified time than already existing files on the
		disk will be extracted.
	*/
	private void extractResources(boolean forceUpdate, ScanResults results)
	{
		// Extract resources for new and modified extensions
		Pattern resourcePattern = Pattern.compile("resources/(.*)");
		for (ExtensionsFile extFile : installedFiles.values())
		{
			// Do not extract from files with an error
			if (extFile.hasError()) continue;
			
			try
			{
				if (forceUpdate || extFile.isModified())
				{
					int numResources = extFile.extractResources(resourcePattern, "$1", forceUpdate);
					if (numResources > 0)
					{
						results.addMessage(extFile, 
							numResources + " resources extracted successfully.");
					}
				}
			}
			catch (Throwable ex)
			{
				extFile.setError(true);
				results.addErrorMessage(extFile, "Could not extract resources: " + ex.getMessage());
				log.error("Could not extract resources from " + extFile.getName(), ex);
			}
		}
	}
	
	/**
		Load servlet definitions from JAR files. Servlets should be
		defined in META-INF/servlets.xml. 
	*/
	private void loadServlets(ServletContext servletContext, boolean forceUpdate, ScanResults results)
	{
		for (ExtensionsFile extFile : installedFiles.values())
		{
			// Do not load servlets from files with an error
			if (extFile.hasError()) continue;
			
			try
			{
				if (forceUpdate || extFile.isModified())
				{
					int numServlets = extFile.loadServlets(servletContext);
					if (numServlets > 0)
					{
						results.addMessage(extFile, 
								numServlets + " servlets defined successfully.");
					}
				}
			}
			catch (Throwable ex)
			{
				extFile.setError(true);
				results.addErrorMessage(extFile, "Could not load servlets: " + ex.getMessage());
				log.error("Could not load servlets from " + extFile.getName(), ex);
			}
		}
	}

	
	/**
		Register extensions with the registry. Unless forceUpdate is TRUE only
		new and modified extensions are registered. Extensions files that has
		the {@link ExtensionsFile#hasError()} flag set are automatically disabled.
	*/
	private void registerExtensions(Registry registry,	boolean forceUpdate, ScanResults results)
	{
		for (ExtensionsFile extFile : installedFiles.values())
		{
			// Do not register extensions that has an error
			if (extFile.hasError()) continue;
			
			try
			{
				extFile.unregisterMissing(registry);
				int num = extFile.registerExtensionPoints(registry, forceUpdate);
				if (num > 0)
				{
					results.addMessage(extFile, num + " extension points registered.");
				}
			}
			catch (Throwable ex)
			{
				extFile.setError(true);
				results.addErrorMessage(extFile, 
					"Could not register extension points: " + ex.getMessage());
				log.error("Could not register extension points from " + extFile.getName(), ex);
			}
		}
		
		for (ExtensionsFile extFile : installedFiles.values())
		{
			try
			{
				// Do not register extensions that has an error
				if (extFile.hasError()) continue;
	
				try
				{
					int num = extFile.registerExtensions(registry, forceUpdate);
					results.addMessage(extFile, num + " extensions registered.");
				}
				catch (Throwable ex)
				{
					extFile.setError(true);
					results.addErrorMessage(extFile, 
						"Could not register extensions: " + ex.getMessage());
					log.error("Could not register extensions from " + extFile.getName(), ex);
				}
			}
			finally
			{
				// Update the last modified information
				extFile.resetModified();
			}
		}
	}
	

	/**
		Creates a new XmlLoader and registers the {@link #variableConverter}
		and {@link #pathConverter} with it.
	*/
	private XmlLoader createXmlLoader()
	{
		XmlLoader loader = new XmlLoader();
		loader.addValueConverter(variableConverter);
		loader.addValueConverter(pathConverter);
		return loader;
	}
	
	/**
		Clears all files that has been added to the 
		{@link #ignore} set. All files in {@link #staticIgnore}
		are re-added.
	*/
	private void resetIgnore()
	{
		ignore.clear();
		ignore.addAll(staticIgnore);
	}
}
