/**
	$Id: FormatterSettings.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Johan Enell

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.formatter;

import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.formatter.DateFormatter;

/**
	This is autility class for getting and setting formatter-related settings for 
	the logged in user.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
	@see FormatterFactory
*/
public class FormatterSettings
{

	/**
		The default number of decimals to display by a number formatter = @value
	*/
	public static final int DEFAULT_NUM_DECIMALS = 2;
	
	/**
		The default date format for a date formatter = @value
	*/
	public static final String DEFAULT_DATE_FORMAT = "yyyy-MM-dd";

	/**
		The default datetime format for a datetime formatter = @value
	*/
	public static final String DEFAULT_DATETIME_FORMAT = "yyyy-MM-dd HH:mm:ss";
	
	/**
		Get the number of decimals a number formatter should display.
		
		@param sc The logged in user's session control
		@return The number of decimals
		@see #DEFAULT_NUM_DECIMALS
		@see FormatterFactory#getNumberFormatter(SessionControl)
	*/
	public static int getNumDecimals(SessionControl sc)
	{
		return Values.getInt(sc.getUserClientSetting("formatter.number.decimals"), DEFAULT_NUM_DECIMALS);
	}
	
	/**
		Set the number of decimals a number formatter should display.
		
		@param sc The logged in user's session control
		@param numDecimals The number of decimals
		@see #DEFAULT_NUM_DECIMALS
		@see FormatterFactory#getNumberFormatter(SessionControl)
	*/
	public static void setNumDecimals(SessionControl sc, int numDecimals)
	{
		sc.setUserClientSetting("formatter.number.decimals", Integer.toString(numDecimals));
		sc.setSessionSetting("formatter.double", null);
		sc.setSessionSetting("formatter.float", null);
	}
	
	/**
		Get format string for a date formatter.
		
		@param sc The logged in user's session control
		@return The format string
		@see #DEFAULT_DATE_FORMAT
		@see FormatterFactory#getDateFormatter(SessionControl)
	*/
	public static String getDateFormat(SessionControl sc)
	{
		return Values.getString(sc.getUserClientSetting("formatter.date.formatstring"), DEFAULT_DATE_FORMAT);
	}
	
	/**
		Set the format string for a date formatter.
		
		@param sc The logged in user's session control
		@param dateFormat The format string
		@see #DEFAULT_DATE_FORMAT
		@see FormatterFactory#getDateFormatter(SessionControl)
		@see java.text.SimpleDateFormat
	*/
	public static void setDateFormat(SessionControl sc, String dateFormat)
	{
		try
		{
			new DateFormatter(dateFormat);
		}
		catch (Throwable t)
		{
			throw new InvalidDataException("Invalid date format: " + dateFormat, t);
		}
		sc.setUserClientSetting("formatter.date.formatstring", dateFormat);
		sc.setSessionSetting("formatter.date", null);
	}
	
	/**
		Get format string for a datetime formatter.
		
		@param sc The logged in user's session control
		@return The format string
		@see #DEFAULT_DATETIME_FORMAT
		@see FormatterFactory#getDateTimeFormatter(SessionControl)
	*/
	public static String getDateTimeFormat(SessionControl sc)
	{
		return Values.getString(sc.getUserClientSetting("formatter.datetime.formatstring"), DEFAULT_DATETIME_FORMAT);
	}
	
	/**
		Set the format string for a datetime formatter.
		
		@param sc The logged in user's session control
		@param dateTimeFormat The format string
		@see #DEFAULT_DATETIME_FORMAT
		@see FormatterFactory#getDateTimeFormatter(SessionControl)
		@see java.text.SimpleDateFormat
	*/
	public static void setDateTimeFormat(SessionControl sc, String dateTimeFormat)
	{
		try
		{
			new DateFormatter(dateTimeFormat);
		}
		catch (Throwable t)
		{
			throw new InvalidDataException("Invalid date/time format: " + dateTimeFormat, t);
		}
		sc.setUserClientSetting("formatter.datetime.formatstring", dateTimeFormat);
		sc.setSessionSetting("formatter.datetime", null);
	}

	
}
