/*
	$Id:  $

	Copyright (C) Nicklas Nordborg
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.servlet;

import java.io.File;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.basedb.clients.web.extensions.ExtensionsControl;
import net.sf.basedb.clients.web.extensions.ExtensionsDirectory;
import net.sf.basedb.clients.web.extensions.ExtensionsFile;
import net.sf.basedb.clients.web.extensions.ServletWrapper;

/**
	Servlet for starting the extension system when the web server starts. 
	The servlet should be configured with the
	<code>&lt;load-on-startup&gt;1&lt;/load-on-startup&gt;</code>
	option in the web.xml file. When the servlet is loaded it will initialise the
	{@link ExtensionsControl} class.
	<p>
	
	This servlet is also acting as a proxy for requests to extension defined
	servlets. Such servlets are defined in <code>META-INF/servlets.xml</code>
	in the extension JAR file. A request to a servlet must have the following
	URL pattern:
	<code>/extensions/[jar-name]/[servlet-name].servlet</code>
	
	@author nicklas
	@version 2.7
	@base.modified $Date: 2008-03-17 14:40:46 +0100 (Mon, 17 Mar 2008) $
*/
public class ExtensionsServlet 
	extends HttpServlet
{
	
	private static final long serialVersionUID = 338904698493055683L;

	public ExtensionsServlet()
	{}
	
	@Override
	public void init()
		throws ServletException
	{
		super.init();
		ServletConfig cfg = getServletConfig();
		ServletContext context = cfg.getServletContext();
		File extensionsDir = new File(context.getRealPath(ExtensionsControl.EXTENSIONS_URL));
		File resourcesDir = new File(context.getRealPath(ExtensionsControl.RESOURCES_URL));
		String rootUrl = context.getContextPath();
		
		ExtensionsDirectory extensions = new ExtensionsDirectory(
				extensionsDir, resourcesDir, rootUrl, rootUrl + ExtensionsControl.RESOURCES_URL);
		ExtensionsControl.init(extensions, context);
	}

	@Override
	public void destroy()
	{
		ExtensionsControl.close();
		super.destroy();
	}
	
	/**
		Path to servlet must match: /extensions/[jar-file-name]/[servlet-name].servlet
		group(1) = JAR name, group(2) = Servlet name
	*/
	private static final Pattern PATH_MATCH = 
		Pattern.compile(ExtensionsControl.RESOURCES_URL + "/([^/]+)/(.+)\\.servlet");
	
	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException
	{
		
		String servletPath = request.getServletPath();
		Matcher m = PATH_MATCH.matcher(servletPath);
		if (!m.matches())
		{
			response.sendError(HttpServletResponse.SC_NOT_FOUND, servletPath);
			return;
		}
		
		String jarName = m.group(1);
		String servletName = m.group(2);
		ExtensionsControl ec = ExtensionsControl.get(null);
		ExtensionsFile file = ec.getFile(jarName);
		if (file == null)
		{
			response.sendError(HttpServletResponse.SC_NOT_FOUND, servletPath);
			return;
		}
		else if (!file.isJar())
		{
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, servletPath + " is not a JAR file.");
			return;
		}
	
		ServletWrapper wrapper = file.getServlet(servletName);
		if (wrapper == null)
		{
			response.sendError(HttpServletResponse.SC_NOT_FOUND, servletPath);
			return;
		}
		wrapper.service(request, response);
	}

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException
	{
		doGet(req, resp);
	}

}
