/*
	$Id: PlotServlet.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.servlet;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.Application;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.File;
import net.sf.basedb.core.ItemAlreadyExistsException;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.JoinType;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Select;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.core.query.SqlResultIterator;
import net.sf.basedb.core.query.WhenStatement;
import net.sf.basedb.clients.web.formatter.FormatterFactory;
import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.util.BioAssaySetUtil;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.formatter.Formatter;
import net.sf.basedb.util.jep.ChannelFunction;
import net.sf.basedb.util.jep.Jep;
import net.sf.basedb.util.jep.JepFunction;
import net.sf.basedb.util.plot.HistogramPlot;
import net.sf.basedb.util.plot.PlotAnnotation;
import net.sf.basedb.util.plot.ScatterPlot;

import javax.imageio.ImageIO;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.renderer.xy.XYStepRenderer;
import org.jfree.chart.title.TextTitle;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.OutputStream;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

/**
 	This servlet can plot spot data for a bioassay or bioassay set. It uses
 	the JFreePlot package to generate the plots. It accepts the following parameters:
 	
 	<table border="1" cellspacing="0" cellpadding="2">
 	<tr>
 		<th>Parameter</th>
 		<th>Required</th>
 		<th>Default value</th>
 		<th>Description</th>
 	</tr>
 	
 	<tr>
 		<td>ID</td>
 		<td>yes</td>
 		<td></td>
 		<td>The SessionControl ID. See {@link Application#getSessionControl(String, String)}.</td>
 	</tr>

 	<tr>
 		<td>bioassayset_id</td>
 		<td>no</td>
 		<td></td>
 		<td>The ID of the bioassay set to plot. If no value is specified the bioassay_id is 
 		required.</td>
 	</tr>
 	
 	<tr>
 		<td>bioassay_id</td>
 		<td>if a bioassay set hasn't been specified</td>
 		<td></td>
 		<td>The ID of the bioassay to plot.</td>
 	</tr>

 	<tr>
 		<td>type</td>
 		<td>no</td>
 		<td>scatter</td>
 		<td>The type of plot to create. Supported values are "scatter", "histogram" or "cfplot".</td>
 	</tr>

 	<tr>
 		<td>download</td>
 		<td>no</td>
 		<td>false</td>
 		<td>A boolean value. If 1 or true, the browser will ask to save the image on the local disk.</td>
 	</tr>

 	<tr>
 		<td>saveAs</td>
 		<td>no</td>
 		<td></td>
 		<td>If specified, the image is saved as a file in the BASE file structure.</td>
 	</tr>

 	<tr>
 		<td>overwrite</td>
 		<td>no</td>
 		<td>false</td>
 		<td>A boolean values, that is used with the "saveAs" parameter. If 1 or true, an 
 		existing file will be overwritten with the image.</td>
 	</tr>

 	<tr>
 		<td>width</td>
 		<td>no</td>
 		<td>600</td>
 		<td>The width of the generated image in pixels. Must be between 100 and 1000 pixels.
 		The default and max width can be configured in the web.xml file.</td>
 	</tr>

 	<tr>
 		<td>height</td>
 		<td>no</td>
 		<td>400</td>
 		<td>The hieght of the generated image in pixels. Must be between 100 and 800.
 		The default and max height can be configured in the web.xml file.</td>
 	</tr>

 	<tr>
 		<td>format</td>
 		<td>no</td>
 		<td>png</td>
 		<td>The format of the image, "png" and "jpg" are the only supported values.</td>
 	</tr>

	<tr>
 		<td>xLog</td>
 		<td>no</td>
 		<td>false</td>
 		<td>A boolean parameter. If 1 or true the logarithm of the x values is calculated
 		before plotting them.</td>
 	</tr>
 	
	<tr>
 		<td>yLog</td>
 		<td>no</td>
 		<td>false</td>
 		<td>A boolean parameter. If 1 or true the logarithm of the y values is calculated
 		before plotting them.</td>
 	</tr>
 	
 	<tr>
 		<td>x</td>
 		<td>yes, unless type is "cfplot"</td>
 		<td></td>
 		<td>The expression to plot on the x axis. Multiple x parameters may be specified
 		to plot multiple series.</td>
 	</tr>

 	<tr>
 		<td>y</td>
 		<td>no, unless type is "scatter"</td>
 		<td></td>
 		<td>The expression to plot on the y axis. Multiple y parameters may be specified
 		to plot multiple series.</td>
 	</tr>

 	<tr>
 		<td>filter</td>
 		<td>no</td>
 		<td></td>
 		<td>An optional filter expression that limits the result returned by the spot query.</td>
 	</tr>

 	<tr>
 		<td>title</td>
 		<td>no</td>
 		<td></td>
 		<td>The title of the plot. If not specified the plot is generated without title.</td>
 	</tr>

 	<tr>
 		<td>xLabel</td>
 		<td>no</td>
 		<td></td>
 		<td>The label on the x-axis. If not specified the plot is generated without label.</td>
 	</tr>

 	<tr>
 		<td>yLabel</td>
 		<td>no</td>
 		<td></td>
 		<td>The label on the y-axis. If not specified the plot is generated without label.</td>
 	</tr>

 	<tr>
 		<td>annotation</td>
 		<td>no</td>
 		<td></td>
 		<td>Can only be used when plotting a bioassay set. The only allowed value so far
 		is "bioassay" which plots the spots for each bioassay in different series and color.</td>
 	</tr>

 	<tr>
 		<td>binSize</td>
 		<td>no</td>
 		<td>1.0</td>
 		<td>Only used for histogram plots. This is the range of values put into each
 		bin in the histogram. Ie. if the size is 1, the first bin may have values from -5 to -4, 
 		the second bin from -4 to -3, and so on.</td>
 	</tr>
 	
  	<tr>
 		<td>yAggregate</td>
 		<td>no</td>
 		<td>count</td>
 		<td>Only used for histogram plots. Determines what to plot on the y axis. 
 		Allowed values are "count", "mean", "sum", "max", "min", "stdev". If a value other 
 		than "count" is used an expression for the y-axis must also be specified.</td>
 	</tr>	

 	<tr>
 		<td>hiloAggregate</td>
 		<td>no</td>
 		<td></td>
 		<td>Only used for histogram plots when the yAggregate parameter isn't "count".
 		Allowed values are "hilomaxmin" and "hilostdev". This options adds a high-low bar
 		to each bin that is either the max/min range or the plotted value +/- 1 standard
 		deviation of the values in each bin.
 		</td>
 	</tr>
 	
 	</table>

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public final class PlotServlet
	extends HttpServlet
{	
	
	/**
		
	*/
	private static final long serialVersionUID = 80128526417552573L;
	private int maxWidth = 1000;
	private int maxHeight = 800;
	private int defaultWidth = 600;
	private int defaultHeight = 400;
	private String defaultFormat = "png";

	public void init()
		throws ServletException
	{
		ServletConfig cfg = getServletConfig();
		maxWidth = Values.getInt(cfg.getInitParameter("maxWidth"), maxWidth);
		maxHeight = Values.getInt(cfg.getInitParameter("maxHeight"), maxHeight);
		defaultWidth = Values.getInt(cfg.getInitParameter("defaultWidth"), defaultWidth);
		defaultHeight = Values.getInt(cfg.getInitParameter("defaultHeight"), defaultHeight);
		defaultFormat = Values.getString(cfg.getInitParameter("defaultFormat"), defaultFormat);
	}

	@SuppressWarnings("unchecked")
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws IOException, ServletException
	{
		final String ID = request.getParameter("ID");
		
		// Where to get the source data
		final int bioAssayId = Values.getInt(request.getParameter("bioassay_id"));
		final int bioAssaySetId = Values.getInt(request.getParameter("bioassayset_id"));
		final String type = Values.getString(request.getParameter("type"), "scatter");
		
		// Where to deliver the image
		final boolean download = Values.getBoolean(request.getParameter("download"));
		final String saveAs = Values.getStringOrNull(request.getParameter("saveAs"));
		final boolean overwrite = Values.getBoolean(request.getParameter("overwrite"));

		// Image settings, width, height and format
		int width = Values.getInt(request.getParameter("width"), defaultWidth);
		if (width <= 100) width = defaultWidth;
		if (width > maxWidth) width = maxWidth;
		int height = Values.getInt(request.getParameter("height"), defaultHeight);
		if (height <= 100) height = defaultHeight;
		if (height > maxHeight) height = maxHeight;
		String format = Values.getString(request.getParameter("format"), defaultFormat);
		if (!format.equals("jpeg") && !format.equals("png"))
		{
			format = defaultFormat;
		}
		
		// Formulas to plot
		final boolean xLog = Values.getBoolean(request.getParameter("xLog"));
		final boolean yLog = Values.getBoolean(request.getParameter("yLog"));
		final String[] xFormulas = request.getParameterValues("x");
		final String[] yFormulas = request.getParameterValues("y");
		final String filterFormula = request.getParameter("filter");

		// Axis, legends and titles
		String title = Values.getStringOrNull(request.getParameter("title"));
		String subTitle = Values.getStringOrNull(request.getParameter("subTitle"));
		String xLabel = Values.getStringOrNull(request.getParameter("xLabel"));
		if (xLog && xLabel != null) xLabel = "log2("+xLabel+")";
		String yLabel = Values.getStringOrNull(request.getParameter("yLabel"));
		if (yLog && yLabel != null) yLabel = "log2("+yLabel+")";
		final String annotation = Values.getStringOrNull(request.getParameter("annotation"));
		final boolean hasAnnotation = annotation != null;
		Select annotationSelect = null;

		// Scatter plot specified parameters
		
		// Histogram specific parameters
		final float binSize = Values.getFloat(request.getParameter("binSize"), 1.0f);
		final String yAggregate = Values.getString(request.getParameter("yAggregate"), "count");
		final boolean isCount = "count".equals(yAggregate);
		if (!isCount && yLabel != null) yLabel = yLabel + " (" + yAggregate + ")";
		final String hiloAggregate = isCount ? null : 
			Values.getStringOrNull(request.getParameter("hiloAggregate"));
		
		
		RenderedImage image = null;
		DbControl dc = null;

		try
		{
			final SessionControl sc = Application.getSessionControl(ID, request.getRemoteAddr());
			dc = sc.newDbControl();
			BioAssay ba = bioAssayId == 0 ? null : BioAssay.getById(dc, bioAssayId);
			BioAssaySet bas = ba == null ? BioAssaySet.getById(dc, bioAssaySetId) : ba.getBioAssaySet();
			
			JFreeChart chart = null;
			if ("cfplot".equals(type))
			{
				chart = generateCFPlot(ba, xLabel, yLabel);
			}
			else
			{
				// Get the query
				DynamicSpotQuery query = ba != null ? ba.getSpotData() : bas.getSpotData();
				query.setAutoJoinType(JoinType.INNER);

				// Filter
				Restriction filter = filterFormula == null ? 
					null : BioAssaySetUtil.createJepRestriction(dc, bas, filterFormula, false);
				
				// Convert formulas to Expression:s
				Expression[] x = xFormulas == null ? null : new Expression[xFormulas.length];
				Expression[] y = yFormulas == null ? null : new Expression[yFormulas.length];
				
				if (xFormulas != null)
				{
					for (int i = 0; i < xFormulas.length; ++i)
					{
						x[i] = BioAssaySetUtil.createJepExpression(dc, bas, xFormulas[i], false);
						if (xLog) x[i] = Expressions.log2(x[i]);
					}
				}
				if (yFormulas != null)
				{
					for (int i = 0; i < yFormulas.length; ++i)
					{
						y[i] = BioAssaySetUtil.createJepExpression(dc, bas, yFormulas[i], false);
						if (yLog) y[i] = Expressions.log2(y[i]);
					}
				}
	
				// Get annotations
				Collection<PlotAnnotation> annotations = null;
				if (hasAnnotation)
				{
					annotations = new TreeSet<PlotAnnotation>();
					List<BioAssay> bioAssays = bas.getBioAssays().list(dc);
					annotationSelect = Dynamic.select(VirtualColumn.COLUMN);
					if ("bioassay".equals(annotation))
					{
						for (BioAssay bioAssay : bioAssays)
						{
							annotations.add(new PlotAnnotation(bioAssay.getName(), 
								Collections.singleton(new Integer(bioAssay.getDataCubeColumnNo()))));
						}
					}
					else if (annotation.matches("\\d+"))
					{
						int annotationTypeId = Values.getInt(annotation);
						AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
						Map<Set<?>, List<Integer>> groups = new HashMap<Set<?>, List<Integer>>();
						Formatter formatter = FormatterFactory.getTypeFormatter(sc, at.getValueType());
						for (BioAssay bioAssay : bioAssays)
						{
							Set<?> values = BioAssaySetUtil.getAnnotationValues(dc, bioAssay, at);
							if (!groups.containsKey(values))
							{
								groups.put(values, new ArrayList<Integer>());
							}
							groups.get(values).add((int)bioAssay.getDataCubeColumnNo());
						}
						for (Map.Entry<Set<?>, List<Integer>> entry : groups.entrySet())
						{
							String name = Values.getString(entry.getKey(), ", ", true, formatter);
							if (name == null || name.trim().equals("")) name = "null";
							annotations.add(new PlotAnnotation(name, entry.getValue()));
						}
					}
					else if (annotation.startsWith("="))
					{
						// JEP restriction
						Restriction r = 
							BioAssaySetUtil.createJepRestriction(dc, bas, annotation.substring(1), false);
						Expression e = Expressions.caseWhen(Expressions.integer(0), 
								new WhenStatement(r, Expressions.integer(1)));
						annotationSelect = Selects.expression(e, "ann");
						annotations.add(new PlotAnnotation("", 0));
						annotations.add(new PlotAnnotation(annotation.substring(1), 1));
					}
					else
					{
						Map<Set<Object>, List<Integer>> groups = new HashMap<Set<Object>, List<Integer>>();
						for (BioAssay bioAssay : bioAssays)
						{
							Set<Object> values = BioAssaySetUtil.getParentProperties(dc, bioAssay, annotation);
							
							if (!groups.containsKey(values))
							{
								groups.put(values, new ArrayList<Integer>());
							}
							groups.get(values).add((int)bioAssay.getDataCubeColumnNo());
						}
						for (Map.Entry<Set<Object>, List<Integer>> entry : groups.entrySet())
						{
							String name = Values.getString(entry.getKey(), ", ", true);
							if (name == null || name.trim().equals("")) name = "null";
							annotations.add(new PlotAnnotation(name, entry.getValue()));
						}
						
					}
				}
				
				if (filter != null) query.restrictPermanent(filter);
	
				if ("scatter".equals(type))
				{
					ScatterPlot plot = new ScatterPlot(xLabel, yLabel);
					for (int i = 0; i < x.length; ++i)
					{
						query.reset();
						if (annotationSelect != null)
						{
							query.select(annotationSelect);
						}
						query.select(Selects.expression(x[i], "x"));
						query.select(Selects.expression(y[i], "y"));
						if (hasAnnotation)
						{
							plot.addData(query.iterate(dc), annotations);
						}
						else
						{
							plot.addData(query.iterate(dc), Integer.toString(i));
						}
					}
					chart = plot.getChart();
				}
				else
				{
					HistogramPlot plot = new HistogramPlot(xLabel, yLabel, 
						hasAnnotation ? new XYStepRenderer() : null);
					query.setReturnTotalCount(true);
					for (int i = 0; i < x.length; ++i)
					{
						query.reset();
						if (annotationSelect != null)
						{
							query.select(annotationSelect);
						}
						query.select(Selects.expression(x[i], "x"));
						if (!isCount) query.select(Selects.expression(y[i], "y"));
						HistogramPlot.YAggregate ya = HistogramPlot.YAggregate.valueOf(yAggregate.toUpperCase());
						HistogramPlot.YAggregate hilo = hiloAggregate == null ? null : 
							HistogramPlot.YAggregate.valueOf(hiloAggregate.toUpperCase());
						if (hasAnnotation)
						{
							plot.addData(query.iterate(dc), annotations, binSize, ya, hilo);
						}
						else
						{
							plot.addData(query.iterate(dc), Integer.toString(i), binSize, ya, hilo);
						}
					}
					chart = plot.getChart();
				}
			}
			
			if (title != null) chart.setTitle(title);
			if (!hasAnnotation) chart.removeLegend();
			if (subTitle != null) chart.addSubtitle(new TextTitle(subTitle));
			image = chart.createBufferedImage(width, height);
			
			if (saveAs != null)
			{
				Path path = new Path(saveAs, Path.Type.FILE);
				File f = File.getByPath(dc, path, true);
				f.setMimeType("image/"+format);
				
				if (f.isInDatabase())
				{
					if (!overwrite) throw new ItemAlreadyExistsException("File[path="+saveAs+"]");
				}
				else
				{
					dc.saveItem(f);
				}
				OutputStream out = f.getUploadStream(false);
				ImageIO.write(image, format, out);
				out.flush();
				out.close();
				dc.commit();
				image = null;
			}
		}
		catch (Throwable t)
		{
			t.printStackTrace();
			if (saveAs == null)
			{
				// Generate error image
				try
				{
					BufferedImage img = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
					Graphics2D graphics = img.createGraphics();
					graphics.setBackground(Color.WHITE);
					graphics.clearRect(0, 0, width, height);
					graphics.setColor(Color.RED);
					FontRenderContext context = graphics.getFontRenderContext();

					Font topFont = new Font("SansSerif", Font.BOLD, 12);
					Font stackTraceFont = new Font("SansSerif", Font.PLAIN, 12);

					float x = .0f;
					float y = .0f;
					String causedBy = "";
					do
					{
						graphics.setFont(topFont);
						String msg = causedBy + t.getClass().getSimpleName() + ": " + t.getMessage();
						x = 5.0f;
						Rectangle2D bounds = topFont.getStringBounds(msg, context);
						y += bounds.getHeight()+5;
						if (bounds.getWidth() > width - x)
						{
							msg = cutMiddle(msg, (width - x) / bounds.getWidth());
						}
						graphics.drawString(msg, x, y);
						
						graphics.setFont(stackTraceFont);
						x = 15.0f;
						for (StackTraceElement st : t.getStackTrace())
						{
							msg = "at " + st.getClassName() + "." + st.getMethodName() + 
								":" + st.getLineNumber();
							bounds = stackTraceFont.getStringBounds(msg, context);
							y += bounds.getHeight();
							if (bounds.getWidth() > width - x)
							{
								msg = cutMiddle(msg, (width - x) / bounds.getWidth());
							}
							graphics.drawString(msg, x, y);
						}
						t = t.getCause();
						causedBy = "Caused by: ";
					} while (t != null && y < height);
					image = img;
				}
				catch (Throwable t2)
				{
					t2.printStackTrace();
				}
			}
			else 
			{
				// Let servlet container redirect to appropriate error page
				throw new ServletException(t);
			}
		}
		finally 
		{
			if (dc != null) dc.close();
		}
		
		if (image != null)
		{
			if (download)
			{
				response.setHeader("Content-Disposition", "attachment;filename="+HTML.urlEncode(title)+"." + format);
			}
			response.setContentType("image/" + format);
			response.setHeader("Cache-Control", "max-age=3600");
			OutputStream out = response.getOutputStream();
			ImageIO.write(image, format, out);			
			out.flush();
			out.close();
		}
		else
		{
			if (saveAs != null)
			{
				// Image was save to BASE filesystem, redirect to close popup page
				response.sendRedirect(request.getContextPath()+"/common/close_popup.jsp?message=Plot+saved+successfully");
			}
			else
			{
				// Something went wrong when genering the error image - send redirect to a predefined image
				response.sendRedirect(request.getContextPath() + "/images/plot_error.gif");
			}
		}
	}

	/**
	   Forwards requests to {@link #doGet(HttpServletRequest, HttpServletResponse)}
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response)
		throws IOException, ServletException
	{
		doGet(request, response);
	}
	
	/**
		Generates a correction-factor plot, which is plotting the
		MA-data for the parent together with the difference in M between
		the parent and child.
	*/
	private JFreeChart generateCFPlot(BioAssay ba, String xLabel, String yLabel)
		throws Exception
	{
		DbControl dc = ba.getDbControl();
		// Get parent bioassay
		List<BioAssay> parents = ba.getParents().list(dc);
		
		if (parents.size() != 1)
		{
			throw new BaseException("A bioassay must have exactly one parent to generate a correction factor plot");
		}
		
		BioAssay parent = parents.get(0);
		
		// Generate MA-data for it
		JepFunction ch = new ChannelFunction();
		
		Expression m = Jep.formulaToExpression("log2(ch(1) / ch(2))", ch);
		Expression a = Jep.formulaToExpression("log(ch(1) * ch(2)) / 2", ch);
		Restriction positiveIntensity = Jep.formulaToRestriction("ch(1) > 0 && ch(2) > 0", ch);
		Expression position = Dynamic.column(VirtualColumn.POSITION);
		
		DynamicSpotQuery parentQuery = parent.getSpotData();
		parentQuery.select(Selects.expression(position, "pos"));
		parentQuery.select(Selects.expression(m, "m"));
		parentQuery.select(Selects.expression(a, "a"));
		parentQuery.restrict(positiveIntensity);
		parentQuery.order(Orders.asc(position));

		DynamicSpotQuery query = ba.getSpotData();
		query.select(Selects.expression(position, "pos"));
		query.select(Selects.expression(m, "m"));
		query.restrict(positiveIntensity);
		query.order(Orders.asc(position));
		
		// Add CF-data to the plot
		
		SqlResultIterator cf = new CFResultIterator(parentQuery.iterate(dc), query.iterate(dc));
		
		ScatterPlot plot = new ScatterPlot(xLabel, yLabel);
		plot.addData(cf, "ma", "cf");
		return plot.getChart(); 
	}
	
	
	private String cutMiddle(String msg, double fraction)
	{
		int charactersToCut = (int)(msg.length() * (1 - fraction)) + 3;
		int beginAt = (msg.length() - charactersToCut) / 2;
		return msg.substring(0, beginAt) + "..." + msg.substring(beginAt + charactersToCut);
	}
	
	/**
		Special implementation of a <code>SqlResultIterator</code>
		that combines the result from the parent and child into 
		one virtual iterator. The parent iterator must select
		the following columns in this order: postion, parent M, parent A.
		The child iterator must selected the following columns in this order:
		position, child M.
		<p>
		The we map our own columns like this:
		a --> parent A, m --> parent M, cf --> parent M - child M
	*/
	private static class CFResultIterator
		implements SqlResultIterator
	{
		
		private final SqlResultIterator parent;
		private final SqlResultIterator child;
		private final CFSqlResult next;
		
		CFResultIterator(SqlResultIterator parent, SqlResultIterator child)
		{
			this.parent = parent;
			this.child = child;
			this.next = new CFSqlResult();
		}

		/*
			From the Iterator interface
			-------------------------------------------
		*/
		public boolean hasNext()
		{
			return child.hasNext();
		}
		public SqlResult next()
		{
			try
			{
				SqlResult nextChild = child.next();
				/*
					The child may contain fewer spots than the parent so we
					iterate until we find the same position.
				*/
				while (parent.hasNext())
				{
					SqlResult nextParent = parent.next();
					if (nextChild.getInt(1) == nextParent.getInt(1))
					{
						next.setResult(nextParent, nextChild);
						return next;
					}
				}
			}
			catch (SQLException ex)
			{
				throw new BaseException(ex);
			}
			return null;
		}
		/**
			Not supported.
			@throws UnsupportedOperationException Always
		*/
		public void remove()
		{
			throw new UnsupportedOperationException("remove");
		}
		// -------------------------------------------

		/*
			From the QueryResult interface
			-------------------------------------------
		*/
		public long getTotalCount()
		{
			return child.getTotalCount();
		}
		// -------------------------------------------

		/*
			From the ResultIterator interface
			-------------------------------------------
		*/
		public void close()
		{
			parent.close();
			child.close();
		}
		public boolean isClosed()
		{
			return child.isClosed();
		}
		// -------------------------------------------

		/*
			From the SqlResultIterator interface
			-------------------------------------------
		*/
		public int getIndex(String name) 
			throws SQLException
		{
			if ("a".equals(name))
			{
				return 1;
			}
			else if ("m".equals(name))
			{
				return 2;
			}
			else if ("cf".equals(name))
			{
				return 3;
			}
			throw new SQLException("Unknown column name: " + name);
		}
		// -------------------------------------------
		
	}
	
	/**
	 	A single result row that is combined from the parent and
	 	child results. 1 --> parent A, 2 --> parent M, 3 --> parent M - child M
	*/
	private static class CFSqlResult
		implements SqlResult
	{
		
		private SqlResult parent;
		private SqlResult child;
		
		CFSqlResult()
		{}

		void setResult(SqlResult parent, SqlResult child)
		{
			this.parent = parent;
			this.child = child;
		}
		
		/*
			From the SqlResult interface
			-------------------------------------------
		*/
		public float getFloat(int index)
			throws SQLException
		{
			switch (index)
			{
				case 1:
					return parent.getFloat(3);
				case 2:
					return parent.getFloat(2);
				case 3: 
					return parent.getFloat(2) - child.getFloat(2);
				default:
					throw new ArrayIndexOutOfBoundsException(index);
			}
		}
		public int getInt(int index) 
			throws SQLException
		{
			return (int)getFloat(index);
		}
		public Object getObject(int index) 
			throws SQLException
		{
			return getFloat(index);
		}
		public short getShort(int index) 
			throws SQLException
		{
			return (short)getFloat(index);
		}
		public String getString(int index) 
			throws SQLException
		{
			return String.valueOf(getFloat(index));
		}
		// -------------------------------------------
		
	}
}
