/*
	$Id: HideableSection.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.taglib;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

import net.sf.basedb.core.ItemContext;

/**
	This tag is used to display a section on a web page that
	can be shown/hidden by the user by clicking on an icon
	on it's title.
	<p>
	<b>Example:</b>
	<pre>
   &lt;%@ taglib prefix="base" uri="/WEB-INF/base.tld" %&gt;
   &lt;base:section id="my.section" title="Click to hide/show"&gt;
   This is the contents of this section...
   &lt;/base:section&gt;
</pre>

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;base:section
      id=...
      clazz=...
      showclazz=...
      hideclazz=...
      style=...
      contentstyle=...
      title=...
      visible=...
      ontoggle=...
      initial=show|hide
      context=...
   &gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>no</td>
		<td>The ID of the section. The value of this attribute
		goes directly into the standard <code>id</code> attribute.
		This means that if you need a reference to the scetion object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		section = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>note</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute both when the section contents is hidden or
			visible.
		</td>
	</tr>
	<tr>
		<td>showclazz</td>
		<td>note</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute when the section contents is visible.
		</td>
	</tr>
	<tr>
		<td>hideclazz</td>
		<td>note</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute when the section contents is hidden.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different sections.
		</td>
	</tr>
	<tr>
		<td>contentstyle</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute of the content part. This allows you to apply different styles
			to different sections.
		</td>
	</tr>
	<tr>
		<td>title</td>
		<td></td>
		<td>no</td>
		<td>The title of the section.</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the section should be visible or not.
		</td>
	</tr>
	<tr>
		<td>ontoggle</td>
		<td>HideableSection.toggle or HideableSection.toggleAndSendWithAjax</td>
		<td>no</td>
		<td>
			The javascript method to call when a user click the show/hide icon. If no
			context has been set the default value is HideableSection.toggle, otherwise
			it is HideableSection.toggleAndSendWithAjax. If no context is set
			the given method is called with the value of the <code>id</code> attribute 
			as the only parameter. If a context is set the method is called with three
			parameters: <code>id</code>, <code>context.getItemType().name()</code>
			and <code>context.getSubContext</code> in that order.
		</td>
	</tr>
	<tr>
		<td>initial</td>
		<td>show</td>
		<td>no</td>
		<td>
			If the value is <code>hide</code> the section is hidden, otherwise it
			is visible. This setting is ignored if a <code>context</code> is given.  
		</td>
	</tr>
	<tr>
		<td>context</td>
		<td>-</td>
		<td>no</td>
		<td>
			An optional {@link ItemContext} object. If given the <code>initial</code>
			atribute is ignored. The valus is instead taken from the context setting
			<code>section.id</code> where id is replaced with the value of the 
			<code>id</code> attribute. This should have the value "1" to show the
			section or "0" to hide the section. If a context is given the default
			javascript method will also send an Ajax request to the web server which
			will update the setting in the context with a new value for the
			show/hide setting. Thus, all settings are remembered during a session 
			and also when the user returns to the next session.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.4
*/
public class HideableSection
	extends TagSupport
{

	/**
		
	*/
	private static final long serialVersionUID = 4750788983708350436L;

	/**
		The parent &lt;base:page&gt; tag.
	*/
	private Page page;

	/**
		Optional <code>class</code> attribute
	*/
	private String clazz = null;
	
	private String showClass = "docked";
	private String hideClass = null;
	
	/**
		Optional <code>style</code> attribute
	*/
	private String style = null;
	

	/**
		Optional <code>style</code> attribute for the content section
	*/
	private String contentStyle = null;

	/**
		The title of the section.
	*/
	private String title = null;

	/**
		If the section should be visible or not.
	*/
	private boolean visible = true;
	
	/**
		Javascript to call when showing/hiding section
	*/
	private String onToggle = null;
	
	/**
		If the section should be shown or hidden to start with.
	*/
	private boolean showInitially = true;
	
	private ItemContext context = null;
	
	/*
		Taglib initialization methods
	*/
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	public String getClazz()
	{
		return clazz;
	}	
	public void setShowclazz(String clazz)
	{
		this.showClass = clazz;
	}
	public String getShowclazz()
	{
		return showClass;
	}	
	public void setHideclazz(String clazz)
	{
		this.hideClass = clazz;
	}
	public String getHideclazz()
	{
		return hideClass;
	}	

	public void setStyle(String style)
	{
		this.style = style;
	}
	public String getStyle()
	{
		return style;
	}
	
	public void setContentstyle(String contentStyle)
	{
		this.contentStyle = contentStyle;
	}
	public String getContentstyle()
	{
		return contentStyle;
	}

	public void setTitle(String title)
	{
		this.title = title;
	}
	public String getTitle()
	{
		return title;
	}

	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}

	public void setInitial(String initial)
	{
		this.showInitially = !"hide".equals(initial);
	}
	public String getInitial()
	{
		return showInitially ? "show" : "hide";
	}

	public void setOntoggle(String onToggle)
	{
		this.onToggle = onToggle;
	}
	public String getOntoggle()
	{
		return onToggle != null ? onToggle : 
			context == null ? "HideableSection.toggle" : "HideableSection.toggleAndSendWithAjax";
	}
	
	public void setContext(ItemContext context)
	{
		this.context = context;
	}
	public ItemContext getContext()
	{
		return context;
	}

	public int doStartTag()
		throws JspException
	{
		page = (Page)findAncestorWithClass(this, Page.class);
		if (page == null) throw new JspTagException("Tag <base:section> must be inside a <base:page> tag");
		String id = getId();
		if (id == null) throw new JspTagException("id attribute must have a value");
		if (!isVisible()) return SKIP_BODY;

		boolean showContent = context == null ? showInitially : !"0".equals(context.getSetting("section."+id));
		String clazz = getClazz();
		String style = getContentstyle();
		String icon = "";
		if (showContent)
		{
			if (getShowclazz() != null) clazz = getShowclazz() + (clazz == null ? "" : " " + clazz);
			icon = "hide_section.gif";
		}
		else
		{
			if (getHideclazz() != null) clazz = getHideclazz() + (clazz == null ? "" : " " + clazz);
			style = "display: none;" + (style != null ? style : "");
			icon = "show_section.gif";
		}
		
		StringBuilder sb = new StringBuilder();
		sb.append("<div");
		sb.append(" id=\"").append(id).append("\"");
		if (clazz != null) sb.append(" class=\"").append(clazz).append("\"");
		if (getShowclazz() != null) sb.append(" showclass=\"").append(getShowclazz()).append("\"");
		if (getHideclazz() != null) sb.append(" hideclass=\"").append(getHideclazz()).append("\"");
		if (getStyle() != null) sb.append(" style=\"").append(getStyle()).append("\"");
		sb.append(">\n");
		sb.append("<h4>");
		sb.append("<img id=\"").append(id).append(".icon\" src=\"");
		sb.append(page.getRoot()).append("images/").append(icon).append("\"");
		sb.append(" title=\"Click to show/hide the contents of this section\"");
		sb.append(" style=\"cursor: pointer;\"");
		sb.append(" onclick=\"").append(getOntoggle()).append("('").append(id).append("'");
		if (context != null) 
		{
			sb.append(",'").append(context.getItemType().name()).append("'");
			sb.append(",'").append(context.getSubContext()).append("'");
		}
		sb.append(")\">");
		sb.append(getTitle()).append("</h4>\n");
		sb.append("<div");
		sb.append(" id=\"").append(id).append(".content\"");
		if (style != null) sb.append(" style=\"").append(style).append("\"");
		sb.append(">");

		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	public int doEndTag()
		throws JspException
	{
		if (!isVisible()) return EVAL_PAGE;
		StringBuilder sb = new StringBuilder();
		sb.append("</div>\n</div>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}
}




