/*
	$Id: Cell.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.taglib.table;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import net.sf.basedb.util.formatter.Formatter;

/**
	This tag defines a new cell in a row. It should be placed inside a
	{@link Row &lt;tbl:row&gt;} tag.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:cell
      clazz=...
      style=...
      column=...
      visible=true|false
      value=...
   &gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>cell</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>column</td>
		<td>-</td>
		<td>yes</td>
		<td>
			The ID of the {@link ColumnDef &lt;tbl:columndef&gt;} this cell
			is linked to.
		</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the cell should be visible or not.
		</td>
	</tr>
	<tr>
		<td>value</td>
		<td>-</td>
		<td>no</td>
		<td>
			A value to display in the cell. If a {@link Formatter} has been
			defined for this column the value is formatted otherwise it is just
			converted to a string with the toString() method.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.0
*/
public class Cell
	extends BodyTagSupport
{

	/**
		
	*/
	private static final long serialVersionUID = -1992298892993613884L;

	/**
		The parent &lt;tbl:table&gt; tag.
	*/
	private Table table;
	
	/**
		The parent &lt;tbl:data&gt; tag.
	*/
	private Data data;

	/**
		Optional <code>class</code> attribute
	*/
	private String clazz = "cell";
	
	/**
		Optional <code>style</code> attribute
	*/
	private String style = null;

	/**
		The ID of the column to which this cell belongs.
	*/
	private String columnId;

	/**
		If the cell should be visible or not.
	*/
	private boolean visible = true;
	
	/**
		The value to displaye.
	*/
	private Object value = null;

	private Formatter formatter;
	/*
		Taglib initialization methods
	*/
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	public String getClazz()
	{
		return clazz;
	}

	public void setStyle(String style)
	{
		this.style = style;
	}
	public String getStyle()
	{
		return style;
	}
	
	public void setColumn(String columnId)
	{
		this.columnId = columnId;
	}
	public String getColumn()
	{
		return columnId;
	}

	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}
	
	public void setValue(Object value)
	{
		this.value = value;
	}
	
	public Object getValue()
	{
		return value;
	}

	public Formatter<?> getFormatter()
	{
		return formatter;
	}
	
	/*
		Display methods
	*/
	public int doStartTag()
		throws JspException
	{
		if (!(getParent() instanceof Row)) throw new JspTagException("Tag <tbl:cell> must be inside a <tbl:row> tag");
		data = ((Row)getParent()).getData();
		table = data.getTable();
		if (!isVisible() || !table.isColumnVisible(columnId)) return SKIP_BODY;
		formatter = table.getFormatter(columnId);
		return EVAL_BODY_BUFFERED;
	}

	@SuppressWarnings("unchecked")
	public int doEndTag()
		throws JspException
	{
		if (!isVisible()) return EVAL_PAGE;
		String content = bodyContent == null ? "" : bodyContent.getString();
		if (content == null || content.equals("")) content = value == null ? "&nbsp;" : "";

		StringBuilder sb = new StringBuilder();
		sb.append("\t<td");
		if (getClazz() != null) sb.append(" class=\"").append(getClazz()).append("\"");
		if (getStyle() != null) sb.append(" style=\"").append(getStyle()).append("\"");
		sb.append(">");
		if (value != null)
		{
			sb.append(formatter == null ? value.toString() : formatter.format(value));
		}
		sb.append(content).append("</td>\n");

		table.setColumnContent(columnId, sb.toString());
		return EVAL_PAGE;
	}

}


