/**
	$Id: Affymetrix.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.filehandler.CdfFileHandler;
import net.sf.basedb.core.filehandler.CelFileHandler;
import net.sf.basedb.core.signal.ThreadSignalHandler;
import affymetrix.fusion.cdf.FusionCDFData;
import affymetrix.fusion.cel.FusionCELData;

/**
	Utility class for Affymetrix-specific functionality. Use this class
	to link CDF files with array designs and CEL and CHP files with raw 
	bioassays.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class Affymetrix
{
	
	/**
		The name of the {@link AnyToAny} link between an array design 
		and a file that contains the CDF information.
		@deprecated See {@link DataFileType#AFFYMETRIX_CDF}
	*/
	public static final String CDF_LINK_NAME = "cdf-file";
	
	/**
		The name of the {@link AnyToAny} link between a raw bioassay
		and a file that contains the CEL information.
		@deprecated See {@link DataFileType#AFFYMETRIX_CEL}
	*/
	public static final String CEL_LINK_NAME = "cel-file";

	/**
		The name of the {@link AnyToAny} link between a raw bioassay
		and a file that contains the CHP information.
		@deprecated See {@link DataFileType}
	*/
	public static final String CHP_LINK_NAME = "chp-file";

	/**
		Attach a CDF file to an array design. When an array design
		has got a CDF file it counts as if it has features and it can be used
		for linking with {@link RawBioAssay}:s.
		<p>
		Starting with BASE 2.4 this method validates that
		the specified file is actaully a CDF file. See {@link #loadCdfFile(File)}.

		@param design The array design 
		@param file The CDF file, or null to remove the file
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission for the design and use permission for the
			file
		@throws InvalidDataException If the design is null or isn't an
			Affymetrix chip or if the file is not a CDF file
		@throws BaseException If there is another error
		@deprecated Use {@link FileStoreUtil#setDataFile(DbControl, FileStoreEnabled, String, File)}
			instead with {@link DataFileType#AFFYMETRIX_CDF} as the file
			type
	*/
	public static void setCdfFile(ArrayDesign design, File file)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (design == null) throw new InvalidUseOfNullException("design");
		if (!design.isPlatform(Platform.AFFYMETRIX)) 
		{
			throw new InvalidDataException("Array design isn't an Affymetrix chip: " + design);
		}
		DbControl dc = design.getDbControl();
		FileStoreUtil.setDataFile(dc, design, DataFileType.AFFYMETRIX_CDF, file);
		if (file != null) design.getFileSet().validate(dc, true);
	}
	
	/**
		Get the file that has been specified as the CDF file for an
		array design. This method has been deprecated with the 
		implementation of the {@link Platform} and {@link DataFileType} 
		which solves the problem in a more generic way.
		<p>
		This method is equivalent to:
		<pre class="code">
DataFileType cdfType = 
   DataFileType.getByExternalId(dc, DataFileType.AFFYMETRIX_CDF);
File cdfFile = design.getFileSet().getMember(cdfType.getFile());
</pre>
		
		@param design The design to get the CDF file from, it must
			be an Affymetrix chip
		@return The CDF file or null if no CDF file has been specified
		@throws PermissionDeniedException If the logged in user doesn't
			have read permission for the file
		@throws InvalidDataException If the design is null or isn't an
			Affymetrix chip
		@throws BaseException If there is another error
		@deprecated Use {@link FileStoreUtil#getDataFile(DbControl, FileStoreEnabled, String)}
			with a data file type of {@link DataFileType#AFFYMETRIX_CDF}
			instead
	*/
	public static File getCdfFile(ArrayDesign design)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (design == null) throw new InvalidUseOfNullException("design");
		if (!design.isPlatform(Platform.AFFYMETRIX)) 
		{
			throw new InvalidDataException("Array design isn't an Affymetrix chip: " + design.getName());
		}
		DbControl dc = design.getDbControl();
		return FileStoreUtil.getDataFile(dc, design, DataFileType.AFFYMETRIX_CDF);
	}

	/**
		Attach a CEL file to a raw bioassay. When a raw bioassay has got a CEL 
		file it counts as if data has been loaded and it can be used
		in experiments for analysis. To be able to add a CEL file the
		array design must also be connected to an array design with 
		a CDF file. See {@link #setCdfFile(ArrayDesign, File)}.
		<p>
		Starting with BASE 2.4 this method validates that the specified
		file is a CEL file and that it matches the CDF file attached to
		the array design. The check includes:

		<ul>
		<li>That the CEL chip type mathces the file name of the CDF file.
		<li>That the number of rows and columns are the same in both the
			CEL and the CDF
		</ul>
				
		<p>
		This method also sets the number of spots to the number of cells
		found in the CEL file.
	
		@param rawBioAssay The raw bioassay 
		@param file The CEL file, or null to remove the file
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission for the raw bioassay and use permission for the
			file
		@throws InvalidDataException If the raw bioassay is null or isn't an Affymetrix
			raw data type
		@throws BaseException If there is another error
		@deprecated Use {@link FileStoreUtil#setDataFile(DbControl, FileStoreEnabled, String, File)}
			instead with {@link DataFileType#AFFYMETRIX_CEL} as the file
			type
	*/
	public static void setCelFile(RawBioAssay rawBioAssay, File file)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (rawBioAssay == null) throw new InvalidUseOfNullException("rawBioAssay");
		rawBioAssay.checkPermission(Permission.WRITE);

		if (!rawBioAssay.isPlatform(Platform.AFFYMETRIX)) 
		{
			throw new InvalidDataException("Raw bioassay isn't an Affymetrix chip: " + rawBioAssay.getName());
		}
		DbControl dc = rawBioAssay.getDbControl();
		FileStoreUtil.setDataFile(dc, rawBioAssay, DataFileType.AFFYMETRIX_CEL, file);
		if (file != null) rawBioAssay.getFileSet().validate(dc, true);
	}
	
	/**
		Get the file that has been specified as the CEL file for a
		raw bioassay.
		
		@param rawBioAssay The raw bioassay to get the CEL file from, it must
			be a raw bioassay of an Affymetrix raw data type
		@return The CEL file or null if no CEL file has been specified
		@throws PermissionDeniedException If the logged in user doesn't
			have read permission for the file
		@throws InvalidDataException If the raw bioassay is null or isn't an
			Affymetrix raw data type
		@throws BaseException If there is another error
		@deprecated
	*/
	public static File getCelFile(RawBioAssay rawBioAssay)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (rawBioAssay == null) throw new InvalidUseOfNullException("rawBioAssay");
		if (!rawBioAssay.isPlatform(Platform.AFFYMETRIX)) 
		{
			throw new InvalidDataException("Raw data type for " + rawBioAssay.getName() + 
				" is not an Affymetrix raw data type");
		}
		DbControl dc = rawBioAssay.getDbControl();
		return FileStoreUtil.getDataFile(dc, rawBioAssay, DataFileType.AFFYMETRIX_CEL);
	}
	
	/**
		Check if the CEL file on the raw bioassay matches the CDF file on the
		array design.
		
		@param rawBioAssay The raw bioassay
		@param design The array design
		@param required TRUE if the absense of either a CEL or a CDF file is an
			error condition
		@throws InvalidDataException If the raw bioassay or array design are not
			Affymetrix type, or if the CEL doesn't match the CDF
		@since 2.4
	*/
	public static void validateCelAndCdf(RawBioAssay rawBioAssay, ArrayDesign design, boolean required)
	{
		DbControl dc = rawBioAssay.getDbControl();
		File celFile = FileStoreUtil.getDataFile(dc, rawBioAssay, DataFileType.AFFYMETRIX_CEL);;
		File cdfFile = FileStoreUtil.getDataFile(dc, design, DataFileType.AFFYMETRIX_CDF);
		if (celFile != null && cdfFile != null)
		{
			String cdfChipType = cdfFile.getName();
			FusionCELData cel = new CelFileHandler().loadCelFile(celFile);
			FusionCDFData cdf = new CdfFileHandler().loadCdfFile(cdfFile);
			validateCelAndCdf(cel, cdf, cdfChipType);
		}
		else if (required)
		{
			if (celFile == null)
			{
				throw new InvalidDataException("Raw bioassay has no CEL file: " + rawBioAssay);
			}
			else
			{
				throw new InvalidDataException("Array design has no CDF file: " + design);				
			}
		}
	}
	
	private static void validateCelAndCdf(FusionCELData cel, FusionCDFData cdf, String cdfChipType)
		throws InvalidDataException
	{
		String celChipType = cel.getChipType();
		if (!cdfChipType.startsWith(celChipType))
		{
			throw new InvalidDataException("CEL chip type (" + celChipType + 
				") doesn't match CDF chip type (" + cdfChipType + ")");
		}
		
		if (cel.getRows() != cdf.getHeader().getRows() || cel.getCols() != cdf.getHeader().getRows())
		{
			throw new InvalidDataException("CEL size (rows=" + cel.getRows() + ", cols=" + cel.getCols() +
				") doesn't match CDF size (rows=" + 
				cdf.getHeader().getRows() + ", cols=" + cdf.getHeader().getCols()+")");
		}		
	}
	
	/**
		Load a CDF file using the Affymetric Fusion SDK. This method
		checks that the file exists, and read all headers.
		
		@param cdfFile The file to load
		@return A FusionCDFData object representing the CDF file
		@throws ItemNotFoundException If the actaul file is not on the server
		@throws InvalidDataException If the file is not a CDF file
		@since 2.4
		@deprecated Use {@link CdfFileHandler#loadCdfFile(File)} instead
	*/
	public static FusionCDFData loadCdfFile(File cdfFile)
		throws ItemNotFoundException, InvalidDataException
	{
		return new CdfFileHandler().loadCdfFile(cdfFile);
	}
	
	/**
		Load a CEL file using the Affymetric Fusion SDK. This method
		checks that the file exists, and read all headers.
		
		@param celFile The file to load
		@return A FusionCELData object representing the CEL file
		@throws ItemNotFoundException If the actaul file is not on the server
		@throws InvalidDataException If the file is not a CEL file
		@since 2.4
		@deprecated Use {@link CdfFileHandler#loadCdfFile(File)} instead
	*/
	public static FusionCELData loadCelFile(File celFile)
		throws InvalidDataException
	{
		return new CelFileHandler().loadCelFile(celFile);
	}
	
	/**
		Verify that probesets in a CDF file exists as reporters and
		optionally create those that doesn't exists. This method will clear
		the CDF file, read it, and then clear it again. The calling code is responsible
		for committing the transaction.
		
		@param cdf The CDF file
		@param batcher A reporter batcher used to check and insert reporters
		@param create TRUE to create non-existing reporters, FALSE to throw
			an ItemNotFoundException
		@param type The reporter type to assign to new reporters (if create is true)
		@param progress An optional progress reporter 
		@return The number of created reporter
		@throws ItemNotFoundException If create is FALSE and a reporter isn't found
		@throws BaseException If there is another error
		@since 2.4
	*/
	public static int checkIfReportersExists(FusionCDFData cdf, ReporterBatcher batcher,
		boolean create, ReporterType type, ProgressReporter progress)
		throws ItemNotFoundException, BaseException
	{
		if (batcher == null) throw new InvalidUseOfNullException("batcher");
		if (cdf == null) throw new InvalidUseOfNullException("cdf");

		final boolean hasType = type != null;
		final boolean hasProgress = progress != null;
		int numCreated = 0;
		
		if (hasProgress) progress.display(5, "Reading CDF file...");
		if (!cdf.read())
		{
			throw new BaseException("Error reading CDF file: " + cdf.getError());
		}
		int numProbesets = cdf.getHeader().getNumProbeSets();
		int progressInterval = numProbesets / 50;
		if (progressInterval < 10) progressInterval = 10;
		
		int index = 0;
		while (index < numProbesets)
		{
			ThreadSignalHandler.checkInterrupted();
			String probesetId = cdf.getProbeSetName(index);
			if (!batcher.exists(probesetId, true, true))
			{
				if (create)
				{
					ReporterData reporter = Reporter.getNew(probesetId);
					if (hasType) Reporter.setReporterType(reporter, type);
					reporter.setName(probesetId);
					batcher.insert(reporter);
					++numCreated;
				}
				else
				{
					throw new ItemNotFoundException("Reporter[externalId=" + probesetId+"]");
				}
			}
			++index;
			if (hasProgress && (index % progressInterval == 0))
			{
				int percent = 5 + (int)(90L * (long)index / numProbesets);
				int remain = numProbesets - index;
				String created = create ? "; " + numCreated + " created" : "";
				String msg = "Checking probsets (" + index + " done" + created + "; " + remain + " remaining)...";
				progress.display(percent, msg);
			}
		}
		batcher.flush();
		cdf.clear();
		if (hasProgress) progress.display(100, "Checked: " + numProbesets + "; created: " + numCreated);
		return numCreated;
	}
		
}
