/*
	$Id: AnnotationType.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ParameterValueData;
import net.sf.basedb.core.data.AnnotationTypeData;

import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.info.AnnotationTypeInfo;
import net.sf.basedb.info.ToTransferable;
import net.sf.basedb.util.formatter.Formatter;

import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.Collections;

/**
	Objects of this class defines an annotation type. An
	annotation type is always of a specific {@link Type},
	ie. a string, integer, etc. Limitations on the values can be specified, ie.
	min and max values, max string length, etc. It is also possible
	to use a predefined list of allowed values, ie an enumeration.
	<p>
	An annotation type also specifies which types of items it can
	be used on.

	@author Nicklas
	@version 2.0
	@see Annotatable
	@see Annotation
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class AnnotationType
	extends CommonItem<AnnotationTypeData>
	implements ToTransferable<AnnotationTypeInfo>
{
	/**
		The type of item represented by this class.
		@see Item#ANNOTATIONTYPE
		@see #getType()
	*/
	public static final Item TYPE = Item.ANNOTATIONTYPE;

	/**
		The maximum length of the external id that can be stored in the database.
		@see #setExternalId(String)
		@since 2.2
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = AnnotationTypeData.MAX_EXTERNAL_ID_LENGTH;

	
	/**
		Create a new <code>AnnotationType</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param valueType The type of values to use
		@return The new <code>AnnotationType</code> item
		@throws BaseException If there is an error
	*/
	public static AnnotationType getNew(DbControl dc, Type valueType)
		throws BaseException
	{
		if (valueType == null) throw new InvalidUseOfNullException("valueType");
		AnnotationType at = dc.newItem(AnnotationType.class);
		at.setName("New "+valueType.toString().toLowerCase()+" annotation type");
		at.getData().setValueType(valueType.getValue());
		at.setMultiplicity(1);
		at.setRequiredForMiame(false);
		at.setHeight(valueType == Type.TEXT ? 4 : 1);
		at.setWidth(40);
		return at;
	}

	/**
		Get an <code>AnnotationType</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param id The id of the item to load
		@return The <code>AnnotationType</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static AnnotationType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		AnnotationType at = dc.loadItem(AnnotationType.class, id);
		if (at == null) throw new ItemNotFoundException("AnnotationType[id="+id+"]");
		return at;
	}

	/**
		Get a {@link ItemQuery} that returns annotation types. If the itemType
		parameter is null, annotation types for all item types will be returned
		
		@param itemType Optional, limit the annotation types to those
			that are enabled for the specified item
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<AnnotationType> getQuery(Item itemType)
	{
		ItemQuery<AnnotationType> query = new ItemQuery<AnnotationType>(AnnotationType.class);
		if (itemType != null)
		{
			query.joinPermanent(Hql.innerJoin("itemTypes", "itemType"));
			query.restrictPermanent(
				Restrictions.eq(
					Hql.alias("itemType"),
					Expressions.integer(itemType.getValue())
				)
			);
		}
		return query;
	}

	AnnotationType(AnnotationTypeData annotationTypeData)
	{
		super(annotationTypeData);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Checks if:
		<ul>
		<li>Annotations of this type exists
		<li>Experiments are using this annotation type as an experimental factor
		<li>Protocols are using this annotation type as a protocol parameter
		</ul>
	*/
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		boolean used = false;
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_ANNOTATIONS_FOR_ANNOTATIONTYPE", "count(*)");
			/*
				SELECT {1}
				FROM AnnotationData a
				WHERE a.annotationType = :annotationType
			*/
			query.setEntity("annotationType", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_EXPERIMENTS_FOR_ANNOTATIONTYPE", "count(*)");
			/*
				SELECT {1}
				FROM ExperimentData exp
				JOIN exp.experimentalFactors ef
				WHERE ef = :annotationType
			*/
			query.setEntity("annotationType", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_PROTOCOLS_FOR_ANNOTATIONTYPE", "count(*)");
			/*
				SELECT {1}
				FROM ProtocolData prt
				JOIN prt.parameters pp
				WHERE pp = :annotationType
			*/
			query.setEntity("annotationType", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		return used || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>Annotations of this type
		<li>Experiments that are using this annotation type as an experimental factor
		<li>Protocols that are using this annotation type as a protocol parameter
		<ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Annotations
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_ANNOTATIONS_FOR_ANNOTATIONTYPE", "a.id");
			/*
				SELECT {1}
				FROM AnnotationData a
				WHERE a.annotationType = :annotationType
			*/
		query.setEntity("annotationType", this.getData());
		addUsingItems(using, Item.ANNOTATION, query);
		
		// Experimental factors
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_EXPERIMENTS_FOR_ANNOTATIONTYPE", "exp.id");
			/*
				SELECT {1}
				FROM ExperimentData exp
				JOIN exp.experimentalFactors ef
				WHERE ef = :annotationType
			*/
		query.setEntity("annotationType", this.getData());
		addUsingItems(using, Item.EXPERIMENT, query);

		// Protocol parameters
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PROTOCOLS_FOR_ANNOTATIONTYPE", "prt.id");
			/*
				SELECT {1}
				FROM ProtocolData prt
				JOIN prt.parameters pp
				WHERE pp = :annotationType
			*/
		query.setEntity("annotationType", this.getData());
		addUsingItems(using, Item.PROTOCOL, query);

		return using;
	}
	
	public AnnotationTypeInfo toTransferable(AnnotationTypeInfo info)
	{
		super.toTransferable(info);
		info.setDefaultValue(getDefaultValue());
		info.setExternalId(getExternalId());
		info.setEnumeration(isEnumeration());
		if (isEnumeration())
		{
			info.setEnumValues(net.sf.basedb.util.Values.toStrings(getValues(), null));
		}
		info.setMultiplicity(getMultiplicity());
		info.setProtocolParameter(isProtocolParameter());
		info.setRequiredForMiame(isRequiredForMiame());
		info.setValueType(getValueType().name());
		info.setItemTypes(net.sf.basedb.util.Values.toStrings(getEnabledItems(), 
			new Formatter<Item>()
			{
				public String format(Item item)
				{
					return item.name();
				}
				public Item parseString(String value)
				{
					return Item.valueOf(value);
				}
			}
		));
		return info;
	}
	
	// -------------------------------------------

	/**
		Get the external id of the annotation type. This value can be used to link
		with information in external databases. It is not used by the BASE core
		and it doesn't have to be unique.
		@return The external id for the annotation type
		@since 2.2
	*/
	public String getExternalId()
	{
		return getData().getExternalId();
	}
	/**
		Set the external id of the annotation type. The value
		may be null but must not be longer than the value specified by
		the {@link #MAX_EXTERNAL_ID_LENGTH} constant.
		@param externalId The new value for the external id
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@throws InvalidDataException If the string is too long
		@since 2.2
	*/
	public void setExternalId(String externalId)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setExternalId(StringUtil.setNullableString(externalId, "externalId",
			MAX_EXTERNAL_ID_LENGTH));
	}

	
	/**
		Add a category to this annotation type. 
		@param category The category to add
		@throws PermissionDeniedException If the logged in user doesn't have write 
			permission for the annotation type and use permission for the category
		@throws InvalidDataException If the category is null
	*/
	public void addCategory(AnnotationTypeCategory category)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (category == null) throw new InvalidUseOfNullException("category");
		category.checkPermission(Permission.USE);
		getData().getCategories().add(category.getData());
	}
	
	/**
		Remove a category from this annotation type.
		@param category The category to remove
		@throws PermissionDeniedException If the logged in user doesn't have write permission 
			for the annotation type
		@throws InvalidDataException If the category is null
	*/
	public void removeCategory(AnnotationTypeCategory category)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (category == null) throw new InvalidUseOfNullException("category");
		getData().getCategories().remove(category.getData());
	}

	/**
		Checks if the annotation type is a member of a category.
		@param category The category to check
		@return TRUE if the annotation type is a member, FALSE otherwise
	*/
	public boolean isMember(AnnotationTypeCategory category)
	{
		return getData().getCategories().contains(category.getData());
	}
	
	/** 
		Get a query that returns the categories an
		annotation type belongs to.
		@return An <code>ItemQuery<code> object
	*/
	public ItemQuery<AnnotationTypeCategory> getCategories()
	{
		ItemQuery<AnnotationTypeCategory> query= AnnotationTypeCategory.getQuery();
		query.joinPermanent(Hql.innerJoin("annotationTypes", Item.ANNOTATIONTYPE.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(Item.ANNOTATIONTYPE.getAlias()), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Get the value type for this annotation type. It can't be change once
		the object has been created.
	 	@return A {@link Type} constant
	*/
	public Type getValueType()
	{
		return Type.fromValue(getData().getValueType());
	}
	
	/**
		Get the multiplicity. The multiplicity tells the core how many values an annotation
		of this annotation type may contain. The default is one. Zero denotes unlimited
		values.
	 	@return the multiplicity as an int. 
	*/
	public int getMultiplicity()
	{
		return getData().getMultiplicity();
	}
	/**
		Set the multiplicity. The multiplicity tells the core how many values an annotation
		of this annotation type may contain. The default is one. Zero denotes unlimited
		values.
	  	@param multiplicity Multiplicity to set. 
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the multiplicity is lower than zero
	*/
	public void setMultiplicity(int multiplicity)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setMultiplicity(IntegerUtil.checkMin(multiplicity, "multiplicity", 0));
	}
	
	/**
		Check an annotation of this annotation type required for an item to
		be compliant with the MIAME recommendation.
	 	@return TRUE if required, FALSE otherwise. 
		@see <a href="http://www.mged.org/Workgroups/MIAME/miame.html">MIAME</a>
	*/
	public boolean isRequiredForMiame()
	{
		return getData().isRequiredForMiame();
	}
	/**
		Set if an annotation of this annotation type is required for an item to
		be compliant with the MIAME recommendation.
	  	@param required Should be TRUE if required or FALSE if it is not. 
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@see <a href="http://www.mged.org/Workgroups/MIAME/miame.html">MIAME</a>
	*/
	public void setRequiredForMiame(boolean required)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setRequiredForMiame(required);
	}
	
	/**
		Check if this annotation type provides a list of predefined allowed values.
	 	@return TRUE if it is provided, FALSE otherwise. 
	*/
	public boolean isEnumeration()
	{
		return getData().isEnumeration();
	}
	/**
		Set if this annotation type should provide a list of predefined allowed values.
	 	@param isEnumeration TRUE if this should be provided, FALSE otherwise 
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the value type doesn't support enumerations,
			see {@link Type#canEnumerate}
	*/
	public void setEnumeration(boolean isEnumeration)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (isEnumeration && !getValueType().canEnumerate())
		{
			throw new InvalidDataException(getValueType()+" annotation type doesn't support enumerations.");
		}
		getData().setEnumeration(isEnumeration);
	}

	/**
		Check if the enumeration values should be displayed as a selection list
		or as radio buttons/checkboxes by the client application. The client application
		may decide to use some other display format that better suits the situation.
	 	@return TRUE if it should be displayed as a selection list, 
	 		FALSE if radio buttons/check-boxes. 
	*/
	public boolean getDisplayAsList()
	{
		return getBooleanOption("displayAsList", true);
	}
	
	/**
		Set if the enumeration values should be displayed as a selection list
		or as radio buttons/checkboxes by the client application. The client application
		may decide to use some other display format that better suits the situation.
	 	@param displayAsList Set to TRUE to display as a selection list or 
	 		set to FALSE to display as radio buttons/check-boxes.
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException Never
	*/
	public void setDisplayAsList(boolean displayAsList)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		setOption("displayAsList", Boolean.valueOf(displayAsList).toString());
	}
	
	/**
		Get the recommended height in characters a client application should use
		to render an input field for annotations of this annotation type. A client
		application is not required to use this value if it is not appropriate for
		the situation.
	 	@return Characters height as an int. 
	*/
	public int getHeight()
	{
		return getData().getHeight();
	}
	/**
		Set the recommended height in characters a client application should use
		to render an input field for annotations of this annotation type. A client
		application is not required to use this value if it is not appropriate for
		the situation.
		@param height The height in characters
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the height is less than one
	*/
	public void setHeight(int height)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setHeight(IntegerUtil.checkMin(height, "height", 1));
	}

	/**
		Get the recommended width in characters a client application should use
		to render an input field for annotations of this annotation type. A client
		application is not required to use this value if it is not appropriate for
		the situation.
	 	@return The width as an int. 
	*/
	public int getWidth()
	{
		return getData().getWidth();
	}
	/**
		Set the recommended width in characters a client application should use
		to render an input field for annotations of this annotation type. A client
		application is not required to use this value if it is not appropriate for
		the situation.
		@param width The width in characters
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the width is less than one
	*/
	public void setWidth(int width)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setWidth(IntegerUtil.checkMin(width, "width", 1));
	}

	/**
		The maximum length of the default value.
	*/
	public static final int MAX_DEFAULT_VALUE_LENGTH = AnnotationTypeData.MAX_DEFAULT_VALUE_LENGTH;
	/**
		Get the recommended default value for new annotations of this annotation type. 
		It is not certain that the string represents a value that can be converted
		to the correct value type as specified by the {@link #getValueType()} setting.
	 	@return Default value as a String. 
	*/
	public String getDefaultValue()
	{
		return getData().getDefaultValue();
	}
	/**
		Set the recommended default value for new annotation of this annotation type.
		It is not checked that the string can be converted to the correct value type 
		as specified by the {@link #getValueType()} setting.
	 	@param defaultValue The default value for new annotation. 
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the default value is longer than
			{@link #MAX_DEFAULT_VALUE_LENGTH}
	*/
	public void setDefaultValue(String defaultValue)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setDefaultValue(StringUtil.setNullableString(defaultValue, "defaultValue", MAX_DEFAULT_VALUE_LENGTH));
	}

	/**
		Check if this annotation type can be used to annotate item of the specified type.
		@param itemType The {@link Item} type
		@return TRUE if annotations are possible, FALSE otherwise
	*/
	public boolean isEnabledForItem(Item itemType)
	{
		if (itemType == null) return false;
		return getData().getItemTypes().contains(itemType.getValue());
	}
	/**
		Enable this annotation type to be used for items of the specified type.
		@param itemType The {@link Item} type to enable this annotation type for
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the item type is null, or
			not represents an annotatable item
	*/
	public void enableForItem(Item itemType)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (itemType == null) throw new InvalidUseOfNullException("itemType");
		if (!Metadata.getAnnotatableItems().contains(itemType))
		{
			throw new InvalidDataException(itemType+" is not an annotatable item type.");
		}
		getData().getItemTypes().add(itemType.getValue());
	}
	/**
		Disable this annotation type to be used for items of the specified type.
		@param itemType The {@link Item} type to disable this annotation type for
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the item type is null
	*/
	public void disableForItem(Item itemType)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (itemType == null) throw new InvalidUseOfNullException("itemType");
		getData().getItemTypes().remove(itemType.getValue());
	}
	/**
		Get the {@link Item} types this annotation type is enabled for.
		@return A <code>Set</code> of <code>Item</code>:s
	*/
	public Set<Item> getEnabledItems()
	{
		Set<Integer> itemCodes = getData().getItemTypes();
		Set<Item> items = new HashSet<Item>(itemCodes.size());
		for (int itemCode : itemCodes)
		{
			items.add(Item.fromValue(itemCode));
		}
		return items;
	}
	
	/**
		If this annotation type is intended to be used as a protocol parameter
		or as a regular annotations.
		@return TRUE if the annotation type is a protocol parameter, FALSE if it
			is a regular annotation type
		@since 2.2
	*/
	public boolean isProtocolParameter()
	{
		return getData().isProtocolParameter();
	}
	
	/**
		Specify if this annotation type should be used as a protocol parameter
		or as a regular annotation type
		@param isProtocolParameter TRUE to use the annotation type as a protocol
			parameter, FALSE to use it as a regular annotation type
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@since 2.2
	*/
	public void setProtocolParameter(boolean isProtocolParameter)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setProtocolParameter(isProtocolParameter);
	}
	
	/**
		Check if this annotation type is a parameter to the specified protocol. NOTE!
		This method doesn't use the value from the {@link #isProtocolParameter()}.
		
		@param protocol The protocol to check, null always return false
		@return TRUE if the annotation type is a parameter to the protocol, FALSE otherwise
		@since 2.2
		@see Protocol#isParameter(AnnotationType)
	*/
	public boolean isProtocolParameter(Protocol protocol)
	{
		return protocol == null ? false : protocol.isParameter(this);
	}
	
	/**
		Get the minumum allowed value for an {@link Type#INT} or {@link Type#LONG}
		annotation.
		@return The minimum allowed value or null if none has been specfied or
			if this annotation type hasn't an <code>INT</code> or <code>LONG</code>
			value type
	*/
	public Long getMinValueLong()
	{
		return getValueType() == Type.INT || getValueType() == Type.LONG ? getLongOption("minValue", null) : null;
	}
	/**
		Set the minimum allowed value for an {@link Type#INT} or {@link Type#LONG} annotation.
		@param minValue The minimum allowed value or null if no limit is specified
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If this annotation type hasn't an <code>INT</code> or 
			<code>LONG</code> value type
	*/
	public void setMinValueLong(Long minValue)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (getValueType() != Type.INT && getValueType() != Type.LONG) 
		{
			throw new InvalidDataException("Invalid option for valueType="+getValueType());
		}
		setOption("minValue", minValue == null ? null : minValue.toString());
	}
	/**
		Get the maximum allowed value for an {@link Type#INT} or {@link Type#LONG} annotation.
		@return The maximum allowed value or null if none has been specfied or
			if this annotation type hasn't an <code>INT</code> or <code>LONG</code> value type
	*/
	public Long getMaxValueLong()
	{
		return getValueType() == Type.INT || getValueType() == Type.LONG ? getLongOption("maxValue", null) : null;
	}
	/**
		Set the maximum allowed value for an {@link Type#INT} or {@link Type#LONG} annotation.
		@param maxValue The maximum allowed value or null if no limit is specified
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If this annotation type hasn't an <code>INT</code> 
			value type
	*/
	public void setMaxValueLong(Long maxValue)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (getValueType() != Type.INT && getValueType() != Type.LONG) 
		{
			throw new InvalidDataException("Invalid option for valueType="+getValueType());
		}
		setOption("maxValue", maxValue == null ? null : maxValue.toString());
	}

	
	/**
		Get the minumum allowed value for a {@link Type#FLOAT} or {@link Type#DOUBLE} 
		annotation.
		@return The minimum allowed value or null if none has been specfied or
			if this annotation type hasn't a <code>FLOAT</code> or <code>DOUBLE</code>
			value type
	*/
	public Double getMinValueDouble()
	{
		return getValueType() == Type.FLOAT || getValueType() == Type.DOUBLE ? getDoubleOption("minValue", null) : null;
	}
	/**
		Set the minimum allowed value for a {@link Type#FLOAT} or {@link Type#DOUBLE} annotation.
		@param minValue The minimum allowed value or null if no limit is specified
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If this annotation type hasn't a <code>FLOAT</code> 
			or <code>DOUBLE</code> value type
	*/
	public void setMinValueDouble(Double minValue)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (getValueType() != Type.FLOAT && getValueType() != Type.DOUBLE)
		{
			throw new InvalidDataException("Invalid option for valueType="+getValueType());
		}
		setOption("minValue", minValue == null ? null : minValue.toString());
	}

	/**
		Get the maximum allowed value for a {@link Type#FLOAT} or {@link Type#DOUBLE} annotation.
		@return The maximum allowed value or null if none has been specfied or
			if this annotation type hasn't a <code>FLOAT</code> or <code>DOUBLE</code> 
			value type
	*/
	public Double getMaxValueDouble()
	{
		return getValueType() == Type.FLOAT || getValueType() == Type.DOUBLE ? getDoubleOption("maxValue", null) : null;
	}
	/**
		Set the maximum allowed value for a {@link Type#FLOAT} or {@link Type#DOUBLE} annotation.
		@param maxValue The maximum allowed value or null if no limit is specified
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If this annotation type hasn't a <code>FLOAT</code> 
			or <code>DOUBLE</code> value type
	*/
	public void setMaxValueDouble(Double maxValue)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (getValueType() != Type.FLOAT && getValueType() != Type.DOUBLE)
		{
			throw new InvalidDataException("Invalid option for valueType="+getValueType());
		}
		setOption("maxValue", maxValue == null ? null : maxValue.toString());
	}

	/**
		Get the maximum allowed string length for a {@link Type#STRING} annotation.
		@return The maximum allowed string length or null if none has been specfied or
			if this annotation type hasn't an <code>STRING</code> value type
	*/
	public Integer getMaxLength()
	{
		return getValueType() == Type.STRING ? getIntOption("maxLength", null) : null;
	}
	/**
		Set the maximum allowed string length for a {@link Type#STRING} annotation.
		@param maxLength The maximum allowed value or null if no limit is specified
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If this annotation type hasn't a <code>STRING</code> 
			value type
	*/
	public void setMaxLength(Integer maxLength)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (getValueType() != Type.STRING) throw new InvalidDataException("Invalid option for valueType="+getValueType());
		if (maxLength != null && (maxLength > 255 || maxLength < 1))
		{
			throw new NumberOutOfRangeException("maxLength", maxLength, 1, 255);
		}
		setOption("maxLength", maxLength == null ? null : maxLength.toString());
	}
	
	/**
		Get the list of allowed values if this annotation type is an enumeration.
		@return A <code>List</code> with the allowed values or null if this annotation type
			is not an enumeration or no values has been specified
		@see #isEnumeration()
		@throws BaseException If there is an error
	*/
	public List<?> getValues()
		throws BaseException
	{
		if (!isEnumeration()) return null;
		ParameterValueData<?> pv = getData().getEnumerationValues();
		return pv == null ? null : Collections.unmodifiableList(Values.getItemValues(getDbControl(), pv.getValues()));
	}
	/**
		Set the list of allowed values for an enumerated annotation type.
		@param values The list of allowed values, or null to clear the values
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the list contains values of the wrong type
			as checked by the {@link Type#validate(List)} method
	*/
	public void setValues(List<?> values)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (!isEnumeration()) throw new InvalidDataException("Invalid option for non-enumerated type");
		Type valueType = getValueType();
		if (values != null) valueType.validate(values);
		ParameterValueData<?> pv = getData().getEnumerationValues();
		if (pv == null)
		{
			pv = valueType.newParameterValueData();
			getData().setEnumerationValues(pv);
		}
		pv.replaceValues(Values.getDataValues(values));
	}
	
	/**
		This method should be used on enumerated string annotation types to find a
		matching predefined string value, optionally ignoring case and white-space.
		
		@param value The value to look for
		@param ignoreCase TRUE to ignore case
		@param trim TRUE to trim white-space from the start and end of string
		@return The matching string, or null if not found or if the annotation
			isn't an enumeration of strings
		@since 2.4
	*/
	@SuppressWarnings("unchecked")
	public String findValue(String value, boolean ignoreCase, boolean trim)
	{
		if (value == null || !isEnumeration() || getValueType() != Type.STRING)
		{
			return null;
		}
		List<String> values = (List<String>)getData().getEnumerationValues().getValues();
		if (trim) value = value.trim();
		for (String enumValue : values)
		{
			String compareTo = trim ? enumValue.trim() : enumValue;
			if ((ignoreCase && value.equalsIgnoreCase(compareTo))
				|| value.equals(compareTo)) 
			{
				return enumValue;
			}
		}
		return null;
	}
	
	/**
		Get an option.
	*/
	String getOption(String name)
	{
		return getData().getOptions().get(name);
	}
	/**
		Get an option and convert the value to an integer. If the option
		doesn't exist or the value can't be converted, the default value is
		returned.
	*/
	Integer getIntOption(String name, Integer defaultValue)
	{
		String option = getOption(name);
		if (option == null) return defaultValue;
		try
		{
			return Integer.valueOf(option);
		}
		catch (Exception ex)
		{
			return defaultValue;
		}
	}
	/**
		Get an option and convert the value to a long. If the option
		doesn't exist or the value can't be converted, the default value is
		returned.
	*/
	Long getLongOption(String name, Long defaultValue)
	{
		String option = getOption(name);
		if (option == null) return defaultValue;
		try
		{
			return Long.valueOf(option);
		}
		catch (Exception ex)
		{
			return defaultValue;
		}
	}
	/**
		Get an option and convert the value to a float. If the option
		doesn't exist or the value can't be converted, the default value is
		returned.
	*/
	Float getFloatOption(String name, Float defaultValue)
	{
		String option = getOption(name);
		if (option == null) return defaultValue;
		try
		{
			return Float.valueOf(option);
		}
		catch (Exception ex)
		{
			return defaultValue;
		}
	}
	/**
		Get an option and convert the value to a double. If the option
		doesn't exist or the value can't be converted, the default value is
		returned.
	*/
	Double getDoubleOption(String name, Double defaultValue)
	{
		String option = getOption(name);
		if (option == null) return defaultValue;
		try
		{
			return Double.valueOf(option);
		}
		catch (Exception ex)
		{
			return defaultValue;
		}
	}
	/**
		Get an option and convert the value to a boolean. If
		the option doesn't exist the default value is returned. 
		If the option contains the string "true" (case ignored)
		true is returned, false otherwise.
	*/
	boolean getBooleanOption(String name, boolean defaultValue)
	{
		String option = getOption(name);
		if (option == null) return defaultValue;
		return Boolean.parseBoolean(option);
	}

	/**
		The maximum length of an option.
	*/
	public static final int MAX_OPTION_LENGTH = AnnotationTypeData.MAX_OPTION_LENGTH;
	/**
		Set an option.
	*/
	void setOption(String name, String value)
		throws InvalidDataException
	{
		if (value == null)
		{
			getData().getOptions().remove(name);
		}
		else
		{
			if (value.length() > MAX_OPTION_LENGTH) throw new StringTooLongException(name, value, MAX_OPTION_LENGTH);
			getData().getOptions().put(name, value);
		}
	}
	
	/**
		Validate an annotation value. We check the following:
		<ul>
		<li>The object is of the correct type, ie. <code>Float</code> if {@link #getValueType()} 
			returns {@link Type#FLOAT}, etc.
		<li>The value is within the min and max bouds if specified
		<li>A string value is not too long
		<li>The value exists in the predefined list if this annotation type
			is an enumeration
		</ul>
	 	@param value The object to be validated 
	 	@throws InvalidDataException If the value object can not be used.
	*/
	public void validateAnnotationValue(Object value)
		throws InvalidDataException
	{
		if (value == null) throw new InvalidUseOfNullException("value");
		Type valueType = getValueType();
		valueType.validate(value);
		if (valueType == Type.INT)
		{
			Integer intValue = (Integer)value;
			Integer minValue = getIntOption("minValue", null);
			Integer maxValue = getIntOption("maxValue", null);
			if ((minValue != null && intValue.compareTo(minValue) < 0)
				||
				(maxValue != null && intValue.compareTo(maxValue) > 0))
			{
				if (minValue != null && maxValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), intValue.intValue(), minValue.intValue(), maxValue.intValue());
				}
				else if (minValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), intValue.intValue(), minValue.intValue(), false);
				}
				else if (maxValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), intValue.intValue(), maxValue.intValue(), true);
				}
			}
		}
		else if (valueType == Type.LONG)
		{
			Long longValue = (Long)value;
			Long minValue = getLongOption("minValue", null);
			Long maxValue = getLongOption("maxValue", null);
			if ((minValue != null && longValue.compareTo(minValue) < 0)
				||
				(maxValue != null && longValue.compareTo(maxValue) > 0))
			{
				if (minValue != null && maxValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), longValue.longValue(), minValue.longValue(), maxValue.longValue());
				}
				else if (minValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), longValue.longValue(), minValue.longValue(), false);
				}
				else if (maxValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), longValue.longValue(), maxValue.longValue(), true);
				}
			}
		}
		else if (valueType == Type.FLOAT)
		{
			Float floatValue = (Float)value;
			Float minValue = getFloatOption("minValue", null);
			Float maxValue = getFloatOption("maxValue", null);
			if ((minValue != null && floatValue.compareTo(minValue) < 0)
				||
				(maxValue != null && floatValue.compareTo(maxValue) > 0))
			{
				if (minValue != null && maxValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), floatValue.floatValue(), minValue.floatValue(), maxValue.floatValue());
				}
				else if (minValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), floatValue.floatValue(), minValue.floatValue(), false);
				}
				else if (maxValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), floatValue.floatValue(), maxValue.floatValue(), true);
				}
			}
		}
		else if (valueType == Type.DOUBLE)
		{
			Double doubleValue = (Double)value;
			Double minValue = getDoubleOption("minValue", null);
			Double maxValue = getDoubleOption("maxValue", null);
			if ((minValue != null && doubleValue.compareTo(minValue) < 0)
				||
				(maxValue != null && doubleValue.compareTo(maxValue) > 0))
			{
				if (minValue != null && maxValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), doubleValue.doubleValue(), minValue.doubleValue(), maxValue.doubleValue());
				}
				else if (minValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), doubleValue.doubleValue(), minValue.doubleValue(), false);
				}
				else if (maxValue != null)
				{
					throw new NumberOutOfRangeException(this.getName(), doubleValue.doubleValue(), maxValue.doubleValue(), true);
				}
			}
		}
		else if (valueType == Type.STRING)
		{
			String stringValue = (String)value;
			Integer maxLength = getIntOption("maxLength", 255);
			if (stringValue.length() > maxLength)
			{
				throw new StringTooLongException(this.getName(), stringValue, maxLength);
			}
		}
		if (isEnumeration())
		{
			ParameterValueData<?> pv = getData().getEnumerationValues();
			if (pv == null || !pv.getValues().contains(value))
			{
				throw new DisallowedValueException("Value '"+value+"' is not among the list of allowed values " +
					"for annotation type: "+getName());
			}
		}
	}
	
}
