/*
	$Id: BioMaterialEvent.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;

import net.sf.basedb.core.data.BioMaterialEventData;
import net.sf.basedb.core.data.MeasuredBioMaterialData;
import net.sf.basedb.core.data.UserData;
import net.sf.basedb.core.data.UsedQuantity;
import net.sf.basedb.core.data.SharedData;

import java.util.Map;
import java.util.HashMap;
import java.util.Date;
import java.util.Iterator;

/**
	This class represents an event in the life of a {@link MeasuredBioMaterial}. 
	There are three main types of events: creation events,
	hybridization events and other events. A creation event is an event that
	produces a measured biomaterial from other biomaterials. A hybridization event 
	is an event that produces a hybridization from labeled extracts. Other events 
	are used to leave a note about something that affected an already existing biomaterial.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class BioMaterialEvent
	extends ChildItem<BioMaterialEventData>
{

	/**
		The type of item represented by this class.
		@see Item#BIOMATERIALEVENT
		@see #getType()
	*/
	public static final Item TYPE = Item.BIOMATERIALEVENT;

	/**
		Create a new <code>BioMaterialEvent</code> of the {@link Type#OTHER} type.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param bioMaterial The affected biomaterial
		@return The new <code>BioMaterialEvent</code> item
		@throws BaseException If there is an error
	*/
	public static BioMaterialEvent getNew(DbControl dc, MeasuredBioMaterial bioMaterial)
		throws BaseException
	{
		BioMaterialEvent bme = dc.newItem(BioMaterialEvent.class);
		bme.setBioMaterial(bioMaterial);
		bme.getData().setEntryDate(new Date());
		bme.getData().setEventDate(new Date());
		bme.getData().setEventType(Type.OTHER.getValue());
		org.hibernate.Session session = dc.getHibernateSession();
		SessionControl sc = dc.getSessionControl();
		bme.getData().setUser(HibernateUtil.loadData(session, UserData.class, sc.getLoggedInUserId()));
		return bme;
	}
	
	/**
		Create a new <code>BioMaterialEvent</code> of the {@link Type#CREATION}
		type (creation event).
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param bioMaterial The biomaterial that was created.
		@return The new <code>BioMaterialEvent</code> item
		@throws BaseException If there is an error
	*/
	static BioMaterialEvent getNewCreationEvent(DbControl dc, MeasuredBioMaterial bioMaterial)
		throws BaseException
	{
		BioMaterialEvent bme = dc.newItem(BioMaterialEvent.class);
		bme.setBioMaterial(bioMaterial);
		bme.getData().setEntryDate(new Date());
		bme.getData().setEventDate(new Date());
		bme.getData().setEventType(Type.CREATION.getValue());
		org.hibernate.Session session = dc.getHibernateSession();
		SessionControl sc = dc.getSessionControl();
		bme.getData().setUser(HibernateUtil.loadData(session, UserData.class, sc.getLoggedInUserId()));
		return bme;
	}
	
	/**
		Create a new <code>BioMaterialEvent</code> of the {@link Type#HYBRIDIZATION}
		type.
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param hybridization The hybridization that was created.
		@return The new <code>BioMaterialEvent</code> item
		@throws BaseException If there is an error
	*/
	static BioMaterialEvent getNewHybridizationEvent(DbControl dc, Hybridization hybridization)
		throws BaseException
	{
		BioMaterialEvent bme = dc.newItem(BioMaterialEvent.class);
		bme.setHybridization(hybridization);
		bme.getData().setEntryDate(new Date());
		bme.getData().setEventDate(new Date());
		bme.getData().setEventType(Type.HYBRIDIZATION.getValue());
		org.hibernate.Session session = dc.getHibernateSession();
		SessionControl sc = dc.getSessionControl();
		bme.getData().setUser(HibernateUtil.loadData(session, UserData.class, sc.getLoggedInUserId()));
		return bme;
	}


	/**
		Get a <code>BioMaterialEvent</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>BioMaterialEvent</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static BioMaterialEvent getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		BioMaterialEvent bme = dc.loadItem(BioMaterialEvent.class, id);
		if (bme == null) throw new ItemNotFoundException("BioMaterialEvent[id="+id+"]");
		return bme;
	}

	/**
		Get a query that returns events for a biomaterial. All events where the
		given biomaterial is either a source or a product are returned. 
	
		@param bioMaterial The biomaterial to retreive events for, null is not allowed
		@return An {@link ItemQuery} object
		@throws InvalidDataException If the <code>bioMaterial</code> is null
		@see MeasuredBioMaterial#getEvents()
	*/
	public static ItemQuery<BioMaterialEvent> getQuery(MeasuredBioMaterial bioMaterial)
		throws InvalidDataException
	{
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		// Create query, no runtime filter since read permission to a bioMaterial grants 
		// read permission to it's events (event other creation events)
		ItemQuery<BioMaterialEvent> query = 
			new ItemQuery<BioMaterialEvent>(BioMaterialEvent.class, null);
		query.setDistinct(true);  // Otherwise, the join to sources can result in one copy for each source
		query.joinPermanent(Hql.leftJoin("sources", "src"));
		query.restrictPermanent(
			Restrictions.or(
				Restrictions.eq(
					Hql.property("bioMaterial"), 
					Hql.entity(bioMaterial)
				),
				Restrictions.eq(
					Hql.index("src", null),
					Hql.entity(bioMaterial)
				)
			)
		);
		return query;
	}
	

	BioMaterialEvent(BioMaterialEventData bioMaterialEventData)
	{
		super(bioMaterialEventData);
	}
	
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Return used guantity to biomaterials on delete action.
	*/
	void onBeforeCommit(Transactional.Action action)
		throws BaseException
	{
		super.onBeforeCommit(action);
		if (action == Transactional.Action.DELETE)
		{
			setUsedQuantity(null);
		}
	}

	/**
		READ permission is granted and DELETE permissions is denied
		to evereyone for creation events. Other permissions are always 
		checked on the parent biomaterial or hybridization.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		switch (getEventType())
		{
			case CREATION:
			{
				granted |= Permission.grant(Permission.READ);
				denied |= Permission.deny(Permission.DELETE);
				break;
			}
			case HYBRIDIZATION:
			{
				granted |= Permission.grant(Permission.READ);
				denied |= Permission.deny(Permission.DELETE);
				break;
			}
			case OTHER:
			{
				break;
			}
			default:
			{
				throw new AssertionError("Invalid event type: "+getEventType());
			}
		}
		super.initPermissions(granted, denied);
	}
	// -------------------------------------------
	/*
		From the ChildItem class
		-------------------------------------------
	*/
	SharedData getSharedParent()
	{
		switch (getEventType())
		{
			case CREATION:
			{
				return getData().getBioMaterial();
			}
			case HYBRIDIZATION:
			{
				return getData().getHybridization();
			}
			case OTHER:
			{
				return getData().getBioMaterial();
			}
			default:
			{
				throw new AssertionError("Invalid event type: "+getEventType());
			}
		}
	}
	// -------------------------------------------

	/**
		Get the type of event.
		@see Type
	*/
	public Type getEventType()
	{
		return Type.fromValue(getData().getEventType());
	}
	
	/**
		Get the {@link MeasuredBioMaterial} that was affected or created by this event.
		@return A <code>MeasuredBioMaterial</code> item or null if this is 
			hybridization event
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the biomaterial
		@throws BaseException If there is another error
	*/
	public MeasuredBioMaterial getBioMaterial()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(MeasuredBioMaterial.class, getData().getBioMaterial());
	}
	/**
		Set the biomaterial that was affected or created by this event.
	*/
	private void setBioMaterial(MeasuredBioMaterial bioMaterial)
		throws PermissionDeniedException, InvalidDataException
	{
		// No write permission check since this is only called at object creation
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		bioMaterial.checkPermission(Permission.USE);
		getData().setBioMaterial((MeasuredBioMaterialData)bioMaterial.getData());
	}
	
	/**
		Get the {@link Hybridization} that was created by this event.
		@return A <code>Hybridization</code> item or null if this isn't a
			hybridization event
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the hybridization
		@throws BaseException If there is another error
	*/
	public Hybridization getHybridization()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Hybridization.class, getData().getHybridization());
	}

	/**
		Set the hybidization that was created by this event.
	*/
	private void setHybridization(Hybridization hybridization)
		throws PermissionDeniedException, InvalidDataException
	{
		// No write permission check since this is only called at object creation
		if (hybridization == null) throw new InvalidUseOfNullException("hybridization");
		hybridization.checkPermission(Permission.USE);
		getData().setHybridization(hybridization.getData());
	}

	/**
		Get the {@link User} that is responsible for this event.
		@return A <code>User</code> item 
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the user
		@throws BaseException If there is another error
	*/
	public User getUser()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(User.class, getData().getUser());
	}

	/**
		Get the {@link Protocol} describing the procedure used in this event.
		@return A <code>Protocol</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the protocol
		@throws BaseException If there is another error
	*/
	public Protocol getProtocol()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Protocol.class, getData().getProtocol());
	}

	/**
		Set the {@link Protocol} that describes the procedure used in this event.
		@param protocol A <code>Protocol</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event or use permission for the protocol
	*/
	public void setProtocol(Protocol protocol)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (protocol != null) protocol.checkPermission(Permission.USE);
		getData().setProtocol(protocol == null ? null : protocol.getData());
	}
	
	/**
		Get the {@link Hardware} that was used in this event.
	 	@return A <code>Hardware</code> item, or null if not known
	 	@throws PermissionDeniedException If the logged in user doesn't have 
	 		read permission to the hardware
	 	@throws BaseException If there is another error
	*/
	public Hardware getHardware()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Hardware.class, getData().getHardware());
	}
	
	/**
	 	Get the {@link Hardware} that was used in this event.
	 	@param hardware A <code>Hardware</code> item, or null if not known
	 	@throws PermissionDeniedException If the logged in user doesn't have
	 		write permission on this event or use permission for the hardware
	*/
	public void setHardware(Hardware hardware)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (hardware != null) hardware.checkPermission(Permission.USE);
		getData().setHardware(hardware == null ? null : hardware.getData());
	}

	/**
		Get the quantity that was used from the biomaterial by this event.
		If this is a creation event this value is the negative value of
		the original quantity of the created biomaterial.
	*/
	public Float getUsedQuantity()
	{
		return getData().getUsedQuantity();
	}
	/**
		Set the used quantity.
	 	@param usedQuantity The quantity to set. 
	 	@throws PermissionDeniedException If logged in user doesn't have write 
	 		permission on current item. 
	*/
	public void setUsedQuantity(Float usedQuantity)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		MeasuredBioMaterialData data = getData().getBioMaterial();
		Float used = getData().getUsedQuantity();
		if (used != null) 
		{
			MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), data, -used);
		}
		if (usedQuantity != null) MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), data, usedQuantity);
		getData().setUsedQuantity(usedQuantity);
	}

	/**
		Get the quantity that was used by this event for the specified biomaterial.
		@param bioMaterial A biomaterial object
	*/
	public Float getUsedQuantity(MeasuredBioMaterial bioMaterial)
	{
		if (bioMaterial.getData().equals(getData().getBioMaterial()))
		{
			return getUsedQuantity();
		}
		else
		{
			UsedQuantity used = getData().getSources().get((MeasuredBioMaterialData)bioMaterial.getData());
			return used == null ? null : used.getUsedQuantity();
		}
	}
	
	/**
		Get the source group of the biomaterial
		@param bioMaterial The source biomaterial
		@return The source group index or 0 if the biomaterial is not a source of this event
		@since 2.6
		@see #setSourceGroup(MeasuredBioMaterial, int)
	*/
	public int getSourceGroup(MeasuredBioMaterial bioMaterial)
	{
		UsedQuantity used = getData().getSources().get(bioMaterial.getData());
		return used != null ? used.getSourceGroup() : 0;
	}
	
	/**
		Add a source biomaterial to this event or update
		the used quantity of an existing source. Sources can only be specified if
		the produced biomaterial is a pooled biomaterial and this is a creation event.
		
		@param bioMaterial The biomaterial used as a source
		@param usedQuantity The quantity that was used from the source biomaterial
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for this event of use permission for the source
		@throws InvalidDataException If this is not a creation event or 
			if the produced biomaterial is not pooled or if the source
			biomaterial is of another type
		@throws BaseException If there is another error
	*/
	public void addSource(MeasuredBioMaterial bioMaterial, Float usedQuantity)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		checkPermission(Permission.WRITE);
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		bioMaterial.checkPermission(Permission.USE);
		checkAllowedSource(bioMaterial);
		MeasuredBioMaterialData data = (MeasuredBioMaterialData)bioMaterial.getData();
		UsedQuantity used = getData().getSources().get(data);
		if (used != null)
		{
			// There is already an entry for this biomaterial
			if (used.getUsedQuantity() != null)
			{
				// Return the old value to the biomaterial
				MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), data, -used.getUsedQuantity());
			}
			// Set the new value
			used.setUsedQuantity(usedQuantity);
		}
		else
		{
			// This is a new entry
			used = new UsedQuantity(usedQuantity, 1); 
			getData().getSources().put(data, used);
		}
		if (usedQuantity != null) 
		{
			// Take away the new used quantity from the biomaterial
			MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), data, usedQuantity);
		}
	}
	
	/**
		Set the source group for a biomaterial that was used as a source in this event.
		The source group value is used to group sources that are related. The only
		example and use case is when using multi-array hybridizations
		where it is essential to know which array a labeled extract goes on. The source
		group is then simply the number of the array on the hybridization and should be
		a value between 1 and {@link Hybridization#getNumArrays()}.
		<p>
		A single labeled extract can only be used in a single group. Experiments using a n-channel
		platform and a common reference must first split the reference labeled extract
		into individual labeled extracts. This can be done with the 'pooling' functionality.
		<p>
		If the source biomaterial has not been added with {@link #addSource(MeasuredBioMaterial, Float)}
		this method will automatically add it with usedQuantity = null.
	
		@param bioMaterial The source biomaterial
		@param sourceGroup The source group
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for this event of use permission for the source
		@throws InvalidDataException If this is not a creation event or 
			if the produced biomaterial is not pooled or if the source
			biomaterial is of another type
		@since 2.6
	*/
	public void setSourceGroup(MeasuredBioMaterial bioMaterial, int sourceGroup)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		checkPermission(Permission.WRITE);
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		MeasuredBioMaterialData data = (MeasuredBioMaterialData)bioMaterial.getData();

		UsedQuantity used = getData().getSources().get(data);
		if (used == null)
		{
			addSource(bioMaterial, null);
			used = getData().getSources().get(data);
		}
		if (used != null)
		{
			used.setSourceGroup(sourceGroup);
		}
	}
	
	/**
		Remove a biomaterial used as a source. Sources can only be specified if
		the produced biomaterial is a pooled biomaterial and this is a creation event.
		@param bioMaterial The biomaterial used as a source
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for this event of use permission for the source
		@throws InvalidDataException If this is not a creation event or 
			if the produced biomaterial is not pooled or if the source
			biomaterial is of another type
		@throws BaseException If there is another error
	*/
	public void removeSource(MeasuredBioMaterial bioMaterial)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		checkPermission(Permission.WRITE);
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		checkAllowedSource(bioMaterial);
		MeasuredBioMaterialData data = (MeasuredBioMaterialData)bioMaterial.getData();
		UsedQuantity used = getData().getSources().remove(data);
		Float usedQuantity = used == null ? null : used.getUsedQuantity();
		if (usedQuantity != null) MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), data, -usedQuantity);
	}

	/**
		Check if the biomaterial is allowed as a source. It is allowed if:
		<ul>
		<li>This is a creation event
		<li>The created biomaterial is pooled
		<li>It is of the same type as the created biomaterial
		<li>It is not the same item as the created biomaterial
		</ul>
	*/
	private void checkAllowedSource(MeasuredBioMaterial bioMaterial)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		Type eventType = getEventType();
		switch (eventType)
		{
			case OTHER:
			{
				throw new InvalidDataException("OTHER events can't have sources");
			}
			case HYBRIDIZATION:
			{
				if (bioMaterial.getType() != Item.LABELEDEXTRACT)
				{
					throw new InvalidDataException("The source for a hybridization must be a labeled extract");
				}
				break;
			}
			case CREATION:
			{
				MeasuredBioMaterial parent = getBioMaterial();
				if (!parent.isPooled())
				{
					throw new InvalidDataException("Non-pooled biomaterials can't have sources");
				}
				else if (bioMaterial.getType() != parent.getType())
				{
					throw new InvalidDataException("The source biomaterial must be of the same type as the produced biomaterial");
				}
				else if (bioMaterial.equals(parent))
				{
					throw new InvalidDataException("The source biomaterial cannot be the same as the produced biomaterial");
				}
				break;
			}
			default:
			{
				throw new AssertionError("Invalid event type: "+eventType);
			}
		}
	}

	/**
		Set a single source. This method is used for non-pooled biomaterial
		to specify the parent. 
		@see Extract#setSample(Sample, Float)
		@see LabeledExtract#setExtract(Extract, Float)
	*/
	void setSource(MeasuredBioMaterial bioMaterial, Float usedQuantity)
		throws PermissionDeniedException, InvalidDataException
	{
		clearSources();
		if (bioMaterial != null)
		{
			MeasuredBioMaterialData data = (MeasuredBioMaterialData)bioMaterial.getData();
			getData().getSources().put(data, new UsedQuantity(usedQuantity, 1));
			if (usedQuantity != null) MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), data, usedQuantity);
		}
	}
	
	/**
		Clear the sources collection and return the used quantities to
		the source biomaterials.
	*/
	void clearSources()
	{
		Iterator<Map.Entry<MeasuredBioMaterialData, UsedQuantity>> i = getData().getSources().entrySet().iterator();
		while (i.hasNext())
		{
			Map.Entry<MeasuredBioMaterialData, UsedQuantity> entry = i.next();
			MeasuredBioMaterialData source = entry.getKey();
			UsedQuantity used = entry.getValue();
			Float usedQuantity = used == null ? null : used.getUsedQuantity();
			if (usedQuantity != null) MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), source, -usedQuantity);
			i.remove();
		}
	}

	/**
		Get a query that return all biomaterials used as sources
		for this event. To filter or sort on the used quantity or source group columns
		use 'srcevt' as alias. For example to only return sources in source group 2
		and to order the result by used quantity:
		<pre class="code">
query = event.getSources();
query.restrict(Expressions.gteq(
  Hql.property("srcevt", "sourceGroup"), Expressions.integer(2))
);
query.order(Orders.asc(Hql.property("srcevt", "usedQuantity")));
</pre>
		<p>
		NOTE! The filtering and ordering on the "srcevt" columns is only
		available in BASE 2.6 and later.
		
		@return An {@link ItemQuery} object
	*/
	public ItemQuery<? extends MeasuredBioMaterial> getSources()
	{
		ItemQuery<? extends MeasuredBioMaterial> query = null;
		MeasuredBioMaterialData bioMaterialData = getData().getBioMaterial();
		boolean pooled = bioMaterialData != null ? bioMaterialData.isPooled() : false;
		Item parentType = Item.fromDataObject(bioMaterialData == null ? getData().getHybridization() : bioMaterialData);
		switch (parentType)
		{
			case SAMPLE:
			{
				query = Sample.getQuery();
				break;
			}
			case EXTRACT:
			{
				if (pooled)
				{
					query = Extract.getQuery();
				}
				else
				{
					query = Sample.getQuery();
				}
				break;
			}
			case LABELEDEXTRACT:
			{
				if (pooled)
				{
					query = LabeledExtract.getQuery();
				}
				else
				{
					query = Extract.getQuery();
				}
				break;
			}
			case HYBRIDIZATION:
			{
				query = LabeledExtract.getQuery();
				break;
			}
			default:
			{
				throw new AssertionError("Invalid biomaterial type: "+parentType);
			}
		}
		
		query.joinPermanent(Hql.innerJoin("sourceEvents", "srcevt"));
		query.joinPermanent(Hql.innerJoin("srcevt", "event", TYPE.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(TYPE.getAlias()),
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		The maximum length of the comment about this event. Check the length
		against this value before calling the {@link #setComment(String)}
		method to avoid exceptions.
	*/
	public static final int MAX_COMMENT_LENGTH = BioMaterialEventData.MAX_COMMENT_LENGTH;
	/**
		Get the comment about this event.
		@return The comment
	*/
	public String getComment()
	{
		return getData().getComment();
	}
	/**
		Set the comment about this event. The value may be null, but mustn't
		be longer than the value specified by the {@link #MAX_COMMENT_LENGTH} constant.
		@param comment The new comment
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event
		@throws InvalidDataException If the comment is too long
	*/
	public void setComment(String comment)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setComment(StringUtil.setNullableString(comment, "comment", MAX_COMMENT_LENGTH));
	}
	
	/**
		Get the date this event was added to the database.
		@return A <code>Date</code> object
	*/
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}

	/**
		Get the date this event was done in the lab.
		@return A <code>Date</code> object, or null if not known
	*/
	public Date getEventDate()
	{
		return DateUtil.copy(getData().getEventDate());
	}
	/**
		Set the date this event was done in the lab.
		@param eventDate The date, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event
	*/
	public void setEventDate(Date eventDate)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setEventDate(DateUtil.setNullableDate(eventDate, "eventDate"));
	}

	
	/**
		The type of the event.
	*/
	public enum Type
	{
		/**
			An event of this type created a biomaterial.
		*/
		CREATION(1, "Creation"),

		/**
			An event of this type created a hybridization.
		*/
		HYBRIDIZATION(2, "Hybridization"),
		
		/**
			An event of this type affected a single biomaterial in some way.
		*/
		OTHER(3, "Other");

		/**
			Maps the integer that is stored in the database.
		*/
		private static final Map<Integer, Type> valueMapping = new HashMap<Integer, Type>();
		
		static
		{
			for(Type type : Type.values())
			{
				Type t = valueMapping.put(type.getValue(), type);
				assert t == null : "Another BioMaterialEvent.Type with the value "+type.getValue()+" already exists";
			}
		}
		
		/**
			Get the <code>BioMaterialEvent.Type</code> object when you know the integer code.
			
			@param value The integer value.
			@return The type object or null if the value is unknown
		*/
		public static Type fromValue(int value)
		{
			Type type = valueMapping.get(value);
			assert type != null : "type == null for value "+value;
			return type;
		}

		private final int value;
		private final String displayValue;
		private Type(int value, String displayValue)
		{
			this.value = value;
			this.displayValue = displayValue;
		}
		
		public String toString()
		{
			return displayValue;
		}
		
		/**
			Get the integer value that is used when storing a <code>BioMaterialEvent.Type</code> 
			to the database.
			@return The intger value of this type 
		*/
		public int getValue()
		{
			return value;
		}
	}
}

