/*
	$Id: DynamicRestriction.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import java.util.Collection;

import net.sf.basedb.core.query.QueryElement;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Query;
import net.sf.basedb.core.query.QueryType;

/**
	Creates a restriction that compares a dynamic column with a constant
	value: <code>tableAlias.column = value</code> or 
	<code>queryRootAlias.column = value</code>
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
class DynamicRestriction
	implements Restriction
{
	
	private final VirtualTable table;
	private final VirtualColumn column;
	private final int value;

	/**
		Create a new restriction for column in the root entity of the query:
		<code>queryRootAlias.column = value</code>.
		@param column The column to use in the restriction
		@param value The value to look for
	*/
	DynamicRestriction(VirtualColumn column, int value)
	{
		this(null, column, value);
	}

	/**
		Create a new restriction for column in a joined table:
		<code>tableAlias.column = value</code>. It is assumed that
		the table specified has been joined in the query.
		@see DynamicJoin
		@param table The table the column is located in
		@param column The column to use in the restriction
		@param value The value to look for
	*/
	DynamicRestriction(VirtualTable table, VirtualColumn column, int value)
	{
		this.table = table;
		this.column = column;
		this.value = value;
	}
	
	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.SQL)
		{
			return (table == null ? query.getRootAlias() : HibernateUtil.quote(table.getAlias())) + "." + HibernateUtil.quote(column.getColumn()) + " = " + value;
		}
		else
		{
			throw new UnsupportedOperationException("DynamicRestriction not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return (table == null ? "" : table.getAlias() + ".") + column.getColumn() + " = " + value;
	}
	// -------------------------------------------

}
