/*
	$Id: DynamicSpotQuery.java 4479 2008-09-05 15:16:27Z jari $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Select;
import net.sf.basedb.core.query.JoinType;

/**
	Represents a query for spot information in the dynamic database. A single
	query object may query against data in one bioassayset at a time.
	Use factory methods in the {@link Dynamic} class to join other tables and 
	create restrictions based on the data in those tables. You may of course
	also use the regular expressions and restrictions from the {@link
	net.sf.basedb.core.query.Expressions}, {@link net.sf.basedb.core.query.Restrictions} 
	and other classes.
	<p>
	This type of query is executed by the {@link #iterate(DbControl)} method
	and returns the result as a {@link DynamicResultIterator}.
	
	@author Nicklas
	@version 2.0
	@see BioAssaySet#getSpotData()
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class DynamicSpotQuery
	extends DynamicQuery
{

	private Select[] defaultSelects;
	
	private Set<Object> joinedItems;
	
	/**
		Create a new dynamic query.
		@param cube The cube to query against
	*/
	DynamicSpotQuery(DataCube cube)
	{
		super(cube, VirtualTable.SPOT);
		joinedItems = new HashSet<Object>();
		setAutoJoinType(JoinType.LEFT);
	}
	
	/*
		From the AbstractSqlQuery class
		-------------------------------------------
	*/
	/**
		Default selection: column, postion, ch1, ch2, ...
	*/
	Select[] getDefaultSelects()
	{
		if (defaultSelects == null)
		{
			int channels = getDataCube().getRawDataType().getChannels();
			defaultSelects = new Select[2+channels];
			defaultSelects[0] = Dynamic.select(VirtualColumn.COLUMN);
			defaultSelects[1] = Dynamic.select(VirtualColumn.POSITION);
			for (int i = 1; i <= channels; ++i)
			{
				defaultSelects[i+1] = Dynamic.select(VirtualColumn.channel(i));
			}
		}
		return defaultSelects;
	}
	// -------------------------------------------
	/*
		From the AbstractSqlQuery class
		-------------------------------------------
	*/

	/**
	 	Reset join information.
	*/
	@Override
	public void reset()
	{
		super.reset();
		joinedItems.clear();
	}
	// -------------------------------------------

	/**
		Join information about reporters in the query. Now it becomes possible
		to select reporter data or restrict the results based on reporter
		information.
		@param joinType The type of join. Can not be null.
	 	@throws InvalidDataException If joinType parameter is null.
		@see Dynamic#selectReporter(String)
		@see Dynamic#reporter(String)
	*/
	public void joinReporters(JoinType joinType)
		throws InvalidDataException
	{
		if (joinType == null) throw new InvalidUseOfNullException("joinType");
		if (!joinedItems.contains(VirtualTable.POSITION))
		{
			join(new DynamicJoin(getVirtualDb(), joinType, null, VirtualTable.POSITION, 
				VirtualColumn.CUBE, VirtualColumn.POSITION));
			joinedItems.add(VirtualTable.POSITION);
		}
		if (!joinedItems.contains(RealTable.REPORTERS))
		{
			join(new RealJoin(joinType, VirtualTable.POSITION, RealTable.REPORTERS, 
				VirtualColumn.REPORTER_ID));
			joinedItems.add(RealTable.REPORTERS);
		}
	}
	
	/**
	  	Makes a join with a <code>ReporterList</code>
	  	@param reporterList To do the join on.
	  	@param joinType The join type to use. 
	  		@see net.sf.basedb.core.query.JoinType
	*/
	public void joinReporterList(ReporterList reporterList, JoinType joinType)
	{
		if (!joinedItems.contains(VirtualTable.POSITION))
		{
			join(new DynamicJoin(getVirtualDb(), joinType, null, VirtualTable.POSITION, 
				VirtualColumn.CUBE, VirtualColumn.POSITION));
			joinedItems.add(VirtualTable.POSITION);
		}
		if (!joinedItems.contains(reporterList))
		{
			join(new ReporterListJoin(reporterList, joinType));
			joinedItems.add(reporterList);
		}
	}
	
	
	/**
		Join information about raw data in the query. Now it becomes possible
		to select raw data or restrict the results based on values in the raw
		data.
		@param joinType The type of join. Can not be null.
	 	@throws InvalidDataException If joinType parameter is null. 
		@see Dynamic#selectRawData(String)
		@see Dynamic#rawData(String)
	*/
	public void joinRawData(JoinType joinType)
		throws InvalidDataException
	{
		RawDataType rdt = getDataCube().getRawDataType();
		if (!rdt.isStoredInDb())
		{
			throw new BaseException("Raw data for raw data type: " + rdt + " is not stored in the database.");
		}
		
		if (joinType == null) throw new InvalidUseOfNullException("joinType");
		if (!joinedItems.contains(VirtualTable.RAWPARENTS))
		{
			join(new DynamicJoin(getVirtualDb(), joinType, null, VirtualTable.RAWPARENTS, 
				VirtualColumn.CUBE, VirtualColumn.COLUMN, VirtualColumn.POSITION));
			joinedItems.add(VirtualTable.RAWPARENTS);
		}
		if (!joinedItems.contains(rdt))
		{
			join(new RealJoin(joinType, VirtualTable.RAWPARENTS, rdt.getRealTable(),
				VirtualColumn.RAWDATA_ID));
			joinedItems.add(rdt);
		}
	}
	
	/**
		Join extra values in the query. Now it becomes possible to select
		those extra values or use them for restrictions.
		@param extraValue The extra values to join. Can not be null.
		@param joinType The type of join. Can not be null.
	 	@throws InvalidDataException If any of the required parameters are null. 
		@see Dynamic#select(ExtraValue, String)
		@see Dynamic#extraValue(ExtraValue)
	*/
	public void joinExtraValue(ExtraValue extraValue, JoinType joinType)
		throws InvalidDataException
	{
		if (extraValue == null) throw new InvalidUseOfNullException("extraValue");
		if (joinType == null) throw new InvalidUseOfNullException("joinType");
		if (extraValue.getCoordinateType() != ExtraValue.CoordinateType.SPOT)
		{
			throw new InvalidDataException("Extra value "+ extraValue+
				" is not attached to spots: "+extraValue.getCoordinateType());
		}
		if (!getDataCube().equals(extraValue.getDataCubeExtraValue().getDataCube()))
		{
			throw new InvalidDataException("Extra value "+extraValue+
				" is not part of the same data cube as this query: " + getDataCube());
		}
		if (!joinedItems.contains(extraValue))
		{
			join(new ExtraValueJoin(extraValue, joinType));
			joinedItems.add(extraValue);
		}
	}
	
	/**
		Specify the join type of automatic joins. The default join type is
		{@link JoinType#LEFT}. This setting doesn't affect joins that has been made
		directly to the the query.
		@param joinType The type of the automatic joins
	*/
	public void setAutoJoinType(JoinType joinType)
	{
		addAutoJoiner(DynamicSpotQueryJoiners.getExtraValueJoiner(joinType));
		addAutoJoiner(DynamicSpotQueryJoiners.getReporterJoiner(joinType));
		addAutoJoiner(DynamicSpotQueryJoiners.getRawDataJoiner(joinType));
		addAutoJoiner(DynamicSpotQueryJoiners.getReporterListJoiner(joinType));
	}
}
