/*
	$Id: Experiment.java 4479 2008-09-05 15:16:27Z jari $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ExperimentData;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.info.ExperimentInfo;
import net.sf.basedb.info.ToTransferable;

import java.util.Date;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
	An experiment is the starting point of the analysis in BASE. An experiment
	is linked to zero or more {@link RawBioAssay}:s of the same {@link RawDataType}
	as itself. It is not possible to mix raw data types. A root 
	{@link Transformation} is used to calculate the first set of intensities from
	the raw data. The intensities are stored in a root {@link BioAssaySet}.
	<p>
	Then, it is possible to apply more transformations using the root
	bioassayset as a source. Each transformation may create new bioassaysets,
	from which it is possible to apply new transformations and so on.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class Experiment
	extends CommonItem<ExperimentData>
	implements DiskConsumable, Transactional, ToTransferable<ExperimentInfo>
{
	/**
		The type of item represented by this class.
		@see Item#EXPERIMENT
		@see #getType()
	*/
	public static final Item TYPE = Item.EXPERIMENT;


	/**
		The maximum length of the authors property that can be stored in the
		database. Check the length against this value before calling the
		{@link #setAuthors(String)} method to avoid exceptions.
	*/
	public static final int MAX_AUTHORS_LENGTH = ExperimentData.MAX_AUTHORS_LENGTH;

	/**
		The maximum length of the affiliations property that can be stored in the
		database. Check the length against this value before calling the
		{@link #setAffiliations(String)} method to avoid exceptions.
	*/
	public static final int MAX_AFFILIATIONS_LENGTH = ExperimentData.MAX_AFFILIATIONS_LENGTH;

	/**
		The maximum length of the title property that can be stored in the
		database. Check the length against this value before calling the
		{@link #setTitle(String)} method to avoid exceptions.
	*/
	public static final int MAX_TITLE_LENGTH = ExperimentData.MAX_TITLE_LENGTH;

	/**
		The maximum length of the abstract property that can be stored in the
		database. Check the length against this value before calling the
		{@link #setAbstract(String)} method to avoid exceptions.
	*/
	public static final int MAX_ABSTRACT_LENGTH = ExperimentData.MAX_ABSTRACT_LENGTH;

	/**
		The maximum length of the publication property that can be stored in the
		database. Check the length against this value before calling the
		{@link #setPublication(String)} method to avoid exceptions.
	*/
	public static final int MAX_PUBLICATION_LENGTH = ExperimentData.MAX_PUBLICATION_LENGTH;

	/**
		The maximum length of the experimentType property that can be stored in the
		database. Check the length against this value before calling the
		{@link #setExperimentType(String)} method to avoid exceptions.
	*/
	public static final int MAX_EXPERIMENT_TYPE_LENGTH = ExperimentData.MAX_EXPERIMENT_TYPE_LENGTH;

	/**
		The maximum length of the experimentDesign property that can be stored in the
		database. Check the length against this value before calling the
		{@link #setExperimentDesign(String)} method to avoid exceptions.
	*/
	public static final int MAX_EXPERIMENT_DESIGN_LENGTH = ExperimentData.MAX_EXPERIMENT_DESIGN_LENGTH;

	/**
		The maximum length of the pubMedId property that can be stored in the
		database. Check the length against this value before calling the
		{@link #setPubMedId(String)} method to avoid exceptions.
	*/
	public static final int MAX_PUBMEDID_LENGTH = ExperimentData.MAX_PUBMEDID_LENGTH;

	/**
		Create a new <code>Experiment</code> item.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param rawDataType The data type of the raw data used in this experiment
		@return The new <code>Experiment</code> item
		@throws BaseException If there is an error
	*/
	public static Experiment getNew(DbControl dc, RawDataType rawDataType)
		throws BaseException
	{
		Experiment e = dc.newItem(Experiment.class);
		e.setRawDataType(rawDataType);
		e.setName("New experiment");
		return e;
	}

	/**
		Get an <code>Experiment</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>Experiment</code> item
		@throws ItemNotFoundException If an item with the specified id 
			is not found
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static Experiment getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		Experiment e = dc.loadItem(Experiment.class, id);
		if (e == null) throw new ItemNotFoundException("Experiment[id=" + id + "]");
		return e;
	}
	
	/**
		Get a query configured to retrieve experiments
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<Experiment> getQuery()
	{
		return new ItemQuery<Experiment>(Experiment.class);
	}

	private long addedBytes = 0;
	private long updatedBytes = 0;
	
	/**
	 	Creates a new experiment item from the given data.
		@param data the data
	*/
	Experiment(ExperimentData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From the DiskConsumning interface
		-------------------------------------------
	*/
	/**
		TRUE if the experiment has been saved to the database, FALSE otherwise.
	*/
	public boolean useDifference()
	{
		return isInDatabase();
	}
	public long getBytes()
	{
		return getData().getBytes();
	}
	public long getDifference()
	{
		return addedBytes;
	}
	// Called from datacube which is called from batchers
	void addBytes(long bytes)
	{
		addedBytes += bytes;
	}
	/**
		Always {@link QuotaType#EXPERIMENT}.
	*/
	public String getQuotaTypeSystemId() 
	{
		return QuotaType.EXPERIMENT;
	}
	/**
		Always {@link Location#PRIMARY}.
	*/
	public Location getLocation()
	{
		return Location.PRIMARY;
	}
	// -------------------------------------------
	/*
		From BasicItem
		-------------------------------------------
	*/
	void onBeforeCommit(Transactional.Action action)
		throws BaseException
	{
		super.onBeforeCommit(action);
		if (action == Transactional.Action.DELETE)
		{
			getDbControl().deleteItem(getVirtualDb());
			unlinkJobs();
		}
		else if (action == Transactional.Action.CREATE)
		{
			getData().setBytes(addedBytes);
		}
		else if (action == Transactional.Action.UPDATE && addedBytes != updatedBytes)
		{
			/*
				In case of an update we handle the bytes ourselves since plugins with 
				long running times has a high risk of getting ItemModifiedException
				if some other thing has happened to the experiment at the same time.
			*/
			org.hibernate.Session session = getDbControl().getHibernateSession();
			org.hibernate.Query query = 
				HibernateUtil.getPredefinedQuery(session, "UPDATE_BYTES_FOR_EXPERIMENT");
			/*
				UPDATE ExperimentData xpm
				SET xpm.bytes = xpm.bytes + :addedBytes
				WHERE xpm.id = :experiment
			*/
			query.setLong("addedBytes", addedBytes - updatedBytes);
			query.setInteger("experiment", getId());
			query.executeUpdate();
			updatedBytes = addedBytes;
		}
	}
	
	/**
	 	@since 2.5
	 */
	public ExperimentInfo toTransferable(ExperimentInfo info)
	{
		super.toTransferable(info);
		info.setAbstractText(this.getAbstract());
		info.setAffiliations(this.getAffiliations());
		info.setAuthors(this.getAuthors());
		info.setExperimentDesign(this.getExperimentDesign());
		info.setExperimentType(this.getExperimentType());
		info.setPublication(this.getPublication());
		info.setTitle(this.getTitle());
		info.setPublicationDate(this.getPublicationDate());
		info.setPubMedId(this.getPubMedId());
		return info;
	}
	
	// -------------------------------------------
	
	/**
		Get the raw data type this experiment uses for the raw data.
		The raw data type cannot be changed once the experiment has been created.
		@return A <code>RawDataType</code> object
	*/
	public RawDataType getRawDataType()
	{
		return RawDataTypes.getSafeRawDataType(getData().getRawDataType());
	}
	
	/**
		Set the raw data type. It cannot be changed.
		@param rawDataType The raw data type, null is not allowed
	*/
	private void setRawDataType(RawDataType rawDataType)
		throws InvalidDataException
	{
		if (rawDataType == null) throw new InvalidUseOfNullException("rawDataType");
		getData().setRawDataType(rawDataType.getId());
	}

	VirtualDb getVirtualDb()
	{
		return getDbControl().getItem(VirtualDb.class, getData().getVirtualDb());
	}
	
	/**
		Get the title for the publication for this experiment.
	*/
	public String getTitle()
	{
		return getData().getTitle();
	}
	/**
		Set the title for the publication of this experiment. The value may be 
		null, but mustn't be longer than the value specified by the 
		{@link #MAX_TITLE_LENGTH} constant.
	
		@param title The new title
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the text is too long
	*/
	public void setTitle(String title)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setTitle(StringUtil.setNullableString(title, "title", MAX_TITLE_LENGTH));
	}

	/**
		Get the associated directory where it is recommended that plugins
		store files that are related to this experiment.

		@return The <code>Directory</code> item
		@throws PermissionDeniedException This exception is thrown if
			the logged in user doesn't have read
			permission to the item
		@throws BaseException If there is another error
	*/
	public Directory getDirectory()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Directory.class, getData().getDirectory());
	}
	
	/**
		Set the directory where it is recommended that plugins
		store files that are related to this experiment.
		@param directory The new {@link Directory}, or null
		@throws PermissionDeniedException If the logged in user doesn't 
			have write permission to the experiment or
			use permission to the directory
	*/
	public void setDirectory(Directory directory)
		throws PermissionDeniedException, InvalidUseOfNullException
	{
		checkPermission(Permission.WRITE);
		if (directory != null) directory.checkPermission(Permission.USE);
		getData().setDirectory(directory == null ? null : directory.getData());
	}

	
	/**
		Get the abstract text for the publication for this experiment.
	*/
	public String getAbstract()
	{
		return getData().getAbstract();
	}
	/**
		Set the abstract text for the publication of this experiment. The value may be 
		null, but mustn't be longer than the value specified by the 
		{@link #MAX_ABSTRACT_LENGTH} constant.

		@param theAbstract The new abstract text
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the text is too long
	*/
	public void setAbstract(String theAbstract)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setAbstract(StringUtil.setNullableString(theAbstract, "abstract", MAX_ABSTRACT_LENGTH));
	}

	/**
		Get the authors of the publication for this experiment.
	*/
	public String getAuthors()
	{
		return getData().getAuthors();
	}
	/**
		Set the authors of the publication for this experiment. The value may be 
		null, but mustn't be longer than the value specified by the 
		{@link #MAX_AUTHORS_LENGTH} constant.

		@param authors The new authors text
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the text is too long
	*/
	public void setAuthors(String authors)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setAuthors(StringUtil.setNullableString(authors, "authors", MAX_AUTHORS_LENGTH));
	}

	/**
		Get the affiliations for this experiment.
	*/
	public String getAffiliations()
	{
		return getData().getAffiliations();
	}
	/**
		Set the affiliations for this experiment. The value may be 
		null, but mustn't be longer than the value specified by the 
		{@link #MAX_AFFILIATIONS_LENGTH} constant.

		@param affiliations The new affiliations text
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the text is too long
	*/
	public void setAffiliations(String affiliations)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setAffiliations(StringUtil.setNullableString(affiliations, "affiliations", MAX_AFFILIATIONS_LENGTH));
	}

	/**
		Get the a text describing the experimental design.
	*/
	public String getExperimentDesign()
	{
		return getData().getExperimentDesign();
	}
	/**
		Set a text describing the experimental design. The value may be 
		null, but mustn't be longer than the value specified by the 
		{@link #MAX_EXPERIMENT_DESIGN_LENGTH} constant.

		@param experimentDesign The new experimental design
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the text is too long
	*/
	public void setExperimentDesign(String experimentDesign)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setExperimentDesign(StringUtil.setNullableString(experimentDesign, "experimentDesign", MAX_EXPERIMENT_DESIGN_LENGTH));
	}

	/**
		Get the a text describing the experiment type.
	*/
	public String getExperimentType()
	{
		return getData().getExperimentType();
	}
	/**
		Set a text describing the experiment type. The value may be 
		null, but mustn't be longer than the value specified by the 
		{@link #MAX_EXPERIMENT_TYPE_LENGTH} constant.

		@param experimentType The new experiment type
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the text is too long
	*/
	public void setExperimentType(String experimentType)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setExperimentType(StringUtil.setNullableString(experimentType, "experimentType", MAX_EXPERIMENT_TYPE_LENGTH));
	}

	/**
		Get the publication text.
	*/
	public String getPublication()
	{
		return getData().getPublication();
	}
	/**
		Set publication text. The value may be 
		null, but mustn't be longer than the value specified by the 
		{@link #MAX_PUBLICATION_LENGTH} constant.

		@param publication The publication text
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the text is too long
	*/
	public void setPublication(String publication)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setPublication(StringUtil.setNullableString(publication, "publication", MAX_PUBLICATION_LENGTH));
	}

	/**
		Get the PubMed ID of the publication.
	*/
	public String getPubMedId()
	{
		return getData().getPubMedId();
	}
	/**
		Set the PubMed ID of the publication. The value may be 
		null, but mustn't be longer than the value specified by the 
		{@link #MAX_PUBMEDID_LENGTH} constant.

		@param pubMedId The PubMed ID
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
		@throws InvalidDataException If the text is too long
	*/
	public void setPubMedId(String pubMedId)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setPubMedId(StringUtil.setNullableString(pubMedId, "pubMedId", MAX_PUBMEDID_LENGTH));
	}

	/**
		Get the date this experiment was published.
		@return A <code>Date</code> object, or null if not known
	*/
	public Date getPublicationDate()
	{
		return DateUtil.copy(getData().getPublicationDate());
	}
	/**
		Set the date this experiment was published.
		@param publicationDate The date, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
	*/
	public void setPublicationDate(Date publicationDate)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setPublicationDate(DateUtil.setNullableDate(publicationDate, "publicationDate"));
	}

	/**
		Add a raw bioassay to this experiment. Spots must have been added to the 
		raw bioassay and it must have the same {@link RawDataType} as this experiment.
		@param rawBioAssay The rawBioAssay to add to this experiment
		@throws PermissionDeniedException If the
			logged in user doesn't have {@link Permission#WRITE}
			permission for the experiment and {@link Permission#USE}
			for the raw bioassay
		@throws InvalidDataException If the rawBioAssay is null,
			or has a different raw data type
	*/
	public void addRawBioAssay(RawBioAssay rawBioAssay)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (rawBioAssay == null) throw new InvalidUseOfNullException("rawBioAssay");
		rawBioAssay.checkPermission(Permission.USE);
		String experimentDataType = getData().getRawDataType();
		String bioAssayDataType = rawBioAssay.getData().getRawDataType();
		if (!bioAssayDataType.equals(experimentDataType))
		{
			throw new InvalidDataException("Raw data types are different for experiment ("+
				experimentDataType+") and raw bioassay ("+bioAssayDataType+")");
		}
		getData().getRawBioAssays().add(rawBioAssay.getData());
	}

	/**
		Remove a raw bioassay from this experiment.
		@param rawBioAssay The rawBioAssay to remove from this experiment
		@throws PermissionDeniedException If the
			logged in user doesn't have {@link Permission#WRITE}
			permission for the experiment
		@throws InvalidDataException If the rawBioAssay is null
	*/
	public void removeRawBioAssay(RawBioAssay rawBioAssay)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (rawBioAssay == null) throw new InvalidUseOfNullException("rawBioAssay");
		getData().getRawBioAssays().remove(rawBioAssay.getData());
	}
	
	/**
		Check if a raw bioassay is used in this experiment.
		@param rawBioAssay The raw bioassay to look for
		@return TRUE if the raw bioassay is used in the experiment, FALSE otherwise
	*/
	public boolean isUsing(RawBioAssay rawBioAssay)
	{
		if (rawBioAssay == null) return false;
		return getData().getRawBioAssays().contains(rawBioAssay.getData());
	}
	
	/**
		Get a query that returns the raw bioassays used in this
		experiment.
		@return An <code>ItemQuery</code> object
	*/
	public ItemQuery<RawBioAssay> getRawBioAssays()
	{
		ItemQuery<RawBioAssay> query = RawBioAssay.getQuery();
		query.joinPermanent(Hql.innerJoin("experiments", Item.EXPERIMENT.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(Item.EXPERIMENT.getAlias()), 
				Hql.entity(this)
			)
		);
		return query;
	}

	/**
		Create a new root transformation for this experiment.
		@param job The job that executed the transformation
		@param rawSources The raw bioassays used as source bioassays
		@throws PermissionDeniedException If the logged in user doesn't have
			use permission for the experiment
		@throws InvalidDataException If any of the raw bioassays are not 
			part of the experiment
		@throws BaseException If there is another error
	*/
	public Transformation newTransformation(Job job, Collection<RawBioAssay> rawSources)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		checkPermission(Permission.USE);
		Transformation t = Transformation.getNew(getDbControl(), this, job);
		t.setRawSources(rawSources);
		return t;
	}

	/**
		Get a query that return all transformations including root 
		transformations in this experiment.
		@return An <code>ItemQuery</code> object
		@see Transformation#getQuery(Experiment)
		@see #getRootTransformations()
	*/
	public ItemQuery<Transformation> getTransformations()
	{
		return Transformation.getQuery(this);
	}
	
	/**
		Get a query that return all root transformations in this
		experiment. A root transformation is a transformation that doesn't 
		have a source bioassayset.
		
		@return An <code>ItemQuery</code> object
		@see #getTransformations()
		@see Transformation#getSource()
	*/
	public ItemQuery<Transformation> getRootTransformations()
	{
		ItemQuery<Transformation> query = Transformation.getQuery(this);
		query.restrictPermanent(Restrictions.eq(Hql.property("source"), null));
		return query;
	}
	
	/**
		Get a query that return all bioassaysets in this
		experiment.
		@return An <code>ItemQuery</code> object
		@see BioAssaySet#getQuery(Experiment)
	*/
	public ItemQuery<BioAssaySet> getBioAssaySets()
	{
		return BioAssaySet.getQuery(this);
	}
	
	
	/**
		Add an annotation type as an experimental factor to this experiment. 
		@param annotationType The annotation type to add to this experiment
		@throws PermissionDeniedException If the
			logged in user doesn't have {@link Permission#WRITE}
			permission for the experiment and {@link Permission#USE}
			for the annotation type
		@throws InvalidDataException If the annotationType is null
	*/
	public void addExperimentalFactor(AnnotationType annotationType)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (annotationType == null) throw new InvalidUseOfNullException("annotationType");
		annotationType.checkPermission(Permission.USE);
		getData().getExperimentalFactors().add(annotationType.getData());
	}
	
	/**
		Remove an annotation type from the experimental factors for this experiment.
		@param annotationType The annotation type to remove from this experiment
		@throws PermissionDeniedException If the
			logged in user doesn't have {@link Permission#WRITE}
			permission for the experiment
		@throws InvalidDataException If the annotationType is null
	*/
	public void removeExperimentalFactor(AnnotationType annotationType)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (annotationType == null) throw new InvalidUseOfNullException("annotationType");
		getData().getExperimentalFactors().remove(annotationType.getData());
	}

	
	/**
	 	Get a query that returns all annotation types used as experimental factors
	 	for this experiment.
		@return An <code>ItemQuery</code> object
	 */
	public ItemQuery<AnnotationType> getExperimentalFactors()
	{
		ItemQuery<AnnotationType> query = AnnotationType.getQuery(null);
		query.joinPermanent(Hql.innerJoin("experiments", Item.EXPERIMENT.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(Item.EXPERIMENT.getAlias()), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Unlink jobs from this experiment that are references from other items
		NOT PART of the experiment. Without the unlinking it is not possible to 
		delete the experiment because it will not be possible to delete the
		jobs that are part of.
		@see ExperimentData#getJobs()
		@since 2.7
	*/
	private void unlinkJobs()
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Check jobs links with raw bioassays
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"LOAD_RAWBIOASSAY_JOBS_IN_EXPERIMENT");
		/*
			SELECT rba.job.id
			FROM RawBioAssayData rba
			WHERE rba.job.experiment = :experiment
		*/
		query.setEntity("experiment", getData());
		Set<Integer> jobs = new HashSet<Integer>(HibernateUtil.loadList(Integer.class, query, null));
		
		// Check jobs linked with array designs
		query = HibernateUtil.getPredefinedQuery(session, 
			"LOAD_ARRAYDESIGN_JOBS_IN_EXPERIMENT");
		/*
			SELECT ad.job.id
			FROM ArrayDesignData ad
			WHERE ad.job.experiment = :experiment
		*/
		query.setEntity("experiment", getData());
		jobs.addAll(HibernateUtil.loadList(Integer.class, query, null));
	
		// Set the experiment to null
		if (jobs.size() > 0)
		{
			query = HibernateUtil.getPredefinedQuery(session, "UNLINK_JOBS_FROM_EXPERIMENT");
			/*
				UPDATE JobData job 
				SET job.experiment = null
				WHERE job.id IN (:list)
			*/
			query.setParameterList("list", jobs, Type.INT.getHibernateType());
			HibernateUtil.executeUpdate(query);
		}
	}
}
