/*
	$Id: FileSet.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.basedb.core.query.EntityQuery;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.signal.ThreadSignalHandler;

import net.sf.basedb.core.data.FileSetData;
import net.sf.basedb.core.data.FileSetMemberData;
import net.sf.basedb.core.data.DataFileTypeData;
import net.sf.basedb.core.filehandler.DataFileHandler;
import net.sf.basedb.core.filehandler.DataFileMetadataReader;
import net.sf.basedb.core.filehandler.DataFileValidator;

/**
	A file set is the container for files that have been attach to an
	item to hold data instead of importing it to the database.

	@author Nicklas
	@version 2.5
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class FileSet
	extends BasicItem<FileSetData>
{
	/**
		The type of item represented by this class.
		@see Item#FILESET
		@see #getType()
	*/
	public static final Item TYPE = Item.FILESET;
	
	/**
		This filter gives everybody read permission to file sets.
	*/
	private static final QueryRuntimeFilter RUNTIME_FILTER = new QueryRuntimeFilterImpl();

	/**
		Create a new file set for the specified item.
	*/
	static FileSet getNew(DbControl dc, FileStoreEnabled item)
		throws BaseException
	{	
		FileSet fs = dc.newItem(FileSet.class, item);		
		fs.getData().setItemType(item.getType().getValue());		
		return fs;
	}
	
	/**
		Get a <code>FileSet</code> item when you know the id.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>FileSet</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static FileSet getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		FileSet fs = dc.loadItem(FileSet.class, id);
		if (fs == null) throw new ItemNotFoundException("FileSet[id="+id+"]");
		return fs;
	}
	
	/**
		Get a {@link ItemQuery} object configured to retrieve <code>FileSet</code> 
		items. This query may return items which the logged in user doesn't have 
		read permission to. At the moment there is no way to solve this problem.
		@return An {@link ItemQuery} object
	*/
	static ItemQuery<FileSet> getQuery()
	{
		return new ItemQuery<FileSet>(FileSet.class, RUNTIME_FILTER);
	}

	/**
		The item this file set belongs to.
	*/
	private FileStoreEnabled item;
	
	/**
		Regular constructor if the item is not known beforehand.
	*/
	FileSet(FileSetData data)
	{
		super(data);		
	}

	/**
		Special constructor to avoid {@link #getItem()} to hit the database
		when the item is known beforehand.
	*/
	FileSet(FileSetData data, FileStoreEnabled item)
	{
		super(data);
		this.item = item;		
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Always return TRUE. A file set always has a one-to-one relation
		to some item. The file set is automatically deleted if that
		item is deleted.
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		return true;
	}
	
	/**
		Get the item this file set belongs to.
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
		throws BaseException
	{
		Set<ItemProxy> using = super.getUsingItems();
		FileStoreEnabled item = getItem();
		using.add(new ItemProxy(item.getId(), item.getType()));
		return using;
	}
	
	/**
		READ permission is granted to all users. CREATE, WRITE and DELETE permissions are granted
		if the logged in user has WRITE permission on the associated item.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{	
		FileStoreEnabled item = null;
		try
		{
			item = getItem();
		}
		catch (PermissionDeniedException ex)
		{}
		if (item != null && item.hasPermission(Permission.WRITE))
		{
			granted |= Permission.grant(Permission.CREATE, Permission.READ, Permission.WRITE, Permission.DELETE);				
		}
		else if (item != null && item.hasPermission(Permission.USE))
		{
			granted |= Permission.grant(Permission.USE);
		}
		else if (item != null && item.hasPermission(Permission.READ))
		{
			granted |= Permission.grant(Permission.READ);				
		}	
		super.initPermissions(granted, denied);
	}
	PluginPermission getPluginPermissions()
	{
		return PluginPermission.NO_PLUGIN;
	}
	// -------------------------------------------

	/**
		Get the item this file set belongs to.
		@return The <code>FileStoreEnabled</code> item
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission to the item
		@throws BaseException If there is another error
	*/
	public FileStoreEnabled getItem()
		throws PermissionDeniedException, BaseException
	{
		if (item == null)
		{	
			Item type = getItemType();
			DbControl dc = getDbControl();
			String hql = "SELECT item"+
				" FROM "+type.getDataClass().getName()+" item"+
				" WHERE item.fileSet = :fileSet";
			org.hibernate.Query query = HibernateUtil.createQuery(dc.getHibernateSession(), hql);
			query.setEntity("fileSet", this.getData());
			item = (FileStoreEnabled)dc.getItem(type.getItemClass(), HibernateUtil.loadData(type.getDataClass(), query));
		}
		return item;
	}
	
	/**
		Set the item this file set belongs to.
	*/
	void setItem(FileStoreEnabled item)
	{
		this.item = item;
	}
	
	/**
		Get the {@link Item} type of the item this file set belongs to.
	*/
	public Item getItemType()
	{
		return Item.fromValue(getData().getItemType());
	}
	
	/**
		Get the members of this file set.
		@return A query returning <code>FileSetMember</code> items
	*/
	public ItemQuery<FileSetMember> getMembers()
	{
		return FileSetMember.getQuery(this);
	}
	
	/**
		Get a query that returns all members in a file set with a specific 
		generic {@link FileType}. Note! The query checks the generic type from the 
		{@link DataFileType#getGenericType()}, not the {@link File#getFileType()}.
		
		@param genericType The system ID of a generic type as defined
			by {@link FileType#getSystemId()}
		@return A query returning <code>FileSetMember</code> items
	*/
	public ItemQuery<FileSetMember> getMembers(String genericType)
	{
		ItemQuery<FileSetMember> query = getMembers();
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("dataFileType.genericType.systemId"), 
				Expressions.string(genericType)
			)
		);
		return query;
	}
	
	/**
		Set a file of a given type as a member of this file set.
		If another file of the given type already exists, it is replaced
		with this file. If no member exists, a new entry is created and
		automatically saved to the database when {@link DbControl#commit()}
		is called.
		@param file The file to set as a member
		@param type The type of the file
		@return The new FileSetMember item
		@throws InvalidUseOfNullException If file or type is null
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission for this file set or use permission for the
			file
	*/
	public FileSetMember setMember(File file, DataFileType type)
	{
		checkPermission(Permission.WRITE);
		if (file == null) throw new InvalidUseOfNullException("file");
		if (type == null) throw new InvalidUseOfNullException("type");
		if (type.getItemType() != getItemType())
		{
			throw new InvalidDataException("File type '" + type.getName() + 
				"' can't be used on " + getItemType());
		}
		file.checkPermission(Permission.USE);
		
		Map<DataFileTypeData, FileSetMemberData> members = getData().getMembers();
		FileSetMemberData memberData = members.get(type.getData());
		FileSetMember member = null;
		if (memberData == null)
		{
			member = FileSetMember.getNew(getDbControl(), this, file, type);
			members.put(type.getData(), member.getData());
			getDbControl().saveItemIf(this, member, false);
		}
		else
		{
			if (!file.getData().equals(memberData.getFile()))
			{
				memberData.setFile(file.getData());
				memberData.setValid(null);
			}
			member = getDbControl().getItem(FileSetMember.class, memberData);
		}
		return member;
	}
	
	/**
		Check if the file set contains a file of the specified type.
		@param type A <code>DataFileType</code> object
		@return TRUE if the file set contains a file of 
			the specified type, FALSE otherwise
	*/
	public boolean hasMember(DataFileType type)
	{
		if (type == null) return false;
		return getData().getMembers().containsKey(type.getData());
	}

	/**
		Get the member of the given type.
		@param type A <code>DataFileType</code> object 
		@return A FileSetMember object or null if no member
			of the specified type exists
	*/
	public FileSetMember getMember(DataFileType type)
	{
		if (type == null) return null;
		FileSetMemberData member = getData().getMembers().get(type.getData());
		if (member == null) return null;
		return getDbControl().getItem(FileSetMember.class, member);
	}
	
	/**
		Remove the member of the given type. If a {@link DataFileMetadataReader}
		is associated with the file type this method will call
		{@link DataFileMetadataReader#resetMetadata(DbControl)}.

		@param type A <code>DataFileType</code> object 
		@throws InvalidUseOfNullException If type is null
		@throws PermissionDeniedException If the logged in user
			doesn't have write permission for this file set
	*/
	public void removeMember(DataFileType type)
	{
		checkPermission(Permission.WRITE);
		if (type == null) throw new InvalidUseOfNullException("type");
		if (getData().getMembers().containsKey(type.getData()))
		{
			DbControl dc = getDbControl();
			FileSetMemberData member = getData().getMembers().remove(type.getData());
			if (member != null)
			{
				FileSetMember m = dc.getItem(FileSetMember.class, member);
				dc.deleteItem(m);
				DataFileMetadataReader metadataReader = type.getMetadataReader();
				if (metadataReader != null)
				{
					metadataReader.setItem(getItem());
					metadataReader.setFile(m);
					metadataReader.resetMetadata(dc);
				}
			}
		}
	}
	
	/**
		Remove all members in a file set.
		@throws PermissionDeniedException If the logged in user
			doesn't have write permission for this file set
	*/
	public void removeAllMembers()
	{
		checkPermission(Permission.WRITE);
		getData().getMembers().clear();
	}
	
	/**
		Get a query that return the {@link DataFileType} for all members
		of this file set.
		@return A query object
	*/
	public ItemQuery<DataFileType> getMemberTypes()
	{
		ItemQuery<DataFileType> query = DataFileType.getQuery();
		query.joinPermanent(Hql.innerJoin("fileSetMembers", Item.FILESETMEMBER.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property(Item.FILESETMEMBER.getAlias(), "fileSet"), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Checks if this file set contains a file for
		each {@link DataFileType} that is required according to
		what has been specified by the {@link Platform} and
		{@link PlatformVariant}.
		@return TRUE if all files are present, FALSE otherwise. If the
			items has no platform TRUE is always returned
		@see PlatformFileType#isRequired()
	*/
	public boolean hasAllRequiredFiles()
	{
		FileStoreEnabled item = getItem();
		Platform platform = item.getPlatform();
		PlatformVariant variant = item.getVariant();
		if (platform == null) return true;
		
		ItemQuery<PlatformFileType> query = platform.getFileTypes(variant, variant == null);
		query.restrict(
			Restrictions.eq(
				Hql.property("required"), 
				Expressions.parameter("isRequired", true, Type.BOOLEAN)
			)
		);
		
		Set<DataFileTypeData> memberTypes = getData().getMembers().keySet();
		for (PlatformFileType pft : query.list(getDbControl()))
		{
			if (!memberTypes.contains(pft.getData().getDataFileType()))
			{
				return false;
			}
		}
		return true;
	}
	
	/**
		Get a query that is returning all file types that
		have been marked as required by the associated platform but has no member
		in this file set.
		@return A query or null if no platform is associated with this
			item
	*/
	public ItemQuery<PlatformFileType> getMissingDataFileTypes()
	{
		FileStoreEnabled item = getItem();
		Platform platform = item.getPlatform();
		PlatformVariant variant = item.getVariant();
		if (platform == null) return null;

		ItemQuery<PlatformFileType> query = platform.getFileTypes(variant, variant == null);
		// Get all required platform file types
		query.restrict(
			Restrictions.eq(
				Hql.property("required"), 
				Expressions.parameter("isRequired", true, Type.BOOLEAN)
			)
		);

		Set<DataFileTypeData> memberTypes = getData().getMembers().keySet();
		Set<Integer> memberIds = new HashSet<Integer>(memberTypes.size());
		for (DataFileTypeData member : memberTypes)
		{
			memberIds.add(member.getId());
		}
		// Ignore all platform file types that we already have
		query.restrictPermanent(
			Restrictions.not(
				Restrictions.in(
					Hql.property("dataFileType.id"), 
					Expressions.parameter("members")
				)
			)
		);
		query.setPermanentParameter("members", memberIds, null);
		return query;
	}
	
	/**
		Validate the files in this file set. This method will 
		load the validator and metadata reader for the {@link DataFileType} 
		for each member in the file set. If more than one file type
		uses the same validator or metadata reader, they will all
		share the same instance. This method will not throw any errors
		that have been caused by the validation process. Errors are returned
		as a list of {@link Throwable}:s and is also reported by setting the 
		{@link FileSetMember#isValid()} and {@link FileSetMember#getErrorMessage()} 
		properties for each file.
		<p>
		Note! {@link PermissionDeniedException}:s are not considered validation errors.
		They are ignored and doesn't change the validation status.
		<p>
		{@link InvalidRelationException} is a less severe type of exception
		and can be used by a validator when the actual file it is validating
		is valid but a dependent file is not. For example, a valid CEL file has
		been attached to a raw bioassay, but it doesn't match the CDF file attached 
		to the array design. If a validator throws this exception the member status
		will be set to invalid, but metadata extraction is still done.
		
		@param dc The DbControl to use in the validation process
		@param extractMetadata TRUE to also extract metadata, FALSE otherwise
		@return A list of validation errors, if everything is successful the
			list is empty
	*/
	public List<Throwable> validate(DbControl dc, boolean extractMetadata)
	{
		checkPermission(Permission.WRITE);
		// Map: Classname --> DataFileHandler instance
		Map<String, DataFileHandlerWrapper> handlers = 
			new HashMap<String, DataFileHandlerWrapper>();
	
		// All validation errors
		List<Throwable> errors = new LinkedList<Throwable>();
		
		// First pass, create validator/metadata reader instances
		for (FileSetMemberData memberData : getData().getMembers().values())
		{
			ThreadSignalHandler.checkInterrupted();
			FileSetMember member = dc.getItem(FileSetMember.class, memberData);
			DataFileType dft = member.getDataFileType();
			String validatorClass = dft.getValidatorClass();
			String metadataReaderClass = dft.getMetadataReaderClass();
			DataFileHandlerWrapper validator = null;
			DataFileHandlerWrapper metadataReader = null;
			
			// Only one instance of each type is created
			if (validatorClass != null)
			{
				validator = handlers.get(validatorClass);
				if (validator == null)
				{
					validator = new DataFileHandlerWrapper(dft.getValidator(), getItem());
					handlers.put(validatorClass, validator);
				}
				validator.setFile(member);
			}
			if (extractMetadata && metadataReaderClass != null)
			{
				metadataReader = handlers.get(metadataReaderClass);
				if (metadataReader == null)
				{
					metadataReader = new DataFileHandlerWrapper(dft.getMetadataReader(), getItem());
					handlers.put(metadataReaderClass, metadataReader);
				}
				if (metadataReader != validator) metadataReader.setFile(member);
			}
		}
		
		// Second pass - validate and extract metadata
		for (DataFileHandlerWrapper handler : handlers.values())
		{
			ThreadSignalHandler.checkInterrupted();
			boolean ignore = false;
			boolean isValid = true;
			boolean invalidRelation = false;
			String errorMessage = null;
			if (handler.isValidator)
			{
				try
				{
					handler.validate(dc);
				}
				catch (Throwable t)
				{
					invalidRelation = t instanceof InvalidRelationException;
					ignore = t instanceof PermissionDeniedException;
					isValid = invalidRelation;
					errorMessage = t.getMessage();
					if (!ignore)
					{
						errors.add(t);
						if (!invalidRelation) handler.resetMetadata(dc);
					}
				}
			}
			if (extractMetadata && handler.isMetadataReader && isValid)
			{
				try
				{
					handler.extractMetadata(dc);
				}
				catch (Throwable t)
				{
					ignore = t instanceof PermissionDeniedException;
					isValid = false;
					errorMessage = t.getMessage();
					if (!ignore)
					{
						errors.add(t);
						handler.resetMetadata(dc);
					}
				}
			}
			if (!ignore) handler.setValid(isValid, errorMessage);
		}
		return errors;
	}
	
	
	private static class QueryRuntimeFilterImpl
		implements QueryRuntimeFilter
	{
		public void enableFilters(QueryRuntimeFilterManager manager, EntityQuery query, DbControl dc)
		{
			// Do not add any filters.
		}
	}
	
	
	/**
		Wraps a {@link DataFileValidator} and /or {@link DataFileMetadataReader}.
		as a single item. This class takes care of calling the validator/metadata reader
		and to set validation status and error messages on the associated file set members.
	*/
	private static class DataFileHandlerWrapper
	{
		private final DataFileHandler handler;
		private final boolean isValidator;
		private final boolean isMetadataReader;
		private final FileStoreEnabled item;
		private final List<FileSetMember> files;
		
		DataFileHandlerWrapper(DataFileHandler handler, FileStoreEnabled item)
		{
			this.handler = handler;
			this.item = item;
			this.isValidator = handler instanceof DataFileValidator;
			this.isMetadataReader = handler instanceof DataFileMetadataReader;
			this.handler.setItem(item);
			this.files = new ArrayList<FileSetMember>();
		}
		
		void setFile(FileSetMember member)
		{
			handler.setFile(member);
			this.files.add(member);
		}
		
		void validate(DbControl dc)
		{
			if (isValidator) ((DataFileValidator)handler).validate(dc);
		}
		
		void extractMetadata(DbControl dc)
		{
			if (isMetadataReader) ((DataFileMetadataReader)handler).extractMetadata(dc);
		}
		
		void resetMetadata(DbControl dc)
		{
			if (isMetadataReader) ((DataFileMetadataReader)handler).resetMetadata(dc);
		}
		
		void setValid(Boolean isValid, String errorMessage)
		{
			for (FileSetMember member : files)
			{
				member.getData().setValid(isValid);
				member.getData().setErrorMessage(errorMessage);
			}
		}
		
	}
	
}
