/*
	$Id: Help.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.HelpData;
import net.sf.basedb.core.data.SharedData;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;

/**
	This class is used to represent a help text for
	a client application.

	@author Nicklas
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@version 2.0
*/
public class Help
	extends ChildItem<HelpData>
	implements Nameable
{
	/**
		The type of item represented by this class.
		@see Item#HELP
		@see #getType()
	*/
	public static final Item TYPE = Item.HELP;

	/**
		The maximum length of the external id variable that can be stored in the
		database. Check the length against this value before calling the
		{@link #setExternalId(String)} method to avoid
		exceptions.
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = HelpData.MAX_EXTERNAL_ID_LENGTH;

	/**
		This filter will only return items if the logged in user has
		generic read permission to clients.
	*/
	private static final QueryRuntimeFilter RUNTIME_FILTER = 
		new QueryRuntimeFilterFactory.ChildFilter(TYPE, Item.CLIENT);
	
	/**
		Create a new <code>Help</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param client The <code>Client</code> which this help text is for
		@param externalId A unique string used for quick retrieval of a specific help text
		@return The new <code>ClientDefaultSetting</code> item
		@throws BaseException If there is an error
		@see Client#newHelp(String)
	*/
	public static Help getNew(DbControl dc, Client client, String externalId)
		throws BaseException
	{
		Help h = dc.newItem(Help.class);
		h.setClient(client);
		h.setExternalId(externalId);
		h.setName("New help");
		return h;
	}

	/**
		Get a <code>Help</code> item when you know the ID.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>Help</code> item
		@throws ItemNotFoundException If an item with the specified 
			ID is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static Help getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		Help h = dc.loadItem(Help.class, id);
		if (h == null) throw new ItemNotFoundException("Help[id="+id+"]");
		return h;
	}

	/**
		Get a query configured to retrieve 
		help texts for the specified client application.
		
		@param client The client to retreive help for, null is allowed if
			the logged in user has generic READ permission for clients in which case
			all help will be returned
		@return An {@link ItemQuery} object
		@see Client#getHelp()
	*/
	public static ItemQuery<Help> getQuery(Client client)
	{
		ItemQuery<Help> query = null;
		if (client != null)
		{
			query = new ItemQuery<Help>(Help.class, null);
			query.restrictPermanent(
				Restrictions.eq(
					Hql.property("client"), 
					Hql.entity(client)
				)
			);
		}
		else
		{
			query = new ItemQuery<Help>(Help.class, RUNTIME_FILTER);
		}
		return query;
	}

	/**
		Creates a new help item
		
		@param data the data
	*/
	Help(HelpData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	public String getName()
	{
		return getData().getName();
	}
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	public String getDescription()
	{
		return getData().getDescription();
	}
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the ChildItem class
		-------------------------------------------
	*/
	SharedData getSharedParent()
	{
		return getData().getClient();
	}
	// -------------------------------------------

	/**
		Get the client application this setting is valid for.
		@return The <code>Client</code> item
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission
		@throws BaseException If there is another error
	*/
	public Client getClient()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Client.class, getData().getClient());
	}
	
	/**
		Set the client application this setting is valid for. Can only be set 
		on a new item.
		@throws InvalidDataException If the client is null
	*/
	void setClient(Client client)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (client == null) throw new InvalidUseOfNullException("client");
		getData().setClient(client.getData());
	}

	/**
		Get the external id for this <code>Help</code> item.
		@return A string with the external id of this item
	*/
	public String getExternalId()
	{
		return getData().getExternalId();
	}
	/**
		Set the external id for this <code>Help</code> item. The value
		cannot be null and must not be longer than the value specified by
		the {@link #MAX_EXTERNAL_ID_LENGTH} constant. It must also be a value
		that is unique for the client application.
		@param externalId The new external id for this item
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@throws InvalidDataException If the new value is null or longer than
			{@link #MAX_EXTERNAL_ID_LENGTH}
	*/
	public void setExternalId(String externalId)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setExternalId(StringUtil.setNotNullString(externalId, "externalId", MAX_EXTERNAL_ID_LENGTH));
	}

}

