/*
  $Id: Location.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import java.util.HashMap;
import java.util.Map;

/**
	This enumeration defines constants that are used to specify different
	file storage locations.

	@author enell
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
 */
public enum Location
{
	/**
		The item is offline and doesn't use any quota.
	*/
	OFFLINE(0, "Offline"),
	/**
		The item is stored in primary storage.
	*/
	PRIMARY(1, "Primary"),
	/**
		The item is stored in secondary storage. Note! This type
		of storage may not be available on all servers.
	*/
	SECONDARY(2, "Secondary");
	
	/**
		Maps an integer to a location.
	*/
	private static final Map<Integer, Location> valueMapping = new HashMap<Integer, Location>();
	
	static
	{
		for(Location location : Location.values())
		{
			Location l = valueMapping.put(location.getValue(), location);
			assert l == null : "Another location with the value "+location.getValue()+" already exists";
		}
	}
	
	/**
		The integer value of this location. 
	*/
	private final int value;
	/**
		Returned by the toString method.
	*/
	private final String displayValue;

	/**
	 	Creates a new location. Used internal only.
	 	
		@param value The integer value of this location.
	*/
	private Location(int value, String displayValue)
	{
		this.value = value;
		this.displayValue = displayValue;
	}
	
	public String toString()
	{
		return displayValue;
	}
	
	/**
		Get the <code>Location</code> object when you know the integer code.
		
		@param value The integer value.
		@return The location for the integer value.
	*/
	public static Location fromValue(int value)
	{
		Location location = valueMapping.get(value);
		assert location != null : "location == null for value "+value;
		return location;
	}
	
	/**
		Get the integer value that is used when storing an location to the database.
		
		@return The integer value for this location.
	*/
	public int getValue()
	{
		return value;
	}
}
