/*
	$Id: PropertyFilter.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import net.sf.basedb.core.data.PropertyFilterData;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Hql;

/**
	Used to store filter settings in the current context. A filter is
	a simple expression of the form: <code>property operator value</code>,
	ie. <code>name = 'Nicklas'</code> or <code>spots &gt; 10000</code>.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see SessionControl#getCurrentContext(Item)
	@see ItemContext
*/
public class PropertyFilter
{

	/**
		The maximum allowed length of a filter value.
		@since 2.6.2
	*/
	public static final int MAX_VALUE_LENGTH = PropertyFilterData.MAX_VALUE_LENGTH;
	
	/**
	 	The string/characters used as a separator in a string with more then one filter values.
	 */
	private final String value_separator = "|";
	
	private final String property;
	private Type valueType;
	private Operator operator;
	private String value;
	
	
	/**
		Create a new filter for a property.
		@param property The name of the property
	*/
	public PropertyFilter(String property)
	{
		this.property = property;
	}

	/**
		Create a new filter for a property.
		@param property The name of the property
		@param operator The operator to use
		@param value The value to check against
		@param valueType The type of the value
	*/
	public PropertyFilter(String property, Operator operator, String value, Type valueType)
	{
		this.property = property;
		this.operator = operator;
		this.value = value;
		this.valueType = valueType;
	}
	
	/**
		Create a new filter from a loaded database object.
	*/
	PropertyFilter(String property, PropertyFilterData filter)
	{
		this.property = property;
		this.valueType = Type.fromValue(filter.getValueType());
		this.operator = Operator.fromValue(filter.getOperator());
		this.value = filter.getValue();
	}
	
	/**
		Create a new database filter object from this filter. 
	*/
	PropertyFilterData getData()
	{
		PropertyFilterData filter = new PropertyFilterData();
		filter.setValueType(valueType.getValue());
		filter.setOperator(operator.getValue());
		filter.setValue(value);
		return filter;
	}
	
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return property + " " + operator + " " + value;
	}
	// -------------------------------------------

	
	/**
		Get the name of the property this filter is used on.
	*/
	public String getProperty()
	{
		return property;
	}
	
	/**
		Get the type of the property.
	*/
	public Type getValueType()
	{
		return valueType;
	}
	/**
		Set the type of the property.
		@param valueType The type
	*/
	public void setValueType(Type valueType)
	{
		this.valueType = valueType;
	}
	
	/**
		Get the operator to use in the filter.
	*/
	public Operator getOperator()
	{
		return operator;
	}
	/**
		Set the operator to use in the filter.
	*/
	public void setOperator(Operator operator)
	{
		this.operator = operator;
	}
	
	/**
		Get the value to use in the filter.
		@return The value as a string
	*/
	public String getValue()
	{
		return value;
	}
	/**
		Set the value to use in the filter.
		@param value The value as a string
	*/
	public void setValue(String value)
	{
		this.value = value;
	}
	
	/**
		Get the value as an object of the correct type.
		The {@link Type#parseString(String)} is used to convert
		the value.
		@return The value as an object
		@throws InvalidDataException If the string value can't be parsed
	*/
	public Object getValueAsObject()
	{
		return valueType.parseString(value);
	}
	
	/**
	 	Splits the value string at each "|" and creates
	 	a list with objects from the substrings that were found.
	 	@return A list with objects, or null if splitting string of values returns null.
	 	@throws InvalidDataException If any of the string values can't be parsed.
	 	@since 2.5 
	 */
	public List<Object> getValuesAsObjects()
	{
		String[] values = value.split("\\"+getValueSeparator(), -1);
		if (values == null) return null;
		List<Object> objects = new ArrayList<Object>(values.length);
		for (String v : values)
		{
			objects.add(v.length() == 0 ? null : valueType.parseString(v));
		}
		return objects;
	}
	
	/**
		This method will fail if used to filter on reporter lists.
		@deprecated Use {@link #getRestriction(DbControl)} instead
	*/
	public Restriction getRestriction()
	{
		return getRestriction(null);
	}

	/**
		Create a restriction from the filter. There are a few special cases:
		<p>
		If the property starts with a $ it is interpreted as a <code>alias.property</code> 
		value, ignoring the root alias of the query. The alias is removed from the
		property before proceeding.
		<p>
		If the property starts with a # or ## it is a filter on annotations. The rest of
		the property is the ID of the annotation type, and we use a
		{@link AnnotationSimpleRestriction} object for the restriction. If there
		is only one # in the prefix only direct annotations are searched, if there
		are two ##, inherited annotations are also searched.
		<p>
		If the property starts with a , the rest of the property is the path that
		joins the {@link ReporterData#getReporterListScores()} collection. The filter
		value is the ID of the reporter list.
		<p>
		If the property starts with a @ the property is a collection of values and
		we always use an IN restriction, ignoring the specified operator.
		<p>
		The property may also start with $@.
		@return The restriction
		@since 2.8
	*/
	public Restriction getRestriction(DbControl dc)
	{
		
		Restriction restriction = null;
		String property = getProperty();
		String alias = null;
		
		if (property != null && property.startsWith("$"))
		{
			// Property is alias.property or alias only
			int dotIndex = property.indexOf(".");
			if (dotIndex == -1)
			{
				// Alias only
				alias = property.substring(1);
				property = null;
			}
			else if (property != null)
			{
				// Alias.property
				alias = property.substring(1, dotIndex);
				property = property.substring(dotIndex+1);
			}
		}
		
		if (property != null && property.startsWith("#"))
		{
			// Property is id of annotation type: #id
			// If prefixed with two ## inherited annotations should be searched
			boolean searchInherited = property.startsWith("##");
			int annotationTypeId = (Integer)Type.INT.parseString(property.substring(searchInherited ? 2 : 1));
			if (operator.isListOperator())
			{
				restriction = new AnnotationSimpleRestriction(alias,annotationTypeId, getValueType(), 
					getOperator(), getValuesAsObjects(), searchInherited);
			}
			else
			{
				Object value = getValueAsObject();
				Operator operator = getOperator();
				if (value == null && (operator == Operator.EQ || operator == Operator.NEQ))
				{
					restriction = new HasAnnotationRestriction(alias, annotationTypeId, 
						operator == Operator.EQ, searchInherited);
				}
				else
				{
					restriction = new AnnotationSimpleRestriction(alias, annotationTypeId, getValueType(), 
							operator, value, searchInherited);
				}
			}
		}
		
		else if (property != null && property.startsWith(""))
		{
			// Property is reporterListScores collection
			// The value is the ID of the reporter list to
			// filter on
			Integer reporterListId = (Integer)getValueAsObject();
			if (reporterListId != null)
			{
				try
				{
					ReporterList list = ReporterList.getById(dc, reporterListId);
					if (operator == Operator.NEQ)
					{
						restriction = Hql.isNotPartOf(alias, property.substring(1), list);
					}
					else
					{
						restriction = Hql.isPartOf(alias, property.substring(1), list);
					}
				}
				catch (ItemNotFoundException ex)
				{}
			}
		}
		
		else
		{
			String parameterName = "p" + System.identityHashCode(this);
			Expression parameter = (getValue() == null || operator.isListOperator()) ? null : 
				Expressions.parameter(parameterName, getValueAsObject(), getValueType());

			if (property != null && property.startsWith("@"))
			{
				// Property is a collection - we only support equality tests
				property = property.substring(1);
				Expression elements = Hql.elements(alias, property);
				Restriction isEmpty = Restrictions.eq(Hql.size(alias, property), Expressions.integer(0));
				if (operator.isListOperator())
				{
					// filter to match one of several elements in a collection
					List<Restriction> alternatives = new ArrayList<Restriction>();
					List<Object> objects = getValuesAsObjects();
					boolean hasEmptyCheck = false;
					for (Object obj : objects)
					{
						if (obj != null)
						{
							parameterName = "p" + System.identityHashCode(obj);
							Expression e = Expressions.parameter(parameterName, obj, getValueType());
							alternatives.add(Restrictions.in(e, elements));
						}
						else if (!hasEmptyCheck)
						{
							hasEmptyCheck = true;
							alternatives.add(isEmpty);
						}
					}
					restriction = Restrictions.or(alternatives.toArray(new Restriction[alternatives.size()]));
				}
				else if (parameter != null)
				{
					// filter for a single element in a collection
					restriction = Restrictions.in(parameter, elements);
				}
				else
				{
					// filter that returns items with empty collections
					restriction = isEmpty;
				}
			}
			else
			{
				Expression propertyExpression = Hql.property(alias, property);
				if (getValueType() == Type.DATE)
				{
					if (!operator.isListOperator())
					{
						// To use ==, != and <= filter on dates we must add 24 hours to it
						// and use different restrictions
						Date filter = (Date)getValueAsObject();
						Date filterPlus24H = new Date(filter.getTime()+24*3600*1000);
						Expression parameterPlus24H = 
							Expressions.parameter(parameterName+"24h", filterPlus24H, Type.DATE);
						if (operator == Operator.EQ)
						{
							// date <= property < date + 24H
							restriction = Restrictions.between(propertyExpression, parameter, parameterPlus24H);
						}
						else if (operator == Operator.NEQ)
						{
							// NOT date <= property < date + 24H
							restriction = Restrictions.not(Restrictions.between(propertyExpression, parameter, parameterPlus24H));
						}
						else if (operator == Operator.LTEQ)
						{
							// property < date + 24H
							restriction = Restrictions.lt(propertyExpression, parameterPlus24H);
						}
						else if (operator == Operator.GT)
						{
							// property >= date + 24H
							restriction = Restrictions.gteq(propertyExpression, parameterPlus24H);
						}						
					}
					else 
					{
						List<Object> objects = getValuesAsObjects();
						List<Restriction> restrictions = new ArrayList<Restriction>(objects.size());
						for (Object obj : objects)
						{
							Date filter = (Date)obj;
							Date filterPlus24H = new Date(filter.getTime()+24*3600*1000);
							parameterName = "p" + System.identityHashCode(obj);
							Expression parameterPlus24H = 
									Expressions.parameter(parameterName+"24h", filterPlus24H, Type.DATE);
							Expression dateParameter = Expressions.parameter(parameterName, obj, getValueType());
							restrictions.add(Restrictions.between(propertyExpression, dateParameter, parameterPlus24H));
						}
						if (operator == Operator.IN) 
						{
							restriction = Restrictions.or(restrictions.toArray(new Restriction[restrictions.size()]));
						}
						else if (operator == Operator.NOTIN)
						{
							restriction = Restrictions.not(Restrictions.or(restrictions.toArray(new Restriction[restrictions.size()])));
						}
					}
				}
				if (restriction == null)
				{
					if (operator.isListOperator())
					{
						List<Expression> parameters = new ArrayList<Expression>();
						List<Object> objects = getValuesAsObjects();
						for (Object obj : objects)
						{
							Expression e = null;
							if (obj != null)
							{
								parameterName = "p" + System.identityHashCode(obj);
								e = Expressions.parameter(parameterName, obj, getValueType());
							}
							parameters.add(e);
						}
						restriction = operator.getRestriction(propertyExpression, parameters.toArray(new Expression[parameters.size()]));
					}
					else
					{
						restriction = operator.getRestriction(propertyExpression, parameter);
					}
				}
			}
		}
		return restriction;
	}

	public Restriction getDynamicRestriction(DbControl dc)
	{
		Restriction restriction = null;
		String property = getProperty();
		Expression propertyExpression = null;
		if (property.startsWith(""))
		{
			// Property is reporterListScores collection
			// The value is the ID of the reporter list to
			// filter on
			Integer reporterListId = (Integer)getValueAsObject();
			if (reporterListId != null)
			{
				try
				{
					ReporterList list = ReporterList.getById(dc, reporterListId);
					if (operator == Operator.NEQ)
					{
						restriction = Dynamic.isNotPartOf(list);
					}
					else
					{
						restriction = Dynamic.isPartOf(list);
					}
				}
				catch (ItemNotFoundException ex)
				{}
			}
			return restriction;
		}
		else
		{
			propertyExpression = ItemContext.getDynamicExpression(dc, property);
		}
		String parameterName = "p" + System.identityHashCode(this);
		
		String value = getValue();
		Expression parameter = null;
		List<Expression> parameters = null;
		// Experimental - allows expressions like: >ch1
		if (value != null && (valueType == Type.FLOAT || valueType == Type.DOUBLE || valueType == Type.INT || valueType == Type.LONG))
		{
			if (!getOperator().isListOperator() && value.matches("^ch\\d+$"))
			{
				int channel = Integer.parseInt(value.substring(2));
				parameter = Dynamic.column(VirtualColumn.channel(channel));
			}
			else if (getOperator().isListOperator())
			{
				String[] values = value.split("\\"+getValueSeparator());
				for (String s : values)
				{
					if (s.matches("^ch\\d+$"))
					{
						int channel = Integer.parseInt(s.substring(2));
						if (parameters == null)
						{
							parameters = new ArrayList<Expression>();
						}
						parameters.add(Dynamic.column(VirtualColumn.channel(channel)));
					}
				}
			}			
		}
		if (parameter == null && parameters == null)
		{
			if (getOperator().isListOperator())
			{
				List<Object> objects = getValuesAsObjects();				
				parameters = objects == null ? null : new ArrayList<Expression>(objects.size());
				for (Object obj : objects)
				{
					Expression e = null;
					if (obj != null)
					{
						parameterName = "p" + System.identityHashCode(obj);
						e = Expressions.parameter(parameterName,obj);
					}
					parameters.add(e);
				}				
			}
			else
			{
				parameter = value == null ? null : Expressions.parameter(parameterName, getValueAsObject());
			}
		}
		if (propertyExpression != null)
		{
			if (parameters != null)
			{
				restriction = operator.getRestriction(propertyExpression, parameters.toArray(new Expression[parameters.size()]));
			}
			else
			{
				restriction = operator.getRestriction(propertyExpression, parameter);
			}
		}
		return restriction;
	}
	
	public String getValueSeparator()
	{
		return value_separator;
	}

}
