/*
	$Id: SecondaryStorageController.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

/**
	This interface defines the methods that needs to be implemented 
	by a class to be able to use it as a controller for seconday storage.
	<p>
	The core will create one object instance of the class specified by 
	the configuration setting <code>secondary.storage.driver</code>. Leaving
	this setting empty disables the secondary storage.
	<p>
	When the object has been create the {@link #init(String)} method
	is called to let the object initialise itself. The parameters for the
	method call is taken from the <code>secondary.storage.init</code> setting.
	<p>
	The {@link #run()} method is invoked with an interval specified by the 
	<code>secondary.storage.interval</code> setting. The interval is given
	in seconds. Alternatively the setting <code>secondary.storage.time</code>
	may contain a comma-separated list of time-points (hour:minutes) when the storage 
	controller should be invoked. If time-points are given it overrides the interval
	setting which is ignored. Time-point valus must be given with time two-digit 24-based
	hour and two-digit minute values. For example: 03:10,09:00,23:59.
	<p>
	When BASE is stopped the {@link #close()} method is called.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public interface SecondaryStorageController
{
	/**
		Initialises the object. The BASE core calls this method immediatly
		after the object has been created. The contents and syntax of the <code>settings</code>
		string should be defined by the implementing class. It may for
		example be the path to another directory of information about how to
		connect to a remote FTP server.

		@param settings A string containing initialisation parameters
		@throws BaseException If the parameters are invalid
	*/
	public void init(String settings)
		throws BaseException;

	/**
	 	Run the storage controller. It should:
		<ul>
		<li>move files to the secondary storage if {@link File#getAction()} returns
			{@link File.Action#MOVE_TO_SECONDARY}
		<li>restore files to the primary storage if {@link File#getAction()} returns
			{@link File.Action#MOVE_TO_PRIMARY}
		<li>delete all files from the seconday storage which doesn't have a corresponding 
			{@link File} item in the database (because it have been deleted)
		</ul>
		@throws BaseException
	*/
	public void run()
		throws BaseException;
	
	/**
		The core is now done with the storage controller. Clean up and close.
	*/
	public void close();
	
}
