/**
	$Id: Trashcan.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.BasicData;
import net.sf.basedb.core.data.RemovableData;
import net.sf.basedb.core.query.ResultList;
import net.sf.basedb.core.signal.ThreadSignalHandler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.hibernate.HibernateException;

/**
	This class contains functionality to manage items that 
 	have been marked for deletion.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class Trashcan
{

	/**
		Log trashcan events.
	*/
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.Trashcan");
	
	private static final boolean isDebug = log.isDebugEnabled();

	/**
		@see #getItems(DbControl, Set, int, int)
	*/
	public static ResultList<Removable> getItems(DbControl dc, Item itemType, int firstItem, int maxItems)
		throws BaseException
	{
		return getItems(dc, itemType == null ? null : Collections.singleton(itemType), 
			firstItem, maxItems);
	}

	/**
		Load all items flagged for deletion that are owned by the logged in user.

		@param dc The <code>DbControl</code> to use for database access
		@param itemTypes Limit the list to only return items of the types,
			included in the set, use null or an empty set to return items of all types
		@param firstItem The index of the first item to return (0-based)
		@param maxItems The maximum number of items to return, or 0 to return all items
		@return A list containing removable items
	 	@throws BaseException If there is an error of some kind
	 	@since 2.7.1
	*/
	public static ResultList<Removable> getItems(DbControl dc, Set<Item> itemTypes, 
		int firstItem, int maxItems)
		throws BaseException
	{
		try
		{
			org.hibernate.Session session = dc.getHibernateSession();
			SessionControl sc = dc.getSessionControl();
			
			List<RemovableData> removed = null;
			
			// Only load items owned by the logged in user
			org.hibernate.Filter filter = HibernateUtil.enableFilter(session, "ownedBy");
			filter.setParameter("owner", dc.getSessionControl().getLoggedInUserId());
			
			if (itemTypes != null && itemTypes.size() > 0)
			{
				removed = new ArrayList<RemovableData>();
				for (Item item : itemTypes)
				{
					if (RemovableData.class.isAssignableFrom(item.getDataClass()))
					{
						org.hibernate.Query query = HibernateUtil.createQuery(session, 
							"SELECT trash FROM " + item.getDataClass().getName() + " trash " +
							"WHERE trash.removed = TRUE");
						removed.addAll(HibernateUtil.loadList(RemovableData.class, query, sc));
					}
				}
			}
			else
			{
				org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, "GET_REMOVED_ITEMS");
				/*
					SELECT trash
					FROM RemovableData trash
					WHERE trash.removed
				*/
				removed = HibernateUtil.loadList(RemovableData.class, query, sc);
			}
			HibernateUtil.disableFilter(session, "ownedBy");
			
			List<Removable> items = new ArrayList<Removable>(removed.size());
			int index = 0;
			int totalItems = 0;
			boolean useLimit = firstItem >= 0 && maxItems > 0;
			int lastItem = firstItem + maxItems;
			for (RemovableData rd : removed)
			{
				BasicData data = (BasicData)rd;
				try
				{
					BasicItem item = dc.getItem(BasicItem.class, data);
					if (item.hasPermission(Permission.DELETE)) 
					{
						// The 'ownedBy' filter isn't supported on ChildItem:s
						// so we have to check the owner of parent item
						SharedItem parent = item instanceof ChildItem ? 
								dc.getItem(SharedItem.class, ((ChildItem)item).getSharedParent()) : null; 
						
						if (parent == null || parent.isOwner())
						{
							if (!useLimit || (index >= firstItem && index < lastItem))
							{
								items.add((Removable)item);
							}
							index++;
							totalItems++;
						}
					}
				}
				catch (PermissionDeniedException ex)
				{}
			}
			return new UnmodifiableResultList<Removable>(items, totalItems);
		}
		catch (HibernateException ex)
		{
			throw new BaseException(ex);
		}
	}
	
	/**
		Delete items permanently. This method only delete items that
		<ul>
		<li>have been marked for deletion (unless the ignoreFlag parameter is set)
		<li>the logged in user has permission to delete
		<li>not are used by other items
		</ul>
		Other items are ignored. This method may have to open several transactions
		to be able to delete as many items as possible. This means that this operation
		is not atomic and if an exception is thrown some items may have been deleted
		and some may not.
		
		@param sc The session control to use for connecting to the database
		@param items The items to remove
		@param ignoreFlag TRUE to also remove items that hasn't been flagged for removal
		@param progress An optional progress reporter
		@return The number of deleted items
	 	@throws BaseException If an item can't be deleted. 
		@since 2.4
	*/
	public static int delete(SessionControl sc, Collection<? extends Identifiable> items, 
		boolean ignoreFlag, ProgressReporter progress)
		throws BaseException
	{
		ThreadSignalHandler.checkInterrupted();
		if (progress != null) progress.display(0, "Checking which items to remove...");
		
		// Copy items to a set with predictable iteration order
		Set<Identifiable> itemsToRemove = new TreeSet<Identifiable>(Item.DELETION_ORDER);
		for (Identifiable item : items)
		{
			if (item.hasPermission(Permission.DELETE) &&
				(ignoreFlag || (item instanceof Removable && ((Removable)item).isRemoved())))
			{
				itemsToRemove.add(item);
			}
		}
		
		if (isDebug)
		{
			log.debug("Original list: " + items);
			log.debug("Sorted list: " + itemsToRemove);
		}
		
		if (itemsToRemove.size() == 0) return 0;
		int numToRemove = itemsToRemove.size();
		float progressFactor = 95.0f / numToRemove;
		int numRemoved = 0;
		int removedInTransaction = 0;

		DbControl dc = null;
		// We try to delete items from the set as long as there are items
		// left and at least one item was deleted in the last iteration
		do
		{
			ThreadSignalHandler.checkInterrupted();
			dc = sc.newDbControl();
			log.debug("Start new delete transaction");
			try
			{
				removedInTransaction = 0;
				Iterator<Identifiable> iterator = itemsToRemove.iterator();
				deleteTransaction:
				while (iterator.hasNext())
				{
					ThreadSignalHandler.checkInterrupted();
					Identifiable item = iterator.next();
					if (progress != null)
					{
						String name = item instanceof Nameable ? ((Nameable)item).getName() : item.toString();
						progress.display((int)(numRemoved * progressFactor), "Remove " +
								item.getType() + ": " + name + 
								" (" + (numRemoved + 1) + " of " + numToRemove + ")");
					}
					BasicItem basicItem = item.getType().getById(dc, item.getId());
					if (!basicItem.isUsed()) 
					{
						if (isDebug) log.debug("Deleting item " + basicItem);
						dc.deleteItem(basicItem);
						numRemoved++;
						removedInTransaction++;
						iterator.remove();
					}
					else 
					{
						if (isDebug) log.debug("Item is used: " + basicItem);
						if (progress != null) progress.append(" [USED]");
						if (removedInTransaction > 0)
						{
							// Commit, start new transaction and try again
							break deleteTransaction;
						}
					}
				}
				log.debug("Committing transaction; " + removedInTransaction + 
					" items scheduled for deletion; " + numRemoved + " total so far; " +
					itemsToRemove.size() + " remains in list");
				if (progress != null)
				{
					progress.display((int)(numRemoved * progressFactor), 
						numRemoved + " of " + numToRemove + " items removed; flushing to database");
				}
				dc.commit();
			}
			finally
			{
				if (dc != null) dc.close();
			}
		} while (removedInTransaction > 0 && itemsToRemove.size() > 0);
		
		if (progress != null)
		{
			String msg = "";
			if (numRemoved < numToRemove)
			{
				msg = numRemoved + " of " + numToRemove + " items removed. " + 
					(numToRemove - numRemoved) + " items are used and can't be removed";
			}
			else
			{
				msg = "All " + numToRemove + " items removed";
			}
			progress.display(100, msg);
		}
		return numRemoved;
	}
	
	/**
		Delete all items in the trashcan of the logged in user. This method
		calls {@link #getItems(DbControl, Item, int, int)} to load all items
		that have been flagged for removal and then 
		{@link #delete(SessionControl, Collection, boolean, ProgressReporter)}
		to remove those items. See the <code>delete</code> for
		more information about transactions.
		
		@param sc The session control to use for connecting to the database
		@param progress An optional progress reporter
		@return The number of deleted items
		@since 2.4
	*/
	public static int deleteAll(SessionControl sc, ProgressReporter progress)
	{
		DbControl dc = sc.newDbControl();
		try
		{
			List<Removable> items = getItems(dc, (Set<Item>)null, 0, 0);
			dc.close();
			return delete(sc, items, false, progress);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
}
