/*
	$Id: POP3Authenticator.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.authentication;

import java.net.Socket;
import java.io.*;


/**
	This class is an example implementation of the {@link Authenticator}
	interface that authenticates a user against a POP3 server. This means
	that all users having an email account on a specified mail server are
	able to log in to base using the same username and password as they use
	to read their mails.
	<p>
	<b>Example</b>
<pre class="code">
String host = "mail.thep.lu.se";
String login = "nicklas";
String password = "secret";

Authenticator a = new POP3Authenticator();
a.init(host);
if (a.authenticate(login, password))
{
   // Do some stuff that only nicklas is allowed to do!
}
</pre>
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class POP3Authenticator
	implements Authenticator
{

	/**
		The address or IP-number to the POP3 host.
	*/
	private String host = null;

	/**
		The POP3 port to use. The default port is 110.
	*/
	private int port = 110;

	/**
		Create a new <code>POP3Authenticator</code> object.
	*/
	public POP3Authenticator()
	{
		super();
	}

	/**
		Initialises the object. The <code>settings</code> string should be the address or
		IP-number to the POP3 server. Optionally, the name/number may be followed by
		a colon (:) and a port number. For example: <code>pop.yourdomain.com</code>
		or <code>pop.yourdomain.com:110</code>
		@param settings The host name with an optional port number
		@throws AuthenticationException This exception is thrown if the format
			of the <code>settings</code> string is invalid. No check is made that
			the host actually exists and answers at the specified port.
	*/
	public void init(String settings)
		throws AuthenticationException
	{
		int i = settings.indexOf(':');
		if (i >= 0)
		{
			host = settings.substring(0, i-1);
			try
			{
				port = Integer.parseInt(settings.substring(i+1));
			}
			catch (NumberFormatException ex)
			{
				throw new AuthenticationException("Invalid host name: "+settings);
			}
		}
		else
		{
			host = settings;
		}
	}

	/**
		Always return FALSE.
	*/
	public boolean supportsExtraInformation()
	{
		return false;
	}

	/**
		Authenticates a user against the POP3 server.
		@param login The login name of the user
		@param password The password for the user
		@return An {@link AuthenticationInformation}
			object. The {@link AuthenticationInformation#id id} is the same as
			the {@link AuthenticationInformation#login login}. No extra information
			is returned.
	*/
	public AuthenticationInformation authenticate(String login, String password)
		throws UnknownLoginException, InvalidPasswordException, AuthenticationException
	{
		AuthenticationInformation ai = null;
		AuthenticationException error = null;
		try
		{
			/*
				A typical POP3 session:

				1. <connect>
				2. +OK
				3. USER <login>
				4. +OK
				5. PASS <password>
				6. +OK
				7. QUIT
				8. +OK
				9. <disconnect>

				- or -

				4. -ERR Unknown user
				5. QUIT
				6. +OK
				7. <disconnect>

				- or -

				6. -ERR Invalid password
				7. QUIT
				8. +OK
				9. <disconnect>
			*/
			Socket pop = new Socket(host, port);
			BufferedReader in = new BufferedReader(new InputStreamReader(pop.getInputStream()));
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(pop.getOutputStream()));

			String answer = in.readLine();
			if (answer != null && answer.startsWith("+OK"))
			{
				answer = send("USER "+login, out, in);
				if (answer.startsWith("+OK"))
				{
					answer = send("PASS "+password, out, in);
					if (answer.startsWith("+OK"))
					{
						ai = new AuthenticationInformation(login, login);
					}
					else
					{
						answer = answer.replaceFirst("\\-ERR ", "");
						error = new InvalidPasswordException(login, answer);		// Invalid password
					}
				}
				else
				{
					answer = answer.replaceFirst("\\-ERR ", "");
					error = new UnknownLoginException(login, answer);		// Unknown login
				}
			}
			else
			{
				answer = answer == null ? "NULL" : answer.replaceFirst("\\-ERR ", "");
				error = new AuthenticationException(answer);		// Unknown login
			}
			answer = send("QUIT", out, in);
			pop.close();
		}
		catch (Exception ex)
		{
			throw new AuthenticationException(ex);
		}
		if (error != null) throw error;
		return ai;
	}

	/**
		Send <code>cmd</code> to the <code>out</code> buffer. Read a single line 
		of response from the <code>in</code> buffer.
		@return The response from the POP server
	*/
	private String send(String cmd, BufferedWriter out, BufferedReader in)
		throws Exception
	{
		out.write(cmd);
		out.write("\r\n");
		out.flush();
		return in.readLine();
	}

}
