/**
	$Id: CaseExpression.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.core.query;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;

/**
	An SQL case expression:
	<pre class="code">
CASE
   WHEN r1 THEN e1
   WHEN r2 THEN e2
   ...
   ELSE elseExpression
END
</pre> 

	@author nicklas
	@version 2.0
	@see Expressions#caseWhen(Expression, WhenStatement[])
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class CaseExpression
	implements Expression
{

	private final Expression elseExpression;
	private final WhenStatement[] whenStatements;
	
	CaseExpression(Expression elseExpression, WhenStatement... whenStatements)
	{
		this.elseExpression = elseExpression;
		this.whenStatements = whenStatements;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc) 
		throws BaseException
	{
		StringBuilder sb = new StringBuilder();
		sb.append("CASE\n");
		for (WhenStatement when : whenStatements)
		{
			sb.append("WHEN ").append(when.getCondition().toQl(query, dc));
			sb.append(" THEN ").append(when.getValue().toQl(query, dc));
			sb.append("\n");
		}
		if (elseExpression != null)
		{
			sb.append("ELSE ").append(elseExpression.toQl(query, dc));
			sb.append("\n");
		}
		sb.append("END\n");
		return sb.toString();
	}
	/**
		@return All expressions and restrictions used
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		List<QueryElement> all = new ArrayList<QueryElement>(1 + whenStatements.length * 2);
		for (WhenStatement when : whenStatements)
		{
			all.add(when.getCondition());
			all.add(when.getValue());
		}
		if (elseExpression != null) all.add(elseExpression);
		return Collections.unmodifiableList(all);
	}
	
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		StringBuilder sb = new StringBuilder();
		sb.append("CASE\n");
		for (WhenStatement when : whenStatements)
		{
			sb.append("WHEN ").append(when.getCondition().toString());
			sb.append(" THEN ").append(when.getValue().toString());
			sb.append("\n");
		}
		if (elseExpression != null)
		{
			sb.append("ELSE ").append(elseExpression.toString());
			sb.append("\n");
		}
		sb.append("END\n");
		return sb.toString();
	}
	// -------------------------------------------

}
