/**
	$Id: DelegatingSignalHandler.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.signal;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
	A signal handler implementation that can delegate to other signal handlers,
	depending on which signal it receives. Multiple signal handlers may be registered
	for each signal. The order in which they are called is not specified, but it is
	probably different from the order they are registered.
	
	@author nicklas
	@version 2.6
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class DelegatingSignalHandler
	extends AbstractSignalHandler
{
	/**
		Log signals processing.
	*/
	private static final org.apache.log4j.Logger logger = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.signal.DelegatingSignalHandler");
	
	/**
		Holds the registered signal handlers.
	*/
	private Map<Signal, Set<SignalHandler>> handlers;
	
	/**
		Create a new delegating signal handler. Signal handlers to
		delegate to should be registered with 
		{@link #registerSignalHandler(SignalHandler)}.
	*/
	public DelegatingSignalHandler()
	{
		super();
	}
	
	/*
		From the SignalHandler interface
		-------------------------------------------
	*/
	/**
		@return A collection containing all signals that have at least one 
			registered handler
	*/
	@Override
	public Collection<Signal> getSupportedSignals()
	{
		return Collections.unmodifiableSet(handlers.keySet());
	}
	
	/**
		Check if at least one handler has been registered for the given signal.
	*/
	@Override
	public boolean supports(Signal signal)
	{
		return handlers.containsKey(signal);
	}
	
	/**
		Handler the given signal by invoking {@link SignalHandler#handleSignal(Signal)}
		on each registered signal handler. The order of the invokation is not
		predictable.
		@throws UnsupportedSignalException If no handler has been registered for the
			given signal
	*/
	public void handleSignal(Signal signal)
	{
		logger.debug("Got signal: " + signal);
		Set<SignalHandler> all = handlers.get(signal);
		if (all == null) 
		{
			logger.debug("Signal not supported " + signal);
			throw new UnsupportedSignalException(signal);
		}
		for (SignalHandler handler : all)
		{
			logger.debug("Sending signal " + signal + " to: " + handler);
			handler.handleSignal(signal);
		}
	}
	// -------------------------------------------

	/**
		Register a signal handler to delegate to. The handler is registered
		for all signals it supports.
		@param handler The handler to register
	*/
	public void registerSignalHandler(SignalHandler handler)
	{
		logger.debug("Registering signal handler: " + handler);
		for (Signal signal : handler.getSupportedSignals())
		{
			Set<SignalHandler> temp = handlers.get(signal);
			if (temp == null)
			{
				temp = new HashSet<SignalHandler>();
				handlers.put(signal, temp);
			}
			temp.add(handler);
		}
	}
	
	/**
		Unregister a signal handler.
		@param handler The handler to unregister
	*/
	public void unregisterSignalHandler(SignalHandler handler)
	{
		logger.debug("Unregistering signal handler: " + handler);
		Iterator<Map.Entry<Signal, Set<SignalHandler>>> it = handlers.entrySet().iterator();
		while (it.hasNext())
		{
			Map.Entry<Signal, Set<SignalHandler>> entry = it.next();
			Set<SignalHandler> registered = entry.getValue();
			// Remove the registered handler...
			if (registered.remove(handler))
			{
				// ... if no more handlers for this signal, remove the entry
				if (registered.size() == 0) it.remove();
			}
		}
	}

}
