/**
	$Id: ThreadSignalHandler.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.signal;

import java.util.Collections;
import java.util.Set;

/**
	An implementation of a signal handler that uses the {@link Thread} class
	to communicate signals back to the {@link SignalTarget}. This handler only
	supports the {@link Signal#ABORT} signal. When this signal is received, 
	{@link Thread#interrupt()} is called on the thread that was registered with 
	this handler. This should typically be a worker thread for some process.
	The worker thread should regularly check {@link Thread#interrupted()}
	to see if it has been interrupted. If the thread may be waiting for a blocking
	call to end, it should be able to handle the resulting {@link InterruptedException}
	that is thrown if the thread is interrupted. 
	<p>
	If the worker thread becomes interrupted it should stop what
	it is doing, cleanup any allocated resources and exit in a timely fasion.
	Here is a general outline:
	
	<pre class="code">
// ... code in worker thread
threadSignalHandler.setWorkerThread(null);
beginTransaction();
boolean done = false;
boolean interrupted = false;
while (!done && !interrupted)
{
   try
   {
      done = doSomeWork(); // NOTE! This must not take forever!
      interrupted = Thread.interrupted();
   }
   catch (InterruptedException ex)
   {
      // NOTE! Try-catch is only needed if thread calls a blocking method 
      interrupted = true;
   }
}
if (interrupted)
{
   rollbackTransaction();
}
else
{
   commitTransaction();
}
</pre>
	
	@author nicklas
	@version 2.6
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class ThreadSignalHandler
	extends AbstractSignalHandler
{

	/**
		Utility method to check if the current thread has been interrupted and throws
		a SignalException if it has. 
	*/
	public static void checkInterrupted()
	{
		if (Thread.interrupted()) throw new SignalException("Aborted by user");
	}
	
	
	/**
		Log signals processing.
	*/
	private static final org.apache.log4j.Logger logger = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.signal.ThreadSignalHandler");

	
	private static final Set<Signal> supported = Collections.singleton(Signal.ABORT);

	/**
		The worker thread that should be interrupted when {@link Signal#ABORT} is
		received.
	*/
	private Thread workerThread;
	
	/**
		If TRUE, call Thread.stop() instead of Thread.interrupt()
	*/
	private boolean forceStop;
	
	/**
		Create a new thread signal handler. It will interrupt the
		current thread when it recieves the {@link Signal#ABORT} signal.
	*/
	public ThreadSignalHandler()
	{
		this(null);
	}
	
	/**
		Create a new thread signal handler.
		@param workerThread The worker thread to interrupt when it receieves the 
			{@link Signal#ABORT} signal, or null to interrupt the current thread
	*/
	public ThreadSignalHandler(Thread workerThread)
	{
		super(supported);
		setWorkerThread(workerThread);
	}
	
	/*
		From the SignalHandler interface
		-------------------------------------------
	*/
	/**
		Only handles the {@link Signal#ABORT} signal. When receieved, 
		{@link Thread#interrupt()} is called on the worker thread that was
		registered when this object was created. Note that this method
		called from a different thread. The worker thread is busy doing something 
		else. If the worker thread has already ended, this method does nothing.
		@param signal The signal, only {@link Signal#ABORT} is accepted
		@throws UnsupportedSignalException If signal is anything else than
			{@link Signal#ABORT}
	*/
	@SuppressWarnings("deprecation")
	public void handleSignal(Signal signal)
	{
		logger.debug("Got signal: " + signal);
		if (!workerThread.isAlive()) return;
		if (Signal.ABORT.equals(signal))
		{
			logger.debug("Sending signal " + signal + " to thread: " + workerThread);
			if (forceStop)
			{
				workerThread.stop(new InterruptedException("Received signal: ABORT"));
			}
			else
			{
				workerThread.interrupt();
			}
		}
		else
		{
			logger.debug("Signal not supported " + signal);
			throw new UnsupportedSignalException(signal);
		}
	}
	// -------------------------------------------

	/**
		Set the worker thread that should be interrupted when a signal is
		receiver.
		@param workerThread The worker thread, or null to use the current thread
	*/
	public void setWorkerThread(Thread workerThread)
	{
		this.workerThread = workerThread == null ? Thread.currentThread() : workerThread;
	}
	
	/**
		Call this method with a true value to make the signal handler
		use {@link Thread#stop(Throwable)} instead of {@link Thread#interrupt()}.
		Note that the <code>Thread.stop</code> method is deprecated and it is not
		recommended that it is used. For some plug-ins though this may be the
		only option, if they are mainly executing code outside of their control
		which doesn't check the {@link Thread#interrupted()} status. The exception
		passed to the thread will be a {@link InterruptedException}.
		@param forceStop TRUE to stop the thread by force, FALSE to signal to
			it with a flag
	*/
	public void setForceStop(boolean forceStop)
	{
		this.forceStop = forceStop;
	}
	
}
