/**
	$Id: PluginConfigInfo.java 4479 2008-09-05 15:16:27Z jari $
	
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

package net.sf.basedb.util;


import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.jdom.Document;
import org.jdom.Element;

/**
 This class should be used to keep information about a plugin configuration 
 that is loaded from an XML-file. It has a useful method to get all 
 <code>PlguinConfigInfo</code> from an XML-document for a certain plugin.
 @author martin
 @version 2.5
 */
public class PluginConfigInfo
{
	/**
	 	The plugin the configuration can be use with
	 */
	private String pluginClass;
	
	/**
	 	The name of the configuration
	 */
	private String name;
	
	/**
	 	Description about the configuration
	 */
	private String description;
	
	/**
	 	In which place the configuration comes in the file.
	 	This is <0 if it hasn't been defined.
	 */
	private int orderInXml = -1;
	
	
	/**
	 	Constructor to create a new instance with information for a plugin configuration.
	  	@param pluginClassName The class name of PluginDefinition this configuration belongs to, 
	  		see{@link net.sf.basedb.core.PluginConfiguration#getPluginDefinition()}
	  	@param configName Name of the plugin configuration, 
	  		see{@link net.sf.basedb.core.PluginConfiguration#getName()}
	  	@param descr Description about the configuration 
	  		see {@link net.sf.basedb.core.PluginConfiguration#getDescription()}
	 */
	public PluginConfigInfo(String pluginClassName, String configName, String descr)
	{
		this.pluginClass = pluginClassName;
		this.name = configName;
		this.description = descr;
	}

	/**
	 	Loads PluginConfigInfo objects from xml-document for a plugin.
	  	@param doc The xml-document
	  	@param pluginClassName The specific 
	  	@return A list with <code>PluginConfigInfo</code> or, 
	  		null if no configurations were found in the file.
	 */
	public static List<PluginConfigInfo> loadConfigurationsForPlugin(Document doc, String pluginClassName)
	{
		List<PluginConfigInfo> configInfo = new ArrayList<PluginConfigInfo>();		
		ListIterator configs = doc.getRootElement().getChildren().listIterator();		
		while (configs.hasNext())
		{			
			int order = configs.nextIndex();
			Object obj = configs.next();			
			if (Element.class.isInstance(obj))
			{
				Element e = (Element)obj;
				if (e.getAttributeValue("pluginClassName").equals(pluginClassName))
				{
					String configurationName = e.getChildText("configname");
					if (pluginClassName != null && configurationName != null)
					{
						PluginConfigInfo info = new PluginConfigInfo(pluginClassName, configurationName, e.getChildText("description"));
						info.orderInXml = order;
						configInfo.add(info);
					}
				}
			}
		}
		return configInfo;
	}
	
	/**
	 	Gets the class name of the plugin that this configuration is ment for
	 	@return String plugin class name.
	 */
	public String getPluginClass()
	{
		return pluginClass;
	}
	
	/**
	 	Gets the configuration's name
	  	@return String name
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 	Gets the description of the configuration
	  	@return String description
	 */
	public String getDescription()
	{
		return description;
	}

	/**	Gets in wich order the configuration comes in the import file.
	 	@return Returns order/position in the xml file or <0 if not defined.
	*/
	public int getOrderInXml()
	{
		return orderInXml;
	}
}
