/**
	$Id: PluginInfo.java 4479 2008-09-05 15:16:27Z jari $
	
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

package net.sf.basedb.util;

import net.sf.basedb.core.Application;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.Plugin;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;

/**
 	A <code>PluginInfo</code> object is used to keep information about 
 	external plugin located in a jar file. 	 
  	@author Martin
  	@version 2.5
  	@base.modified $Date$
 */
public class PluginInfo
{
	/**
	 	The path to the jarfile the plugin is located in.
	 */
	private String jarPath;
	
	/**
	 	The classname of the plugin
	 */
	private String className;
	
	/**
	 	The BASE version that the plugin at least requires
	 */
	private String minBaseVersion;
	
	/**
	 	The about information from the plugin
	 */
	private About about;
	
	/**
	 	The list with belonging configurations
	 */
	private List<PluginConfigInfo> configs;
	
	/**	
	 	The name of a XML-file that holds information 
	 	about each plugin class in the jar file.
	 */
	private static final String xmlFile = "META-INF/base-plugins.xml";
	
	/**
	 	The path in a jar-file to the file that holds 
	 	configurations to import
	 */
	private static final String configurationImportFile = "META-INF/base-configurations.xml";
	
	/**
		Creates a new <code>PluginInfo</code> object.
	  	@param jarPath The path to the jar file in which the class is located.
	  	@param className The plugin's class name 
	  	@param minBaseVersion BASE version that the plugin at least requires
	  	@param about Information about the plugin.
	  	@param configs A list with configurations for this plugin.
	 */
	public PluginInfo (String jarPath, String className, String minBaseVersion, About about, List<PluginConfigInfo> configs)
	{
		this.jarPath = jarPath;
		this.className = className;
		this.minBaseVersion = minBaseVersion;
		this.about = about;
		this.configs = configs;
	}
	
	/**
	 	Gets a list with <code>PluginInfo</code> objects, one for every plugin listed 
	 	in the base-plugins.xml file, which is located in the jar-file's META-INF 
	 	directory. Plugins which class file couldn't be found or the class doesn't implement 
	 	{@link net.sf.basedb.core.plugin.Plugin} will not be included in the list but already 
	 	installed plugins will.
	 	@param jarFile The jar file to load the plugin info from. 
	 	@return List with PluginInfo about the plugins that were found in jar file.
	  	@throws JDOMException If any error loading the XML-document
	*/
	@SuppressWarnings("unchecked")
	public static List<PluginInfo> loadFromJar(java.io.File jarFile)
		throws JDOMException
	{
		final String dtdFile = "base-plugins.dtd";
		final String configDtdFile ="plugin-configuration-file.dtd";
		final String hasConfigsValue = "yes";
		final String configXMLFile = "base-configurations.xml";
		
		List<PluginInfo> pluginInfos = new ArrayList<PluginInfo>();
		InputStream is = null;
		
		try
		{			
			JarFile jar = new JarFile(jarFile);
			JarEntry basePluginsXml = jar.getJarEntry(xmlFile);

			//Get the info in base-plugins.xml file.
			//Nothing is done if the xml-file is missing.
			if (basePluginsXml != null)
			{
				is = jar.getInputStream(basePluginsXml);
				URL installDtdURL = PluginInfo.class.getResource("/net/sf/basedb/core/dtd/" + dtdFile);
				Element rootElement = XMLUtil.getValidatedXML(is , installDtdURL, null).getRootElement();
				String jarname = rootElement.getAttributeValue("jarname");
				ClassLoader jarLoader = JarClassLoader.newInstance(jarFile.getPath());
				if (!jarname.equalsIgnoreCase(jarFile.getName()))
				{
					throw new InvalidDataException("Did not find the expected jar file: found '" + 
							jarFile.getName() + "' but expected: '" + jarname + "'");
				}
				
				//Loop through each classelement in the xml file and create a PluginInfo-object
				//if the class implements Plugin-interface.
				List<Element> classElements = (List<Element>)rootElement.getChildren();
				for (Element classElement : classElements)
				{			
					String classname = classElement.getAttributeValue("classname");
					String baseVersion = classElement.getChildText("minbaseversion");
					String configElement = classElement.getChildText("hasconfigurations");
					boolean hasConfigs = (configElement != null) && configElement.equals(hasConfigsValue);
									
					Class c = jarLoader.loadClass(classname);				
					if (versionCompatible(baseVersion) && Plugin.class.isAssignableFrom(c))
					{			
						Plugin pl = (Plugin)c.newInstance();
						About about = pl.getAbout();
						List<PluginConfigInfo> configs = null;
						
						//Get info about configurations if there are any.
						if (hasConfigs)
						{
							InputStream confStream = null;
							try
							{
								confStream = jar.getInputStream(new JarEntry("META-INF/" + configXMLFile));
								if (confStream != null)
								{
									URL configDtdURL = PluginInfo.class.getResource("/net/sf/basedb/core/dtd/" + configDtdFile);
									Document configurationDoc = XMLUtil.getValidatedXML(confStream, configDtdURL, null);
									configs = PluginConfigInfo.loadConfigurationsForPlugin(configurationDoc, classname);
								}
							}
							catch (IOException ex)
							{}
							finally
							{
								if (confStream != null) confStream.close();
							}
						}					
						pluginInfos.add(new PluginInfo(jarFile.getPath(), classname, baseVersion, about, configs));					
					}				
				}
			}
		}
		catch(Throwable ex)
		{
			throw new BaseException(ex);
		}
		finally
		{
			try
			{
				if (is != null) is.close();
			}
			catch(IOException ex)
			{}
		}
		return pluginInfos;
	}
	
	/**
	 	Gets the class name for the plugin
	  	@return String A class name like 'net.sf.basedb.classname'
	 */
	public String getClassName()
	{
		return className;
	}
	
	/**
		Gets the name of the jar file holding this plugin
	  	@return String Name of the jar file
	 */
	public String getJarPath()
	{
		return jarPath;
	}
	
	/**
	 	Gets the lowest version of BASE that the plugin is compatible with
	  	@return String Base version
	 */
	public String getMinBaseVersion()
	{
		return minBaseVersion;
	}
	
	/**
	 	Gets the about information that is set for this plugin 
	  	@return About The about information that is in the plugin's about object.
	 */
	public About getAbout()
	{
		return about;
	}
	
	/**
	 	Gets information about the configurations that come together with the plugin in the jar.
	  	@return A list with information about each plugin configuration.
	 */
	public List<PluginConfigInfo> getConfigurations()
	{
		return configs;
	}
	
	/**
	 	The path inside a jar to the xml-file that holds 
	 	the pluginconfigurations that can be imported.
	  	@return String format path.
	 */
	public String getConfigurationsPath()
	{
		return configurationImportFile;
	}
	
	/**
		Gets if a version is compatible with current 
		major.minor-version of BASE
	  	@param version The version to test
	  	@return True if the version is compatible with current BASE version,
	  	 	FALSE otherwise.
	 */
	private static boolean versionCompatible(String version)
	{
		int majorVersion = Integer.valueOf(version.substring(0, (version.indexOf('.'))));
		int minorVersion = Integer.valueOf(version.substring(version.indexOf('.')+1));
		boolean majorCompatible = !(Integer.valueOf(Application.getMajorVersion()) < majorVersion); 
		boolean minorCompatible = !(Integer.valueOf(Application.getMinorVersion()) < minorVersion);
		
		return majorCompatible && minorCompatible;
	}
}