/**
	$Id: QueryParameters.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
	Utiltity class for parsing string values in the form of 
	HTTP request query parameters:
	<p>
	<code>key1=value1&key2=value2...</code>
	
	@author nicklas
	@version 2.6
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class QueryParameters
{
	/**
		Parses a string in HTTP query format (<code>key1=value1&key2=value2...</code>)
		and returns it as a QueryParameters object.
		
		@param query The string to parse
		@return A {@link QueryParameters} object
	*/
	public static QueryParameters parseQueryString(String query)
	{
		return new QueryParameters(query);
	}

	private Map<String, List<String>> parameters;
		
	private QueryParameters(String query)
	{
		parameters = new HashMap<String, List<String>>();
		if (query != null)
		{
			Pattern p = Pattern.compile("([^=]+)=([^&]+)&?");
			Matcher m = p.matcher(query);
			while (m.find())
			{
				String key = m.group(1);
				String value = m.group(2);
				List<String> values = parameters.get(key);
				if (values == null)
				{
					values = new ArrayList<String>();
					parameters.put(key, values);
				}
				values.add(value);
			}
		}	
	}
	
	/**
		Get the (single) value for a key. If the query string contained 
		more than one value for this key, the first value is returned.
		@param key The key
		@return The value, or null if no value was found for the key
	*/
	public String getValue(String key)
	{
		List<String> values = getValues(key);
		if (values != null && values.size() > 0)
		{
			return values.get(0);
		}
		else
		{
			return null;
		}
	}
	
	/**
		Get all values for a key. 
		@return A list of values, or null if no value was found for the key
	*/
	public List<String> getValues(String key)
	{
		return parameters.get(key);
	}
	
	/**
		Get the number of values for a key.
		@param key The key
		@return The number of values
	*/
	public int getNumValues(String key)
	{
		List<String> values = getValues(key);
		return values == null ? 0 : values.size();
	}
	
	/**
		Get all keys that are present in the query string.
		@return A set with the keys
	*/
	public Set<String> getKeys()
	{
		return parameters.keySet();
	}
	
}
