/**
	$Id:ExtensionsInvoker.java 4187 2008-03-20 11:15:25Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.extensions;

import java.util.Collection;
import java.util.Iterator;

/**
	Object of this class handles a single invokation of the extensions
	for one or several extension points. Call 
	{@link Registry#useExtensions(ClientContext, ExtensionsFilter, String...)}
	to create an invoker object. With this object you can
	{@link #iterate()} over all actions created by the extensions, or
	use one of the render methods to render all actions automatically.

	<p>
	The {@link #renderDefault()} method uses the default render objects 
	registered with the extension points and/or extensions (see 
	{@link ExtensionPoint#getRendererFactory()} and {@link Extension#getRendererFactory()}).
	Since the registration of rendering factories are optional, the
	<code>renderDefault()</code> method only works with extensions that have
	done so. If an action can't be associated with a renderer, an exception
	is thrown.

	<p>
	If there is no default rendering factory or if you want to override the default
	renderer, you can use the {@link #render(Renderer)} method. This will force
	the use of the specified renderer for all extensions, ignoring all renderers
	set by extension points or extensions.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class ExtensionsInvoker<A extends Action>
{

	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.util.extensions.ExtensionsInvoker");
	
	final Collection<InvokationContext<A>> contexts;
	
	/**
		Creates a new invoker object.
	*/
	ExtensionsInvoker(Collection<InvokationContext<A>> contexts)
	{
		this.contexts = contexts;
	}
	
	/**
		Create an iterator that iterates over all {@link Action}:s created
		by the extensions. The iterator may return zero actions if none
		has been created. 
		@return An iterator
	*/
	public ActionIterator<A> iterate()
	{
		return new ActionIterator<A>(contexts.iterator());
	}
	
	/**
		Render all actions using the renderers created by the
		default renderer factory as specified by the registered 
		extension points and/or extensions.
		@see ExtensionPoint#getRendererFactory()
		@see Extension#getRendererFactory()
	*/
	public void renderDefault()
	{
		ActionIterator<A> it = iterate();
		while (it.hasNext())
		{
			A action = null;
			Renderer<? super A> renderer = null;
			try
			{
				action = it.next();
				renderer = it.getRenderer();
				if (renderer == null) 
				{
					throw new NullPointerException("No renderer for extension: " + it.getExtension());
				}
				renderer.render(action);
			}
			catch (Throwable t)
			{
				log.error("Could not render action '" + action + "' with renderer '" + 
						renderer + "'", t);
			}
		}
	}
	
	/**
		Render all actions using a specific renderer. The given 
		renderer will be used even if an extension point has provided
		a renderer factory and disallowed overriding it.
		
		@see ExtensionPoint#getRendererFactory()
		@see ExtensionPoint#allowRendererOverride()
	*/
	public void render(Renderer<A> renderer)
	{
		Iterator<A> it = iterate();
		while (it.hasNext())
		{
			A action = null;
			try
			{
				action = it.next();
				renderer.render(action);
			}
			catch (Throwable t)
			{
				log.error("Could not render action '" + action + "' with renderer '" + 
						renderer + "'", t);
			}
		}
	}
}
