/*
	$Id: Jep.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.jep;

import java.util.Collection;
import java.util.Vector;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.util.Enumeration;
import net.sf.basedb.util.jep.convert.ConverterFactory;
import net.sf.basedb.util.jep.convert.JepConversionFunction;

import org.nfunk.jep.JEP;
import org.nfunk.jep.Node;
import org.nfunk.jep.ASTVarNode;
import org.nfunk.jep.ASTConstant;

/**
	Utility class for parsing mathematical expressions. This class uses
	the Java Mathematical Expression Parser package from Singular Systems to
	parse mathematical expressions.
	<p>
	Create a new {@link JEP} expression with the {@link #newJep(String, JepFunction[])}
	method. If the expression contains variables set values for those with the
	{@link JEP#setVarValue(String, Object)} method. Then, evaluate the expression
	using the {@link JEP#getValue()} method.
	<p>
	You can also convert the expression to a Query API {@link Expression}
	object with the {@link #jepToExpression(JEP)} method. The conversion
	doesn't support all methods since the Query API doesn't have support for
	them.
	<p>
	See the <a href="http://www.singularsys.com/jep/">JEP homepage</a> for more
	information about JEP and all supported functions/operators. 
	See {@link #jepToExpression(JEP)} and {@link #jepToRestriction(JEP)} 
	for functions/operators supported by the conversion methods.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see <a href="http://www.singularsys.com/jep/">Java Mathematical Expression Parser package</a>
*/
public class Jep
{

	private static Enumeration<String, String> functions;
	private static Enumeration<String, String> operators;
	
	/**
		Create a new JEP expression. Evaluate the expression using
		the {@link JEP#getValue()} method. Set values for variables
		with the {@link JEP#setVarValue(String, Object)}. 
		
		@param formula The expression to parse
		@param functions Extra functions that are required to parse the
			expression
		@return A <code>JEP</code> object
		@throws BaseException If the expression couldn't be parsed
	*/
	public static JEP newJep(String formula, JepFunction... functions)
		throws BaseException
	{
		formula = formula.replace("'", "\"");
	
		JEP jep = new JEP();
		jep.setAllowUndeclared(true);
		jep.addStandardFunctions();
		addFunctions(jep, Log2Function.getInstance());
		if (functions != null) addFunctions(jep, functions);
		jep.parseExpression(formula);
		if (jep.hasError())
		{
			throw new BaseException(jep.getErrorInfo());
		}
		return jep;
	}
	
	private static void addFunctions(JEP jep, JepFunction... functions)
	{
		for (JepFunction function : functions)
		{
			if (function != null) jep.addFunction(function.getFunctionName(), function);
		}
	}
	
	/**
		Get a list of built-in functions supported by the {@link #jepToExpression(JEP)}
		converter. More functions can be registered when creating the JEP
		expression (see {@link #newJep(String, JepFunction[])}).
		
		@return An enumeration with the name of the function in the key 
			and a description in the value part
	*/
	public static Enumeration<String, String> getFunctions()
	{
		if (functions == null)
		{
			functions = new Enumeration<String, String>();
			// Built-in converters
			Collection<JepConversionFunction<?>> converters = ConverterFactory.getConverters();
			for (JepConversionFunction converter : converters)
			{
				if (converter.isFunction())
				{
					functions.add(converter.getName(), converter.getDescription());
				}
			}
			// Other standard functions that we always add; see newJep() method
			functions.add("log2", "2-based logarithm");
			functions.sortKeys();
			functions.lock();
		}
		return functions;
	}
	
	/**
		Get a list of built-in operators supported by the {@link #jepToExpression(JEP)}
		and {@link #jepToRestriction(JEP)} converter. It is not possible to
		register more operators.
		
		@return An enumeration with the symbol of the operator in the key 
			and a description in the value part
		@since 2.4
	*/
	public static Enumeration<String, String> getOperators()
	{
		if (operators == null)
		{
			operators = new Enumeration<String, String>();
			// Built-in converters
			Collection<JepConversionFunction<?>> converters = ConverterFactory.getConverters();
			for (JepConversionFunction converter : converters)
			{
				if (converter.isOperator())
				{
					operators.add(converter.getName(), converter.getDescription());
				}
			}
			operators.sortKeys();
			operators.lock();
		}
		return operators;
	}

	
	/**
		Convert a JEP expression to a Query API {@link Expression}. The Query API
		only supports a subset of the functions/operators that JEP supports.
		Supported operators are:

		<pre class="code">
+      {@link Expressions#add(Expression, Expression)}
-      {@link Expressions#subtract(Expression, Expression)}
*      {@link Expressions#multiply(Expression, Expression)}
/      {@link Expressions#divide(Expression, Expression)}
-      {@link Expressions#negate(Expression)}
log    {@link Expressions#log10(Expression)}
log2   {@link Expressions#log2(Expression)}
ln     {@link Expressions#ln(Expression)}
sqrt   {@link Expressions#sqrt(Expression)}
abs    {@link Expressions#abs(Expression)}
exp    {@link Expressions#exp(Expression)}
raw    {@link Dynamic#rawData(String)} *
ch     {@link Dynamic#column(VirtualColumn)} *
<i>number</i> {@link Expressions#integer(int)} or
       {@link Expressions#aFloat(float)}
<i>string</i> {@link Expressions#parameter(String, Object)}
<i>variable</i> {@link Expressions#parameter(String)} **
</pre>
		* = This is a optional function that must be registered when creating the 
		JEP object.
		<br>
		** = The NULL string is converted to a NULL value
		
		@param jep The JEP object
		@return An <code>Expression</code> object
		@throws BaseException If the JEP expression can't be converted
		@see #newJep(String, JepFunction[])
		@see #getFunctions()
	*/
	public static Expression jepToExpression(JEP jep)
		throws BaseException
	{
		return nodeToExpression(jep.getTopNode());
	}
	
	/**
		Convert JEP expression to a Query API {@link Expression}. This method
		is equivalent to: <code>jepToExpression(newJep(formula, functions))</code>.
		@see #jepToExpression(JEP)
	*/
	public static Expression formulaToExpression(String formula, JepFunction... functions)
		throws BaseException
	{
		return jepToExpression(newJep(formula, functions));
	}
	
	/**
		Convert a JEP expression to a Query API {@link Restriction}. The
		Query API only supports a subset of the functions/operators that
		JEP supports. 
		Supported functions/operators are:
		<pre class="code">
==      {@link Restrictions#eq(Expression, Expression)}
!=      {@link Restrictions#neq(Expression, Expression)}
&gt;       {@link Restrictions#gt(Expression, Expression)}
&gt;=      {@link Restrictions#gteq(Expression, Expression)}
&lt;       {@link Restrictions#lt(Expression, Expression)}
&lt;=      {@link Restrictions#lteq(Expression, Expression)}
>       {@link Restrictions#gt(Expression, Expression)}
&&      {@link Restrictions#and(Restriction[])}
||      {@link Restrictions#or(Restriction[])}
!       {@link Restrictions#not(Restriction)}
</pre>
		* = This is a optional function that must be registered when creating the JEP object.
		<p>
		All operators supported by the {@link #jepToExpression(JEP)} are of
		course also supported in the appropriate places.
		
		@param jep The JEP object
		@return A <code>Restriction</code> object
		@throws BaseException If the JEP expression can't be converted
		@see #newJep(String, JepFunction[])
	*/
	public static Restriction jepToRestriction(JEP jep)
		throws BaseException
	{
		return nodeToRestriction(jep.getTopNode());
	}
	
	/**
		Convert JEP expression to a Query API {@link Restriction}. This method
		is equivalent to: <code>jepToRestriction(newJep(formula, functions))</code>.
		@see #jepToRestriction(JEP)
	*/
	public static Restriction formulaToRestriction(String formula, JepFunction... functions)
		throws BaseException
	{
		return jepToRestriction(newJep(formula, functions));
	}

	/**
		Convert a node with it's children to an expression.
	 	@param node Node to convert to an expression
	 	@return A {@link net.sf.basedb.core.query.Expression} object
	 	@throws BaseException If the JEP function in node not is supported.
	*/
	@SuppressWarnings("unchecked")
	public static Expression nodeToExpression(Node node)
		throws BaseException
	{
		JepConversionFunction converter = ConverterFactory.getConverterByClass(node);
		Expression e = null;
		if (converter != null)
		{
			e = converter.toExpression(node);
		}
		else
		{
			throw new BaseException("Unsupported JEP function: " + node);
		}
		return e;
	}
	
	/**
		Convert a node with it's children to a restriction.
	 	@param node JEP node to convert. 
	 	@return A {@link net.sf.basedb.core.query.Restriction} object
	 	@throws BaseException If the JEP function in node is not supported by the converter.
		@since 2.4
	*/
	@SuppressWarnings("unchecked")
	public static Restriction nodeToRestriction(Node node)
		throws BaseException
	{
		JepConversionFunction converter = ConverterFactory.getConverterByClass(node);
		Restriction r = null;
		if (converter != null)
		{
			r = converter.toRestriction(node);
		}
		else
		{
			throw new BaseException("Unsupported JEP function: " + node);
		}
		return r;
	}
	
	/**
		Convert a node to a string value. Supported node types are
		constants or variables. Constants will be converted to strings with the
		toString() method. For variables the name is returned as the string. 
		This allows for unquoted strings in expressions. The NULL string
		is converted to a NULL value.
	 	@param node JEP node to convert. 
	 	@return A String object or null if the node does not have any value.
	 	@throws BaseException If the node could not be converted in some way.
	*/
	public static String nodeToString(Node node)
		throws BaseException
	{
		if (node instanceof ASTConstant)
		{
			ASTConstant constNode = (ASTConstant)node;
			Object value = constNode.getValue();
			return value == null ? null : value.toString();
		}
		else if (node instanceof ASTVarNode)
		{
			ASTVarNode varNode = (ASTVarNode)node;
			String name = varNode.getName();
			if ("NULL".equalsIgnoreCase(name)) name = null;
			return name;
		}
		throw new BaseException("Expected string expression: " + node);
	}
	
	/**
		Convert a node to an integer value. Supported node types are
		constants.
	 	@param node JEP node to convert. 
	 	@return an int object
	 	@throws BaseException If the node could not be converted.
	*/
	public static int nodeToInt(Node node)
		throws BaseException
	{
		if (node instanceof ASTConstant)
		{
			ASTConstant constNode = (ASTConstant)node;
			Object value = constNode.getValue();
			if (value instanceof Number)
			{
				return ((Number)value).intValue();
			}
		}
		throw new BaseException("Expected integer expression: " + node);
	}
	
	private static java.lang.reflect.Field jepErrorList = null;
	static
	{
		try
		{
			jepErrorList = JEP.class.getDeclaredField("errorList");
			jepErrorList.setAccessible(true);
		}
		catch (Throwable t)
		{}
	}
	
	/**
		This is a workaround for a bug in the JEP error handling. Before calling
		{@link JEP#getValueAsObject()} the error list should be cleared.
		@param jep
	*/
	public static void clearErrorList(JEP jep)
	{
		try
		{
			Vector v = (Vector)jepErrorList.get(jep);
			v.clear();
		}
		catch (Throwable t)
		{}
	}
	

}
