/*
	$Id: ScoreFunction.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.jep;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ReporterList;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.util.BioAssaySetUtil;

import java.sql.SQLException;
import java.util.Stack;

import org.nfunk.jep.Node;
import org.nfunk.jep.ParseException;

/**
	A JEP function class that adds a <code>score(reporterListId)</code> function to a 
	JEP expression parser. The function will look up a reporters score in the
	reporter list with the ID given as an argument to this function.
	<p>
	To be able to use this function it must be registered with the JEP
	parser and, before the expression is evaluated, a {@link SqlResult} object
	must be set.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see Jep
	@see BioAssaySetUtil#createJepExpression(DbControl, BioAssaySet, String, boolean)
*/
public class ScoreFunction
	implements JepExpressionFunction
{

	private DbControl dc;
	private int scoreIndex;
	private SqlResult result;
	private int numParameters;

	/**
		Create a new instance of this function. The new instance cannot be used
		to dynamically evaluate expressions. It should only be used for converting
		JEP formulas to {@link Expression}:s.
	 	@param dc DbControl to use when this ScoreFunction accessing the database. 
		@see Jep#formulaToExpression(String, JepFunction[])	
	*/
	public ScoreFunction(DbControl dc)
	{
		this.dc = dc;
	}
	
	/**
		Create a new instance of this function which can be used
		to dynamically evaluate expressions.
	 	@param dc DbControl to use when this ScoreFunction accessing the database.
		@param scoreIndex The column index in the SqlResult that holds the score value
		@see #setSqlResult(SqlResult)
	*/
	public ScoreFunction(DbControl dc, int scoreIndex)
	{
		this.dc = dc;
		this.scoreIndex = scoreIndex;
	}
	
	/*
		From the JepFunction interface
		-------------------------------------------
	*/
	/**
		@return The string "score"
	*/
	public String getFunctionName()
	{
		return "score";
	}
	// -------------------------------------------
	/*
		From the JepExpressionFunction interface
		-------------------------------------------
	*/
	/**
		Use the {@link Dynamic#score(ReporterList)} method to create an
		expression referencing the score of a reporter in the reporter list.
	*/
	public Expression toExpression(Node node)
	{
		int numChildren = node.jjtGetNumChildren();
		if (numChildren != 1)
		{
			throw new BaseException("Invalid number of expressions for 'score' function: " + numChildren);
		}
		int reporterListId = Jep.nodeToInt(node.jjtGetChild(0));
		ReporterList list = ReporterList.getById(dc, reporterListId);
		return Dynamic.score(list);
	}
	// -------------------------------------------
	/*
		From the PostfixMathCommandI interface
		-------------------------------------------
	*/
	/**
		@return Always 1
	*/
	public int getNumberOfParameters()
	{
		return 1;
	}
	public void setCurNumberOfParameters(int n)
	{
		this.numParameters = n;
	}
	public boolean checkNumberOfParameters(int n)
	{
		return n == 1;
	}
	@SuppressWarnings({"unchecked"})
	public void run(Stack stack)
		throws ParseException
	{
		if (stack == null || stack.empty()) 
		{
			throw new ParseException("Stack is empty");
		}
		Object reporterListId = stack.pop();
		if (reporterListId instanceof Number)
		{
			stack.push(score());
		}
		else
		{
			throw new ParseException("Invalid parameter type: " + reporterListId + "; expected integer");
		}
	}
	// -------------------------------------------

	/**
		Set a new {@link SqlResult} object that will be used the next time the
		JEP expression is evaluated.
		@param result The result object
	*/
	public void setSqlResult(SqlResult result)
	{
		this.result = result;
	}
	/**
		Get the score value from the current sql result.
	 	@return a {@link java.lang.Object} 
	 	@throws ParseException If no result has been specified or if getting the score result fails.
	*/
	public Object score()
		throws ParseException
	{
		if (result == null)
		{
			throw new ParseException("No result object has been specified for function score()");
		}
		try
		{
			return result.getObject(scoreIndex);
		}
		catch (SQLException ex)
		{
			throw new ParseException(ex.getMessage());
		}
	}
}
