/**
	$Id: Scheduler.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.timer;

import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

/**
	This class is a cousin to the standard {@link Timer} class
	but only allows scheduling of tasks. This allows an application to
	create a core timer and let other code add {@link TimerTask}:s to
	the timer.
	<p>
	Another feature of this class is that it can force the execution of
	tasks in their own threads. This avoids that long-runing tasks hog the
	timer thread and therby prevent other tasks from running.
	<p>
	If new threads are created for the tasks, it is also possible to
	specify on a task-by-task basis if the task may run in multiple 
	threads at the same time.
	<p>
	A side-effect by using a scheduler is that all methods may return
	a different <code>TimerTask</code> object than what was scheduled. 
	If the calling application want to cancel a task it must do so on
	the object that is returned, since that object is the one really lives
	in the timer. However, calling {@link TimerTask#cancel()} passes through
	to the original task as well.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see Timer
	@see TimerTask
	@see ThreadTimerTask
*/
public class Scheduler
{
	private final Timer timer;
	private final boolean newThreads;
	/**
		Create a new scheduler object. 
		
		@param timer The underlying timer object which handles the execution of the tasks
		@param newThreads If TRUE, each task is started in it's own thread, if FALSE
			the tasks are executed in the timers thread
	*/
	public Scheduler(Timer timer, boolean newThreads)
	{
		this.timer = timer;
		this.newThreads = newThreads;
	}

	private TimerTask getTimerTask(TimerTask task, boolean allowMultiple)
	{
		return newThreads ? new ThreadTimerTask(task, allowMultiple) : task;
	}
	
	/**
		@see Timer#schedule(TimerTask, Date)
	*/
	public TimerTask schedule(TimerTask task, Date time, boolean allowMultiple)
	{
		task = getTimerTask(task, allowMultiple);
		timer.schedule(task, time);
		return task;
	}
	
	/**
		@see Timer#schedule(TimerTask, Date, long)
	*/
	public TimerTask schedule(TimerTask task, Date firstTime, long period, boolean allowMultiple)
	{
		task = getTimerTask(task, allowMultiple);
		timer.schedule(task, firstTime, period);
		return task;
	}
	
	/**
		@see Timer#schedule(TimerTask, long)
	*/
	public TimerTask schedule(TimerTask task, long delay, boolean allowMultiple)
	{
		task = getTimerTask(task, allowMultiple);
		timer.schedule(task, delay);
		return task;
	}
	
	/**
		@see Timer#schedule(TimerTask, long, long)
	*/
	public TimerTask schedule(TimerTask task, long delay, long period, boolean allowMultiple)
	{
		task = getTimerTask(task, allowMultiple);
		timer.schedule(task, delay, period);
		return task;
	}
	
	/**
		@see Timer#scheduleAtFixedRate(TimerTask, Date, long)
	*/
	public TimerTask scheduleAtFixedRate(TimerTask task, Date firstTime, long period, boolean allowMultiple)
	{
		task = getTimerTask(task, allowMultiple);
		timer.scheduleAtFixedRate(task, firstTime, period);
		return task;
	}
	
	/**
		@see Timer#scheduleAtFixedRate(TimerTask, long, long)
	*/
	public TimerTask scheduleAtFixedRate(TimerTask task, long delay, long period, boolean allowMultiple)
	{
		task = getTimerTask(task, allowMultiple);
		timer.scheduleAtFixedRate(task, delay, period);
		return task;
	}	
	
}
