/*
	$Id: OneTimeFix.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2005 Samuel Andersson, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.install;

import java.sql.Driver;
import java.util.Properties;

import net.sf.basedb.core.Application;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.StringUtil;
import net.sf.basedb.core.User;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;

/**
	This class is used for one-time fixes that are needed to be
	executed manually before or after an updgrade.

	@author Nicklas
	@version 2.7.2
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class OneTimeFix
{
	public static void main(String[] args)
	{
		
		if (args.length == 0) 
		{
			showUsage();
			return;
		}
		try
		{
			String cmd = args[0];		
			if ("lowess_warn".equals(cmd))
			{
				String login = getOption(args, "-u", "root");
				String pwd = getOption(args, "-p", "root");
				boolean flag = getOption(args, "-f", null) != null;
				
				String msg = 
					"This script looks for analysed data that has been normalized with\n" +
					"the LOWESS plug-in. This plug-in has had a serious bug that caused\n" +
					"incorrect normalization values to be calculated. The bug has been\n" +
					"fixed in BASE 2.7.2, but all data that has been normalized with\n" +
					"the LOWESS plug-in before this fix must be redone, including any\n" +
					"analysis steps following the normalization. For more information\n" +
					"about the bug see: http://base.thep.lu.se/ticket/1077\n\n" +
					"This script will find and display all experiments and bioassay sets\n" +
					"that has used the LOWESS normalization plug-in. It is not possible to\n" +
					"detect if the normalization was done before or after the fix was\n" +
					"applied. Use the '-f' option when executing this script to tag all\n" +
					"affected experiments and bioassay sets with 'FIX LOWESS' in their names.\n\n" +
					"Press ENTER to continue or CTRL+C to abort";
				System.out.println(msg);
				waitForEnter();
				showDbInfo(null);
				System.out.println("Starting BASE. Please wait...");
				Application.start(false);
				SessionControl sc = Application.newSessionControl(null, null, null);
				sc.login(login, pwd, null, false);
				showLowessBioassaySets(sc, flag);
				sc.logout();
				sc.close();
				Application.stop();
			}
			else
			{
				showUsage();
			}
		}
		catch (Throwable t)
		{
			System.out.println("\n");
			t.printStackTrace();
		}
	}

	private static void showUsage()
	{
		System.out.println("Usage: onetimefix.sh cmd <options>");
		System.out.println("generic options: ");
		System.out.println("    -u <login>    : Login name for the root user account; (root)");
		System.out.println("    -p <password> : Password for the root user account; (root)");
		System.out.println("cmd=lowess_warn: ");
		System.out.println("    -f            : Tag names of experiments with 'FIX LOWESS'");
	}

	private static void showDbInfo(ProgressReporter progress)
	{
		final Properties properties = System.getProperties();
		final String jdbcDriver = Config.getString("db.driver");
		String driverVersion = "unknown version";
		try
		{
			final Driver driver = (Driver)Class.forName(jdbcDriver).newInstance();
			driverVersion = "version " + driver.getMajorVersion() + "." + driver.getMinorVersion();
		}
		catch (Throwable t)
		{}
		
		String message = 
			"--System information-----------------------------\n" + 
			"BASE     : " + Application.getMajorVersion() + "." +
				Application.getMinorVersion() + "." + 
				Application.getMaintenanceVersion() + "\n" + 
			"Database : " + Config.getString("db.url") + "\n" + 
			"Dialect  : " + Config.getString("db.dialect") + "\n" + 
			"JDBC     : " + jdbcDriver + "; " + driverVersion + "\n" +
			"Java     : " + properties.getProperty("java.runtime.name") + "; " +
				properties.getProperty("java.runtime.version") + "; " +
				properties.getProperty("java.vendor") + "\n" + 
			"OS       : " + properties.getProperty("os.name") +"; " +
				properties.getProperty("os.arch") + "; " +
				properties.getProperty("os.version") +"\n" +
			"-------------------------------------------------";
		if (progress != null)
		{
			progress.display(0, message);
		}
		else 
		{
			System.out.println(message);
		}
	}
	
	public static String getOption(String[] args, String option, String defaultValue)
	{
		String value = defaultValue;
		for (int i = 0; i < args.length; ++i)
		{
			if (option.equals(args[i]))
			{
				if (i+1 < args.length && !args[i+1].startsWith("-"))
				{
					value = args[i+1];
				}
				else
				{
					value = option;
				}
				return value;
			}
		}
		return value;
	}
	
	private static void showLowessBioassaySets(SessionControl sc, boolean flag)
	{
		DbControl dc = sc.newDbControl();
		int totalBas = 0;
		int totalExp = 0;
		try
		{
			ItemQuery<Experiment> expQuery = Experiment.getQuery();
			expQuery.include(Include.ALL);
			expQuery.order(Orders.asc(Hql.property("name")));
			
			for (Experiment exp : expQuery.list(dc))
			{
				User owner = exp.getOwner();
				ItemQuery<BioAssaySet> basQuery = exp.getBioAssaySets();
				basQuery.order(Orders.asc(Hql.property("name")));
				
				boolean hasPrintedExp = false;
				
				for (BioAssaySet bas : basQuery.list(dc))
				{
					Job job = bas.getTransformation().getJob();
					if (job != null)
					{
						PluginDefinition plugin = job.getPluginDefinition();
						if (plugin != null)
						{
							if (plugin.getClassName().equals("net.sf.basedb.plugins.LowessNormalization"))
							{
								if (!hasPrintedExp)
								{
									String msg = "\nExperiment: " + exp.getName() + 
										"; owner: " + owner.getName() + 
										" \n--------------------------------";
									System.out.println(msg);
									hasPrintedExp = true;
									totalExp++;
									if (flag && !exp.getName().startsWith("FIX LOWESS"))
									{
										String newName = "FIX LOWESS: " + exp.getName();
										exp.setName(StringUtil.trimString(newName, Experiment.MAX_NAME_LENGTH));
									}
								}
								System.out.println("  " + bas.getName() + "; id: " + bas.getId());
								if (flag && !bas.getName().startsWith("FIX LOWESS"))
								{
									String newName = "FIX LOWESS: " + bas.getName();
									bas.setName(StringUtil.trimString(newName, BioAssaySet.MAX_NAME_LENGTH));
								}
								totalBas++;
							}
						}
					}
				}
			}
			dc.commit();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		System.out.println("--------------------------");
		if (totalBas == 0)
		{
			System.out.println("Could not find any experiments that has used the LOWESS plug-in.");
		}
		else
		{
			System.out.println("Found " +totalBas + " LOWESS calculations in " + totalExp + " experiments");
		}
	}
	
	public static void waitForEnter()
	{
		try
		{
			System.in.read();
			System.in.skip(System.in.available());
		}
		catch (Exception ex)
		{}
	}

	
}

