/**
	$Id: AnalysisTest.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.test.roles;

import java.util.List;

import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.FloatParameterType;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.IntegerParameterType;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.overview.ExperimentOverview;
import net.sf.basedb.util.overview.Failure;
import net.sf.basedb.util.overview.Severity;
import net.sf.basedb.util.overview.ValidationOptions;
import net.sf.basedb.util.overview.Validator;

/**
	This class contains the tests for the guest created
	by the root user.
	<ol>
	</ol>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class AnalysisTest
{
	public static boolean ok = true;
	
	/**
		Run the tests in this class.
	*/
	public static boolean run(String user)
	{
		DbControl dc = null;
		TestUtil.write("++Entering analysis test for user: " + user + "\n");
		try
		{
			
			dc = TestUtil.getDbControl();
			Experiment experiment = Util.findExperiment(dc, "Experiment A");
			Experiment affyExperiment = Util.findExperiment(dc, "Affymetrix A");
			Project activeProject = Project.getById(dc, TestUtil.getSessionControl().getActiveProjectId());
			
			ValidationOptions options = new ValidationOptions();
			// Expected, since we never specified a biosource for the reference sample
			options.setSeverity(Validator.MISSING_BIOSOURCE, Severity.IGNORE);
			ExperimentOverview overview = getExperimentOverview(experiment, 
				activeProject, options, 0);
			
			// Expected, since the Affymetrix experiment is using a different platform
			options.setSeverity(Validator.NONDEFAULT_PLATFORM, Severity.IGNORE);
			options.setSeverity(Validator.NONDEFAULT_RAWDATATYPE, Severity.IGNORE);
			options.setSeverity(Validator.NONDEFAULT_ARRAYDESIGN, Severity.IGNORE);
			options.setSeverity(Validator.MISSING_HARDWARE, Severity.IGNORE);
			options.setSeverity(Validator.MISSING_PROTOCOL, Severity.IGNORE);
			ExperimentOverview affyOverview = getExperimentOverview(affyExperiment, 
				activeProject, options, 2);
			
			Formula formula = Util.findFormula(dc, RawDataTypes.getSafeRawDataType("genepix"), Formula.Type.INTENSITY_EXPRESSION, "Mean FG - Mean BG");
			dc.close();
			
			BioAssaySet root = createRootBioAssaySet(experiment, 
				"Root bioassay set (" + user + ")", formula);
			BioAssaySet filtered = filterBioAssaySet(experiment, root, 
				"Filtered bioassay set (" + user + ")", "ch(1) > 0 && ch(2) > 0 && rep('id') != null");

			BioAssaySet normalized = normalizeBioAssaySet(experiment, filtered, 
				"Normalized bioassay set (" + user + ")");
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving analysis test for user: " + user + " (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a root bioassay set.
	*/
	public static BioAssaySet createRootBioAssaySet(Experiment experiment, String name, Formula formula)
		throws Exception
	{
		TestUtil.write("--Creating root bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.IntensityCalculatorPlugin");
			experiment = Experiment.getById(dc, experiment.getId());
			Job job = Job.getNew(dc, plugin, null, experiment);
			job.setName("Creating root bioassay set: " + name);
			job.setParameterValue("experiment", 
				new ItemParameterType<Experiment>(Experiment.class, null), experiment);
			job.setParameterValue("name", new StringParameterType(), name);
			job.setParameterValues("rawBioAssays", 
				new ItemParameterType<RawBioAssay>(RawBioAssay.class, null, true, 0, null), 
				experiment.getRawBioAssays().list(dc));
			job.setParameterValue("formula", 
				new ItemParameterType<Formula>(Formula.class, null), formula);
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
			
			dc = TestUtil.getDbControl();
			BioAssaySet root = Util.findBioAssaySet(dc, experiment, name);
			dc.close();
			return root;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	/**
		Create a filtered bioassay set.
	*/
	public static BioAssaySet filterBioAssaySet(Experiment experiment, BioAssaySet source, String name, String expression)
		throws Exception
	{
		TestUtil.write("--Creating filtered bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.FormulaFilter");
			Job job = Job.getNew(dc, plugin, null, experiment);
			job.setName("Creating filtered bioassay set: " + name);
			job.setParameterValue("source", 
				new ItemParameterType<BioAssaySet>(BioAssaySet.class, null), source);
			job.setParameterValue("childName", new StringParameterType(), name);
			job.setParameterValue("expression", new StringParameterType(), expression);
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
			
			dc = TestUtil.getDbControl();
			BioAssaySet root = Util.findBioAssaySet(dc, experiment, name);
			dc.close();
			return root;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	/**
		Create a normalized bioassay set.
	*/
	public static BioAssaySet normalizeBioAssaySet(Experiment experiment, BioAssaySet source, String name)
		throws Exception
	{
		TestUtil.write("--Creating normalized bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.LowessNormalization");
			Job job = Job.getNew(dc, plugin, null, experiment);
			job.setName("Creating normalized bioassay set: " + name);
			job.setParameterValue("source", 
				new ItemParameterType<BioAssaySet>(BioAssaySet.class, null), source);
			job.setParameterValue("childName", new StringParameterType(), name);
			job.setParameterValue("fitFraction", new FloatParameterType(), 0.33f);
			job.setParameterValue("delta", new FloatParameterType(), 0.1f);
			job.setParameterValue("iter", new IntegerParameterType(), 4);
			job.setParameterValue("blockGroup", new IntegerParameterType(), 12);
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
			
			dc = TestUtil.getDbControl();
			BioAssaySet root = Util.findBioAssaySet(dc, experiment, name);
			dc.close();
			return root;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	public static ExperimentOverview getExperimentOverview(Experiment experiment, 
		Project project, ValidationOptions options, int expected)
	{
		TestUtil.write("--Getting overview for experiment: " + experiment.getName() + "\n");
		ExperimentOverview overview = ExperimentOverview.getOverview(experiment, project);
		List<Failure> failures = overview.getFailures();
		int i = 0;
		for (Failure f : failures)
		{
			Severity severity = options.getSeverity(f.getValidator());
			if (severity != Severity.IGNORE)
			{
				i++;
				TestUtil.write("  " + i + ". " + severity + ": " + f.getMessage() + ": " + f.getNode().getItem() + "\n");
			}
		}
		if (i == 0)
		{
			TestUtil.write("  No errors or warnings\n");
		}
		if (i == expected)
		{
			TestUtil.write("--We expected " + expected + " warnings so this is probably ok.\n");
		}
		else
		{
			TestUtil.write("--Unexpected number of warnings: " + i + "; expected " + 
				expected + "\n");
		}
		return overview;
	}
}
