/**
	$Id: ExperimentClient.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.ws.client;

import net.sf.basedb.info.AnnotationTypeInfo;
import net.sf.basedb.info.BioAssaySetInfo;
import net.sf.basedb.info.ExperimentInfo;
import net.sf.basedb.info.QueryOptions;
import net.sf.basedb.info.RawBioAssayInfo;
import net.sf.basedb.info.ReporterListInfo;

import org.apache.axis2.AxisFault;

/**
 	Webservice client for Experiment.
 	Call services in {@link net.sf.basedb.ws.server.ExperimentService}
  	@author Martin
  	@version 2.5
 */
public class ExperimentClient 
	extends AbstractRPCClient 
{	
	private final SessionClient session;
	
	/**
	 	Creates a new client in active session
	  	@param session The active session
	 */
	public ExperimentClient(SessionClient session)
	{
		super(session.getUrl(), "Experiment", session.getServiceFactory());
		this.session = session;
	}
	
	/**
		Get info about an experiment when you know the ID.
		@param itemId The ID of the experiment
		@return An ExperimentInfo object
		@throws AxisFault If communication with web service fails.
	*/
	public ExperimentInfo getById(int itemId)
		throws AxisFault
	{
		return invokeBlocking("getById", ExperimentInfo.class, 
			session.getId(), itemId);
	}
	
	/**
	 	Gets info about experiments that are available in the logged in session.
	  	@param qOpt Options to put on the query that gets the experiments.
	  	@return An array with info object for each experiment that was found. 
	  	@throws AxisFault If communication with web service fails.
	 */
	public ExperimentInfo[] getExperiments(QueryOptions qOpt)
		throws AxisFault
	{
		ExperimentInfo[] result = invokeBlocking("getExperiments", ExperimentInfo[].class, session.getId(), qOpt);
		return result;
	}
	
	/**
		Gets info about raw bioassays used in an experiment.		
	  	@param experimentId Id of experiment to get raw bioassays from.
	  	@param qOpt Options to put on the query that gets the raw bioassays.
	  	@return An array with info object about each raw bioassay that was found.
	  	@throws AxisFault If communication with web services fails.
	 */
	public RawBioAssayInfo[] getRawBioAssays(int experimentId, QueryOptions qOpt)
		throws AxisFault
	{
		RawBioAssayInfo[] result = invokeBlocking("getRawBioAssays", RawBioAssayInfo[].class, session.getId(), experimentId, qOpt);
		return result;
	}
	
	/**
	 	Gets info about bioassay sets in an experiment
	  	@param experimentId Id of the experiment to get the bioassay sets from
	  	@param qOpt Options to put on the query that gets the bioassay sets.
	  	@return An array with info about each bioassay set found by the query.
	  	@throws AxisFault If communication with web services fails.
	 */
	public BioAssaySetInfo[] getBioAssaySets(int experimentId, QueryOptions qOpt)
		throws AxisFault
	{
		BioAssaySetInfo[] result = invokeBlocking("getBioAssaySets", BioAssaySetInfo[].class, session.getId(), experimentId, qOpt);
		return result;
	}


	/**
		Gets info about the experimental factors for an experiment.
	  	@param experimentId Id of experiment to get experimental factors from
	  	@param qOpt Options to put on the query
	  	@return An array with info object about each annotation type that was found
	  	@throws AxisFault If communication with web services fails.
	*/
	public AnnotationTypeInfo[] getExperimentalFactors(int experimentId, QueryOptions qOpt)
		throws AxisFault
	{
		AnnotationTypeInfo[] result = invokeBlocking("getExperimentalFactors", AnnotationTypeInfo[].class, session.getId(), experimentId, qOpt);
		return result;
	}

	
	/**
	 	Gets info about reporter lists in an experiment.
	  	@param experimentId Id of the experiment the lists are used in
	  	@param qOpt Options to put on the query that gets the reporter lists.
	  	@return An array with info about each reporter list.
	  	@throws AxisFault If communication with web services fails.
	 */
	public ReporterListInfo[] getReporterLists(int experimentId, QueryOptions qOpt)
		throws AxisFault
	{
		ReporterListInfo[] result = invokeBlocking("getReporterLists", ReporterListInfo[].class, session.getId(), experimentId, qOpt);
		return result;
	}
}
